/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot;

import java.io.File;
import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.server.JaDepotDBServiceImpl;
import de.frankmuenster.jameica.depot.finanzen.SearchWertpapier;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlSearch;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.depot.gui.navigation.AktienKontoExtension;
import de.frankmuenster.jameica.depot.utils.AnnotationProcessor;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.plugin.Version;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Haupt-Klasse des jameica.JaDepot Plug-ins.
 *
 * @author Frank Mnster
 *
 */
public class JaDepot extends AbstractPlugin {

    /**
     * Standardkonstruktor
     */
    public JaDepot() {
        super();
    }

    /**
     * Methode die beim Starten des Plug-Ins geladen wird
     *
     * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
     */
    @Override
    public void init() throws ApplicationException {
        super.init();

        Logger.info(I18N.tr("JaDepot.startInit")); //$NON-NLS-1$

        // Datenbank Konsistenz berprfen
        this.call(new ServiceCall() {
            /**
             * @param service
             * @throws ApplicationException
             * @throws RemoteException
             */
            @Override
            public void call(final JaDepotDBService service) throws ApplicationException, RemoteException {
                service.checkConsistency();
            }
        });

        // Menu des Jameica-'Plugins' erweitern
        ExtensionRegistry.register(new AktienKontoExtension(), "jameica.start"); //$NON-NLS-1$

        // Process annotations
        AnnotationProcessor.processAnnotation();

        // Quandl warmup
        this.quandlWarmUp();

        Logger.info(I18N.tr("JaDepot.endInit")); //$NON-NLS-1$
    }

    /**
     * This method is called only the first time, the plugin is loaded (before
     * executing init()). if your installation procedure was not successfull,
     * throw an ApplicationException. You dont need to implement this function.
     *
     * @see de.willuhn.jameica.plugin.AbstractPlugin#install()
     */
    @Override
    public void install() throws ApplicationException {

        // If we are running in client/server mode and this instance
        // is the client, we do not need to create a database.
        // Instead of this we will get our objects via RMI from
        // the server
        if (Application.inClientMode()) {
            return;
        }

        try {

            // Let's create an embedded Database
            final PluginResources res = Application.getPluginLoader().getPlugin(JaDepot.class).getResources();
            final Manifest manifest = Application.getPluginLoader().getManifest(JaDepot.class);
            final EmbeddedDatabase database = new EmbeddedDatabase(res.getWorkPath() + "/db", "aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

            // create the sql tables.
            database.executeSQLScript(new File(manifest.getPluginDir() + File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$

            // That's all. Database installed and tables created ;)
        } catch (final Exception e) { // NOPMD
            throw new ApplicationException(I18N.tr("JaDepot.errorInstalling"), e); //$NON-NLS-1$
        }
    }

    /**
     * Here you can do some cleanup stuff. The method will be called on every
     * clean shutdown of jameica. You dont need to implement this function.
     *
     * @see de.willuhn.jameica.plugin.AbstractPlugin#shutDown()
     */
    @Override
    public void shutDown() {
        super.shutDown();
        Logger.trace(I18N.tr("JaDepot.inShutdown")); //$NON-NLS-1$
    }

    /**
     * This method will be executed on every version change. You don't need to
     * implement this function.
     *
     * @see de.willuhn.jameica.plugin.AbstractPlugin#update(de.willuhn.jameica.plugin.Version)
     */
    @Override
    public void update(final Version oldVersion) throws ApplicationException {
        super.update(oldVersion);
        Logger.trace(I18N.tr("JaDepot.inUpdate")); //$NON-NLS-1$
    }

    /**
     * Hilfsmethode zum bequemen Ausfuehren von Methoden auf dem Service.
     *
     * @param call
     *            der Call.
     * @throws ApplicationException
     */
    private void call(final ServiceCall call) throws ApplicationException {
        if (Application.inClientMode()) {
            return; // als Client muessen wir die DB nicht installieren
        }

        JaDepotDBService service = null;
        try {
            // Da die Service-Factory zu diesem Zeitpunkt noch nicht da ist,
            // erzeugen
            // wir uns eine lokale Instanz des Services.
            service = new JaDepotDBServiceImpl();
            service.start();
            call.call(service);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("JaDepot.errorInitDb"), e); //$NON-NLS-1$
            final StringBuilder msg = new StringBuilder(491);
            msg.append(I18N.tr("JaDepot.errorInitDbMsg", //$NON-NLS-1$
                    e.getMessage()));
            throw new ApplicationException(msg.toString(), e);
        } finally {
            if (service != null) {
                try {
                    service.stop(true);
                } catch (final RemoteException e) {
                    Logger.error(I18N.tr("JaDepot.errorCloseDb"), e); //$NON-NLS-1$
                }
            }
        }
    }

    /**
     * Quandl Warm-UP
     */
    private void quandlWarmUp() {

        final Thread warmup = new Thread() {

            /**
             * @see Thread#run()
             */
            @Override
            public void run() {
                final SearchWertpapier<
                        QuandlStockInfoBean> search = QuandlSearch.getIntance(Settings.getQuandlApiKey());
                search.search("DE0005439004"); //$NON-NLS-1$
            }
        };

        warmup.start();

    }

    /**
     * Hilfsmethode zum bequemen Ausfuehren von Aufrufen auf dem Service.
     */
    private interface ServiceCall {
        /**
         * @param service
         * @throws ApplicationException
         * @throws RemoteException
         */
        void call(JaDepotDBService service) throws ApplicationException, RemoteException;
    }
}
