/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot;

import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.system.Application;

/**
 * Settings Klasse fr den jameica.qrscanner Basiert auf den Settings-Klasse aus
 * dem Beispiel-Plugin von Olaf Willuhn.
 *
 * @author Frank Mnster
 * @see <a href=
 *      "http://www.willuhn.de/products/jameica/dev.php">http://www.willuhn.de/products/jameica/dev.php</a>
 */
public final class Settings {

    /**
     * Our currency name.
     */
    public static final String CURRENCY = "EUR"; //$NON-NLS-1$

    /**
     * Our DateFormatter.
     */
    public static final DateFormat DATEFORMAT = DateFormat.getDateInstance(DateFormat.DEFAULT,
            Application.getConfig().getLocale());

    /**
     * Our DateFormatter.
     */
    public static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofLocalizedDate(FormatStyle.MEDIUM);

    /**
     * Our decimal formatter.
     */
    public static final DecimalFormat DECIMALFORMAT = (DecimalFormat) DecimalFormat
            .getInstance(Application.getConfig().getLocale());

    /**
     * Unser Datenbank-Service
     */
    private static DBService dbService;

    /***
     * Ist Hibiscus auch installiert?
     */
    private static Boolean hibiscusInstalled = Application.getPluginLoader()
            .isInstalled("de.willuhn.jameica.hbci.HBCI"); //$NON-NLS-1$

    /**
     * Settings Objekt von Jameica
     */
    private static de.willuhn.jameica.system.Settings jameicaSettings = Application.getPluginLoader()
            .getPlugin(JaDepot.class) // NOPMD
            .getResources().getSettings();

    /**
     * Small helper function to get the database service.
     *
     * @return db service.
     * @throws RemoteException
     */
    public static DBService getDBService() throws RemoteException {

        if (Settings.dbService != null) {
            return Settings.dbService; // NOPMD by munstfr on 14.09.16 14:38
        }
        try {
            // We have to ask Jameica's ServiceFactory.
            // If we are running in Client/Server mode and we are the
            // client, the factory returns the remote dbService from the
            // Jameica server.
            // The name and class of the service is defined in plugin.xml
            Settings.dbService = (DBService) Application.getServiceFactory().lookup(JaDepot.class, "jaDepotDB"); //$NON-NLS-1$
            return Settings.dbService;
        } catch (final Exception e) { // NOPMD
            throw new RemoteException(I18N.tr("Settings.errorDbService"), e); //$NON-NLS-1$
        }
    }

    /**
     * Liefert die Standardw&auml;hrung.
     *
     * @return Standard-W&auml;hrung, default ist <code>EUR</code>.
     */
    public static String getDepotWaehrung() {

        return Settings.CURRENCY;
    }

    /**
     * Liefert den gespeicherten Quandl-API Key oder einen leeren String.
     *
     * @return den gespeicherten Quandl-API Key oder einen leeren String
     */
    public static String getQuandlApiKey() {

        return Settings.jameicaSettings.getString("JaDepot.quandlApiKey", StringUtils.EMPTY); //$NON-NLS-1$
    }

    /**
     * berprft, ob das Hibiscus -Plugin installiert ist.
     *
     * @return isHibiscusInstalled <b>true</b>, wenn Hibiscus installiert ist,
     *         <b>false</b> sonst
     */
    public static Boolean isHibiscusInstalled() {

        return Settings.hibiscusInstalled;
    }

    /**
     * Ist Quandl Premium-Account
     *
     * @return <i>true</i> falls es sich um einen Premium Quandl-Account handelt
     */
    public static boolean isQuandlPremium() {

        return Settings.jameicaSettings.getBoolean("JaDepot.quandlPremium", false); //$NON-NLS-1$
    }

    /**
     * Liefert die showHibiscusSummery.
     *
     * @return showHibiscusSummery, ist immer <code>false</code>, falls Hibiscus
     *         gar nicht installiert ist.
     */
    public static boolean isShowHibiscusSummery() {

        boolean isShowHibiscus = false;
        if (Settings.isHibiscusInstalled()) {
            isShowHibiscus = Settings.jameicaSettings.getBoolean("JaDepot.showHibiscusSummery", true); //$NON-NLS-1$
        }
        return isShowHibiscus;
    }

    /**
     * Setzt den Quandl-Api Key
     *
     * @param quandlApiKey
     */
    public static void setQuandlApiKey(final String quandlApiKey) {

        Settings.jameicaSettings.setAttribute("JaDepot.quandlApiKey", //$NON-NLS-1$
                quandlApiKey);
    }

    /**
     * Setzt den Quandl Premium-Account
     *
     * @param quandlPremium
     */
    public static void setQuandlPremium(final Boolean quandlPremium) {

        Settings.jameicaSettings.setAttribute("JaDepot.quandlPremium", //$NON-NLS-1$
                quandlPremium);
    }

    /**
     * Setzt showHibiscusSummery.
     *
     * @param showHibiscus
     *            soll Hibsicus Kontenzusammenfassung angezeigt werden.
     */
    public static void setShowHibiscusSummery(final Boolean showHibiscus) {

        Settings.jameicaSettings.setAttribute("JaDepot.showHibiscusSummery", //$NON-NLS-1$
                showHibiscus);
    }

    private Settings() {
    }
}
