/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.rmi;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.Date;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface of the business object for history. According to the SQL table, we
 * define some getter an setter here.
 *
 * <pre>
 * CREATE TABLE history (
 *   id NUMERIC default UNIQUEKEY('history'),
 *   aktien_id int(4) NOT NULL,
 *   oderDate date,
 *   orderAmount float,
 *   orderPrice float,
 *   orderPriceInMainCurrency float,
 *   additionalCosts float,
 *   additionalCostsInMainCurrency float,
 *   additionalCostsCurrency varchar(5),
 *   orderNumber varchar(20);
 *   comment text,
 *   UNIQUE (id),
 *   PRIMARY KEY (id)
 * );
 * </pre>
 *
 * <br>
 * Getters and setters for the primary key (id) are not needed. Every one of the
 * following methods has to throw a RemoteException. <br>
 */
public interface OrderHistory extends DBObject {

    /**
     * Liefert zustzliche Kosten der Order.
     *
     * @return additionalCosts der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */

    BigDecimal getAdditionalCosts() throws RemoteException;

    /**
     * Liefert die W&auml;hrung der zustzliche Kosten der Order.
     *
     * @return currency der zustzliche Kosten der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getAdditionalCostsCurrency() throws RemoteException;

    /**
     * Liefert zustzliche Kosten der Order in der Hauptw&auml;hrung.
     *
     * @return additionalCosts der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getAdditionalCostsInMainCurrency() throws RemoteException;

    /**
     * Liefert das Wertpapier zu der Order.
     *
     * @return das Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    Wertpapier getAktie() throws RemoteException;

    /**
     * Liefert die Beschreibung der Order.
     *
     * @return die Beschreibung der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getComment() throws RemoteException;

    /**
     * Liefert die Anzahl der Aktien der Order.
     *
     * @return orderAmount der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getOrderAmount() throws RemoteException;

    /**
     * Liefert das OrderDate der Order
     *
     * @return orderDate der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */

    Date getOrderDate() throws RemoteException;

    /**
     * Get the order number.
     *
     * @return the order number
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getOrderNumber() throws RemoteException;

    /**
     * Liefert den Aktieneinzelpreis der Order.
     *
     * @return orderPrice der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getOrderPrice() throws RemoteException;

    /**
     * Liefert den Aktieneinzelpreis der Order in der Hauptw&auml;hrung.
     *
     * @return orderPrice der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getOrderPriceInMainCurrency() throws RemoteException;

    /**
     * Returns the total order costs without additional costs
     *
     * @return the total order costs without additional costs
     * @throws RemoteException
     */
    BigDecimal getTotalOrderCosts() throws RemoteException;

    /**
     * Returns the total order cost without additional costs in the main
     * currency
     *
     * @return the total order cost without additional costs in the main
     *         currency
     * @throws RemoteException
     */
    BigDecimal getTotalOrderCostsInMainCurrency() throws RemoteException;

    /**
     * Setzt die zustzlichen Kosten der Order.
     *
     * @param additionalCosts
     *            die zustzlichen Kosten der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setAdditionalCosts(BigDecimal additionalCosts) throws RemoteException;

    /**
     * Setzt die W&auml;hrung der zustzliche Kosten der Order.
     *
     * @param currency
     *            der zustzliche Kosten der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setAdditionalCostsCurrency(String currency) throws RemoteException;

    /**
     * Setzt die zustzlichen Kosten der Order in der Hauptw&auml;hrung.
     *
     * @param additionalCosts
     *            die zustzlichen Kosten der Order in der Hauptw&auml;hrung.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setAdditionalCostsInMainCurrency(BigDecimal additionalCosts) throws RemoteException;

    /**
     * Setzt das Wertpapier der Order.
     *
     * @param wertpapier
     *            das Wertpapier
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setAktie(Wertpapier wertpapier) throws RemoteException;

    /**
     * Setzt die Beschreibung der Order.
     *
     * @param comment
     *            die Beschreibung der Order.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setComment(String comment) throws RemoteException;

    /**
     * Setzt die Aktienanzahl der OrderImpl.
     *
     * @param orderAmount
     *            die Aktienanzahl der OrderImpl. Negative Anzahl heit Verkauf.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setOrderAmount(BigDecimal orderAmount) throws RemoteException;

    /**
     * Setzt das Datum der OrderImpl.
     *
     * @param orderDate
     *            der OrderImpl.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setOrderDate(Date orderDate) throws RemoteException;

    /**
     * Set the order number
     *
     * @param ordnerNumber
     *            the order number to set
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setOrderNumber(String ordnerNumber) throws RemoteException;

    /**
     * Setzt den Preis pro Wertpapier der OrderImpl.
     *
     * @param orderPrice
     *            den Preis pro Wertpapier der OrderImpl.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setOrderPrice(BigDecimal orderPrice) throws RemoteException;

    /**
     * Setzt den Preis pro Wertpapier der Order in der Hauptw&auml;hrung.
     *
     * @param orderPrice
     *            den Preis pro Wertpapier der Order in der Hauptw&auml;hrung.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setOrderPriceInMainCurrency(BigDecimal orderPrice) throws RemoteException;

}
