/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.rmi;

import java.math.BigDecimal;
/**
 * Interface of the business object for wertpapiere.
 * According to the SQL table, we define some getter an setter here.
 *
 * <pre>
 * CREATE TABLE aktien (
 *   id NUMERIC default UNIQUEKEY('wertpapier'),
 *   name varchar(255) NOT NULL,
 *   symbol varchar(20) NOT NULL,
 *   ffmName varchar(100),
 *   ffmInstrument varchar(20),
 *   quandlCode varchar(20),
 *   lastTradeDate date,
 *   lastTradeTime time,
 *   lastTradePriceOnly float,
 *   previousClose float,
 *   currency varchar(5),
 *   value float,
 *   yearLow float,
 *   yearHigh float,
 *   change float,
 *   changeInPercentage float,
 *   type varchar(20).
 *   exchange varchar(50),
 *   comment text,
 *   UNIQUE (id),
 *   PRIMARY KEY (id)
 * );
 * </pre>
 *
 * <br>
 * Getters and setters for the primary key (id) are not needed.
 * Every one of the following methods has to throw a RemoteException. <br>
 */
import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBObject;

/**
 * @author Frank Mnster
 */
public interface Wertpapier extends DBObject {

    /**
     * Returns the actual total sum of the stock. So ammount multiplied by
     * actuacl price.
     *
     * @return the actual total sum of the stock
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getActualTotalValue() throws RemoteException;

    /**
     * Liefert die gesamten zus&auml;tzlichen Kosten des Wertpapiers.
     *
     * @return die zus&auml;tzlichen Kosten.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getAdditionalCosts() throws RemoteException;

    /**
     * Liefert die Anzahl an Aktien im Depot, berechnet aus der OrderHistory
     *
     * @return die Anzahl an Aktien im Depot
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getAnzahl() throws RemoteException;

    /**
     * Liefert die ARD Brese ID des Wertpapieres.
     *
     * @return die ARD Brese ID des Wertpapieres.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getArdId() throws RemoteException;

    /**
     * Liefert den change Wert des Wertpapier.
     *
     * @return change der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getChange() throws RemoteException;

    /**
     * Liefert den changeInPercentage Wert des Wertpapier.
     *
     * @return changeInPercentage der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    double getChangeInPercentage() throws RemoteException;

    /**
     * Liefert die Beschreibung der Aktien.
     *
     * @return die Beschreibung der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getComment() throws RemoteException;

    /**
     * Liefert die W&auml;hrung des Wertpapiers.
     *
     * @return currency des Wertpapiers.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getCurrency() throws RemoteException;

    /**
     * Liefert die B&ouml;rse Wert des Wertpapiers.
     *
     * @return exchange des Wertpapiers.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getExchange() throws RemoteException;

    /**
     * Liefert den Instrument Namen bei der FFM
     *
     * @return den Instrument Namen bei der FFM
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getFfmInstrument() throws RemoteException;

    /**
     * Liefert den Namen bei der FFM
     *
     * @return den Namen bei der FFM
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getFfmName() throws RemoteException;

    /**
     * Liefert den Wert aller Papiere, die noch im Bestand sind.
     *
     * @return den Gesamtwert (Anzahl * Kurs).
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getGesamtWert() throws RemoteException;

    /**
     * Returns the ISIN of the security
     *
     * @return the ISIN of the security
     *
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getIsin() throws RemoteException;

    /**
     * Liefert die gesamten Ausgaben zum Erwerb der Wertpapier.
     *
     * @return die gesamten Ausgaben zum Erwerb der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getKauf() throws RemoteException;

    /**
     * Liefert den lastTradeDate der Wertpapier.
     *
     * @return lastTradeDate der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    Date getLastTradeDate() throws RemoteException;

    /**
     * Liefert den lastTradePriceOnly der Wertpapier.
     *
     * @return lastTradePriceOnly der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getLastTradePriceOnly() throws RemoteException;

    /**
     * Liefert den lastTradeTime der Wertpapier.
     *
     * @return lastTradeTime der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    Time getLastTradeTime() throws RemoteException;

    /**
     * Liefert den Name der Wertpapier.
     *
     * @return name Der Name der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getName() throws RemoteException;

    /**
     * Liefert eine Liste mit alle Orders dieser Wertpapier
     *
     * @return Liste mit OrderImpl der Wertpapier
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    DBIterator<OrderHistory> getOrderHistory() throws RemoteException;

    /**
     * Liefert den previousClose der Wertpapier.
     *
     * @return previousClose der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getPreviousClose() throws RemoteException;

    /**
     * Liefert den Quandl Code
     *
     * @return den Quandl Code
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getQuandlCode() throws RemoteException;

    /**
     * Liefert das Gesamtergebnis der Papieres ber alle Kauf- und
     * Verkaufsorder.
     *
     * @return das Gesamtergebnis des Papieres
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getTotalGewinn() throws RemoteException;

    /**
     * Liefert den Type Wert des Wertpapier.
     *
     * @return type der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    String getType() throws RemoteException;

    /**
     * Zustzlich noch ein eMethode, um all historyValues dieser Wertpapier zu
     * erhalten.
     *
     * @return Liste aller historyValues dieser Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    DBIterator<ValueHistory> getValueHistory() throws RemoteException;

    /**
     * Liefert der realisierten Gewinn der Verkaufsorder.
     *
     * @return den realisierten Gewinn
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getVerkauf() throws RemoteException;

    /**
     * Liefert den yearHigh Wert des Wertpapier.
     *
     * @return yearHigh der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getYearHigh() throws RemoteException;

    /**
     * Liefert den yearLow Wert des Wertpapier.
     *
     * @return yearLow der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    BigDecimal getYearLow() throws RemoteException;

    /**
     * Liefert die Angabe, ob das Wertpapier in der ARD Brese gelistet ist oder
     * nicht. Muss bei jeden erstellen neu berprft und gesetzt werden.
     *
     * @return true falls der Wertpapier in der ARD Brese gelistet ist, false
     *         sonst
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    boolean isInArd() throws RemoteException;

    /**
     * Aktualisiert das Wertpapier mit den Angaben aus einen
     * {@link de.frankmuenster.jameica.finanzen.StockInfoBean} Objekts.
     *
     * @param wpi
     *            das WertpapierInfoBean Objekt.
     * @throws RemoteException
     */
    void parseWertpapier(StockInfoBean wpi) throws RemoteException;

    /**
     * Setzt die Ard ID des Wertpapieres.
     *
     * @param ardId
     *            die Ard ID des Wertpapieres.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setArdId(String ardId) throws RemoteException;

    /**
     * Setzt den change Wert der Wertpapiers.
     *
     * @param change
     *            den change Wert der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setChange(BigDecimal change) throws RemoteException;

    /**
     * Setzt den changeInPercentage Wert der Wertpapiers.
     *
     * @param changeInPercentage
     *            den changeInPercentage Wert der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setChangeInPercentage(double changeInPercentage) // NOPMD
            throws RemoteException;

    /**
     * Setzt die Beschreibung der Aktien.
     *
     * @param comment
     *            die Beschreibung der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setComment(String comment) throws RemoteException;

    /**
     * Setzt die W&auml;hrung des Wertpapiers.
     *
     * @param currency
     *            die W&auml;hrung des Wertpapiers.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setCurrency(String currency) throws RemoteException;

    /**
     * Setzt die B&omul; des Wertpapiers.
     *
     * @param exchange
     *            die B&omul; des Wertpapiers.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setExchange(String exchange) throws RemoteException;

    /**
     * Setzt den Instrument Namen bei der FFM
     *
     * @param ffmInstrument
     *            der Instrument Namen bei der FFM
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setFfmInstrument(String ffmInstrument) throws RemoteException;

    /**
     * Setzt den Namen bei der FFM
     *
     * @param ffmName
     *            der Namen bei der FFM
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setFfmName(String ffmName) throws RemoteException;

    /**
     * Resets information if stock exists in ARD Brse
     *
     * @throws RemoteException
     */
    void setInArd() throws RemoteException;

    /**
     * Sets the ISIN of the security
     *
     * @param isin
     *            of the security
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setIsin(String isin) throws RemoteException;

    /**
     * Setzt den lastTradeDate der Wertpapier.
     *
     * @param lastTradeDate
     *            der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setLastTradeDate(Date lastTradeDate) throws RemoteException;

    /**
     * Setzt den lastTradePriceOnly der Wertpapier.
     *
     * @param lastTradePriceOnly
     *            der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setLastTradePriceOnly(BigDecimal lastTradePriceOnly) throws RemoteException;

    /**
     * Setzt den lastTradeTime der Wertpapier.
     *
     * @param lastTradeTime
     *            der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setLastTradeTime(Time lastTradeTime) throws RemoteException;

    /**
     * Setzt den Name der Wertpapier.
     *
     * @param name
     *            Name der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setName(String name) throws RemoteException;

    /**
     * Setzt den previousClose der Wertpapier.
     *
     * @param previousClose
     *            der Wertpapier.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setPreviousClose(BigDecimal previousClose) throws RemoteException;

    /**
     * Setzt den Quandl Code
     *
     * @param quandlCode
     *            der Quandl Code
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setQuandlCode(String quandlCode) throws RemoteException;

    /**
     * Setzt den Type Wert der Wertpapiers.
     *
     * @param type
     *            den Type Wert der Wertpapiers.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setType(String type) throws RemoteException;

    /**
     * Setzt den yearHigh Wert der Wertpapiers.
     *
     * @param yearHigh
     *            den yearHigh Wert der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setYearHigh(BigDecimal yearHigh) throws RemoteException;

    /**
     * Setzt den yearLow Wert der Wertpapiers.
     *
     * @param yearLow
     *            den yearLow Wert der Aktien.
     * @throws RemoteException
     *             see {@link java.rmi.RemoteException}
     */
    void setYearLow(BigDecimal yearLow) throws RemoteException;
}
