/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;
import java.util.Date;

import de.frankmuenster.jameica.depot.db.rmi.ValueHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementation of the HistoryValue table database access.
 *
 * @author Frank Mnster
 */
public class HistoryValueImpl extends AbstractDBObject implements ValueHistory {

    /**
     * Name for the stock_id column
     */
    private static final String STOCK_ID = "aktie_id"; //$NON-NLS-1$

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 4737675122756337592L;

    /**
     * @throws RemoteException
     *             - see {@link java.rmi.RemoteException}
     */
    public HistoryValueImpl() throws RemoteException {
	super();
    }

    /**
     * @see ValueHistory#getAktie()
     */
    @Override
    public Wertpapier getAktie() throws RemoteException {

	Wertpapier wertpapier = null;
	// Yes, we can cast this directly to Wertpapier, because
	// getForeignObject(String)
	// contains the mapping for this attribute.
	try {
	    wertpapier = (Wertpapier) getAttribute(HistoryValueImpl.STOCK_ID);
	} catch (final ObjectNotFoundException e) {
	    Logger.info(I18N.tr("HistoryValueImpl.stockNotFound") //$NON-NLS-1$
		    + e.getMessage());
	}
	return wertpapier;
    }

    /**
     * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {
	// our primary attribute is the name.
	return "id"; //$NON-NLS-1$
    }

    /**
     * @see ValueHistory#getTradingDate()
     */
    @Override
    public Date getTradingDate() throws RemoteException {
	return (Date) getAttribute("tradingDate"); //$NON-NLS-1$
    }

    /**
     * @see ValueHistory#getTradingPrice()
     */
    @Override
    public double getTradingPrice() throws RemoteException {
	final Double tradingPrice = (Double) getAttribute("tradingPrice"); //$NON-NLS-1$
	return tradingPrice == null || Double.isNaN(tradingPrice) ? 0.0 : tradingPrice.doubleValue();
    }

    /**
     * @see ValueHistory#setAktie(Wertpapier)
     */
    @Override
    public void setAktie(final Wertpapier wertpapier) throws RemoteException {
	// casting wird in foreignObject bernommen
	setAttribute(HistoryValueImpl.STOCK_ID, wertpapier);
    }

    /**
     * @see ValueHistory#setTradingDate(Date)
     */
    @Override
    public void setTradingDate(final Date tradingDate) throws RemoteException {
	setAttribute("tradingDate", tradingDate); //$NON-NLS-1$

    }

    /**
     * @see ValueHistory#setTradingPrice(double)
     */
    @Override
    public void setTradingPrice(final double tradingPrice) throws RemoteException {
	setAttribute("tradingValue", Double.valueOf(tradingPrice)); //$NON-NLS-1$

    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
     */
    @Override
    protected Class<Wertpapier> getForeignObject(final String field) throws RemoteException {
	Class<Wertpapier> clas = null;
	// the system is able to resolve foreign keys and loads
	// the according objects automatically. You only have to
	// define which class handles which foreign key.
	if (HistoryValueImpl.STOCK_ID.equals(field)) {
	    clas = Wertpapier.class;
	}
	return clas;
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {
	// this is the sql table name.
	return "valueHistory"; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {
	try {
	    if (getTradingDate() == null) {
		throw new ApplicationException(I18N.tr("HistoryValueImpl.tradingDateIsNull")); //$NON-NLS-1$
	    }

	    if (getAktie() == null) {
		throw new ApplicationException(I18N.tr("HistoryValueImpl.stockIsNull")); //$NON-NLS-1$
	    }

	    if (getAktie().isNewObject()) {
		throw new ApplicationException(I18N.tr("HistoryValueImpl.firstStoreStock")); //$NON-NLS-1$
	    }

	} catch (final RemoteException e) {
	    Logger.error(I18N.tr("HistoryValueImpl.insertCheckFailed"), e); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(I18N.tr("HistoryValueImpl.saveFailed")) //$NON-NLS-1$
		    .initCause(e);
	}
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {
	// same as insertCheck
	insertCheck();
    }

}
