/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Implementiert den UpdateProvdider der Jamaica SQL-Engine
 *
 * @author Frank Mnster
 */
public class JaDepotUpdateProvider implements UpdateProvider {

    /**
     * Die Version des Updates
     */
    private final transient Version version;

    /**
     * Das Datenbank-Connection Objekt
     */
    private final transient Connection conn;

    /**
     * Plugin Manifest
     */
    private final transient Manifest manifest;

    /**
     * Das Plugin-Resource-Ojekt
     */
    private final transient PluginResources res;

    /**
     * Geschtzer Konstruktor der Klasse
     * 
     * @param conn
     *            das DB-Verbindungs-Objekt
     * @param version
     *            die Versionsinformationen
     */
    protected JaDepotUpdateProvider(final Connection conn, final Version version) {
	this.conn = conn;
	this.version = version;

	final AbstractPlugin abstractPlugin = Application.getPluginLoader().getPlugin(JaDepot.class);
	this.manifest = abstractPlugin.getManifest();
	this.res = abstractPlugin.getResources();
    }

    /**
     * @see UpdateProvider#getConnection()
     */
    @Override
    public Connection getConnection() throws ApplicationException {
	return this.conn;
    }

    /**
     * @see UpdateProvider#getCurrentVersion()
     */
    @Override
    public int getCurrentVersion() throws ApplicationException {
	try {
	    return this.version.getVersion();
	} catch (final RemoteException re) {
	    Logger.error(I18N.tr("JaDepotUpdateProvider.readVersionError")); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(I18N.tr("JaDepotUpdateProvider.readVersionError")) //$NON-NLS-1$
		    .initCause(re);
	}
    }

    /**
     * @see UpdateProvider#getProgressMonitor()
     */
    @Override
    public ProgressMonitor getProgressMonitor() {
	// Liefert den Splashscreen oder im Servermode einen
	// Pseudo-Monitor.
	return Application.getController().getApplicationCallback().getStartupMonitor();
    }

    /**
     * Liefert das PluginResource Objekt.
     * 
     * @return das PluginResource Objekt.
     */
    public PluginResources getResources() {
	return this.res;
    }

    /**
     * @see UpdateProvider#getUpdatePath()
     */
    @Override
    public File getUpdatePath() throws ApplicationException {
	// Ist das Unterverzeichnis "plugins" im Plugin
	return new File(this.manifest.getPluginDir(), "updates"); //$NON-NLS-1$
    }

    /**
     * @see UpdateProvider#setNewVersion(int)
     */
    @Override
    public void setNewVersion(final int newVersion) throws ApplicationException {
	final int current = getCurrentVersion();
	try {
	    if (Logger.isLogging(Level.INFO)) {
		Logger.info(I18N.tr("JaDepotUpdateProvider.applyNewVerison", //$NON-NLS-1$
			this.version.getName(), Integer.toString(newVersion)));
	    }
	    this.version.setVersion(newVersion);
	    this.version.store();
	} catch (final RemoteException e) {
	    // Im Fehlerfall Versionsnummer zuruecksetzen
	    resetVersion(current);
	    Logger.error(I18N.tr("JaDepotUpdateProvider.unableToReadCurrentVersion"), e); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(
		    I18N.tr("JaDepotUpdateProvider.unableToReadCurrentVersion")).initCause(e); //$NON-NLS-1$

	} catch (final ApplicationException e) {
	    // Im Fehlerfall Versionsnummer zuruecksetzen
	    resetVersion(current);
	    // Originalfehler zurueckgegeben
	    throw e;

	}

    }

    /**
     * Setzt Version und verwirft Fehlermeldung
     * 
     * @param version
     *            die zu setzte Version.
     */
    private void resetVersion(final int version) {
	try {
	    this.version.setVersion(version);
	} catch (final RemoteException e2) {
	    Logger.error(I18N.tr("JaDepotUpdateProvider.unableToRollback"), e2); //$NON-NLS-1$
	    // Werfen wir nicht, weil es sonst die eigentliche Exception
	    // verdecken wuerde
	}
    }

}
