/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.util.Date;

import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementation of the OrderHistory table database access.
 *
 * @author Frank Mnster
 */
public class OrderHistoryImpl extends AbstractDBObject implements OrderHistory {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 8548057261191049423L;

    /**
     * Name for the stock_id column
     */
    private static final String STOCK_ID = "aktie_id"; //$NON-NLS-1$

    /**
     * @throws RemoteException
     */
    public OrderHistoryImpl() throws RemoteException {

        super();
    }

    /**
     * @see OrderHistory#getAdditionalCosts()
     */
    @Override
    public BigDecimal getAdditionalCosts() throws RemoteException {

        final Double additionalCosts = (Double) this.getAttribute("additionalCosts"); //$NON-NLS-1$
        return additionalCosts == null || Double.isNaN(additionalCosts) ? null
                : BigDecimal.valueOf(additionalCosts).setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getAdditionalCostsCurrency()
     */
    @Override
    public String getAdditionalCostsCurrency() throws RemoteException {

        return (String) this.getAttribute("additionalCostsCurrency"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getAdditionalCostsInMainCurrency()
     */
    @Override
    public BigDecimal getAdditionalCostsInMainCurrency() throws RemoteException {

        final Double acInMainCurrency = (Double) this.getAttribute("additionalCostsInMainCurrency"); //$NON-NLS-1$
        return acInMainCurrency == null || Double.isNaN(acInMainCurrency) ? null
                : BigDecimal.valueOf(acInMainCurrency).setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getAktie()
     */
    @Override
    public Wertpapier getAktie() throws RemoteException {

        // Yes, we can cast this directly to Wertpapier, because
        // getForeignObject(String)
        // contains the mapping for this attribute.
        try {
            return (Wertpapier) this.getAttribute(OrderHistoryImpl.STOCK_ID); // NOPMD
        } catch (final ObjectNotFoundException e) {
            return null;
        }
    }

    /**
     * @see OrderHistory#getComment()
     */

    @Override
    public String getComment() throws RemoteException {

        return (String) this.getAttribute("comment"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getOrderAmount()
     */
    @Override
    public BigDecimal getOrderAmount() throws RemoteException {

        final Double orderAmount = (Double) this.getAttribute("orderAmount"); //$NON-NLS-1$
        return orderAmount == null || Double.isNaN(orderAmount) ? null
                : BigDecimal.valueOf(orderAmount).setScale(6, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getOrderDate()
     */
    @Override
    public Date getOrderDate() throws RemoteException {

        return (Date) this.getAttribute("orderDate"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getOrderNumber()
     */
    @Override
    public String getOrderNumber() throws RemoteException {

        return (String) this.getAttribute("orderNumber"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getOrderPrice()
     */
    @Override
    public BigDecimal getOrderPrice() throws RemoteException {

        final Double ordnerprice = (Double) this.getAttribute("orderPrice"); //$NON-NLS-1$
        return ordnerprice == null || Double.isNaN(ordnerprice) ? null
                : BigDecimal.valueOf(ordnerprice).setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getOrderPriceInMainCurrency()
     */
    @Override
    public BigDecimal getOrderPriceInMainCurrency() throws RemoteException {

        final Double opInMainCurrency = (Double) this.getAttribute("orderPriceInMainCurrency"); //$NON-NLS-1$
        return opInMainCurrency == null || Double.isNaN(opInMainCurrency) ? null
                : BigDecimal.valueOf(opInMainCurrency).setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {

        return "id"; //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getTotalOrderCosts()
     */
    @Override
    public BigDecimal getTotalOrderCosts() throws RemoteException {

        BigDecimal orderCosts = null;

        if (this.getOrderAmount() != null && this.getOrderPrice() != null) {
            orderCosts = this.getOrderAmount().multiply(this.getOrderPrice()).setScale(2, RoundingMode.HALF_EVEN);
        }
        return orderCosts == null ? null : orderCosts.setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see OrderHistory#getTotalOrderCostsInMainCurrency()
     */
    @Override
    public BigDecimal getTotalOrderCostsInMainCurrency() throws RemoteException {

        BigDecimal orderCosts = null;

        if (this.getOrderAmount() != null && this.getOrderPriceInMainCurrency() != null) {
            orderCosts = this.getOrderAmount().multiply(this.getOrderPriceInMainCurrency()).setScale(2,
                    RoundingMode.HALF_EVEN);
        }
        return orderCosts;
    }

    /**
     * @see OrderHistory#setAdditionalCosts(BigDecimal)
     */
    @Override
    public void setAdditionalCosts(final BigDecimal additionalCosts) throws RemoteException {

        this.setAttribute("additionalCosts", additionalCosts == null ? null : additionalCosts.doubleValue()); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setAdditionalCostsCurrency(String)
     */
    @Override
    public void setAdditionalCostsCurrency(final String additionalCosts) throws RemoteException {

        this.setAttribute("additionalCostsCurrency", additionalCosts); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setAdditionalCostsInMainCurrency(BigDecimal)
     */
    @Override
    public void setAdditionalCostsInMainCurrency(final BigDecimal additionalCosts) throws RemoteException {

        this.setAttribute("additionalCostsInMainCurrency", //$NON-NLS-1$
                additionalCosts == null ? null : additionalCosts.doubleValue());

    }

    /**
     * @see OrderHistory#setAktie(Wertpapier)
     */
    @Override
    public void setAktie(final Wertpapier wertpapier) throws RemoteException {

        // casting wird in foreignObject bernommen
        this.setAttribute(OrderHistoryImpl.STOCK_ID, wertpapier);
    }

    /**
     * @see OrderHistory#setComment(String)
     */

    @Override
    public void setComment(final String comment) throws RemoteException {

        this.setAttribute("comment", comment); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#setOrderAmount(BigDecimal)
     */
    @Override
    public void setOrderAmount(final BigDecimal orderAmount) throws RemoteException {

        this.setAttribute("orderAmount", orderAmount == null ? null : orderAmount.doubleValue()); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderDate(Date)
     */
    @Override
    public void setOrderDate(final Date orderDate) throws RemoteException {

        this.setAttribute("orderDate", orderDate); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderNumber(String)
     */
    @Override
    public void setOrderNumber(final String ordnerNumber) throws RemoteException {

        this.setAttribute("orderNumber", ordnerNumber); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderPrice(BigDecimal)
     */
    @Override
    public void setOrderPrice(final BigDecimal orderPrice) throws RemoteException {

        this.setAttribute("orderPrice", orderPrice == null ? null : orderPrice.doubleValue()); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderPriceInMainCurrency(BigDecimal)
     */
    @Override
    public void setOrderPriceInMainCurrency(final BigDecimal orderPrice) throws RemoteException {

        this.setAttribute("orderPriceInMainCurrency", orderPrice == null ? null : orderPrice.doubleValue()); //$NON-NLS-1$

    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
     */
    @Override
    protected Class<Wertpapier> getForeignObject(final String field) throws RemoteException {

        // the system is able to resolve foreign keys and loads
        // the according objects automatically. You only have to
        // define which class handles which foreign key.
        if (OrderHistoryImpl.STOCK_ID.equals(field)) {
            return Wertpapier.class; // NOPMD
        }
        return null;
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName
     */
    @Override
    protected String getTableName() {

        return "orderHistory"; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    @SuppressWarnings({ "PMD.ModifiedCyclomaticComplexity", "PMD.StdCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    protected void insertCheck() throws ApplicationException {

        try {
            if (this.getOrderDate() == null) {
                throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterDate")); //$NON-NLS-1$
            }

            else if (this.getOrderAmount() == null) {
                throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterAmount")); //$NON-NLS-1$
            }

            else if (this.getOrderPrice() == null) {
                throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterPrice")); //$NON-NLS-1$
            }

            else if (this.getAktie() == null) {
                throw new ApplicationException(I18N.tr("OrderHistoryImpl.chosseStock")); //$NON-NLS-1$
            }

            else if (this.getAktie().isNewObject()) {
                throw new ApplicationException(I18N.tr("OrderHistoryImpl.storeFirst")); //$NON-NLS-1$
            }

        } catch (final RemoteException e) {
            Logger.error(I18N.tr("OrderHistoryImpl.insertCheckFaild"), e); //$NON-NLS-1$
            throw (ApplicationException) new ApplicationException(I18N.tr("OrderHistoryImpl.unableToStrore")) //$NON-NLS-1$
                    .initCause(e);
        }
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {

        // same as insertCheck
        this.insertCheck();
    }

}
