/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung eines Versionsdatensatzes.
 *
 * @author Frank Mnster
 */
public class VersionImpl extends AbstractDBObject implements Version {

    /**
     * Property "name" as a string
     */
    private static final String NAME = "name"; //$NON-NLS-1$

    /**
     * Property "version" as a string
     */
    private static final String VERSION = "version"; //$NON-NLS-1$

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -2807984014766080725L;

    /**
     * ct
     *
     * @throws RemoteException
     */
    public VersionImpl() throws RemoteException {
	super();
    }

    /**
     * @see Version#getName()
     */
    @Override
    public String getName() throws RemoteException {
	return (String) getAttribute(VersionImpl.NAME);
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {
	return VersionImpl.NAME;
    }

    /**
     * @see Version#getVersion()
     */
    @Override
    public int getVersion() throws RemoteException {
	// Wir fangen bei 0 an zu zaehlen
	Integer version = null;
	try {
	    version = (Integer) getAttribute(VersionImpl.VERSION);
	} catch (final RemoteException e) {
	    Logger.warn(I18N.tr("VersionImpl.noDbVersionFound")); //$NON-NLS-1$
	}
	return version == null ? 0 : version.intValue();
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insert()
     */
    @Override
    public void insert() throws RemoteException, ApplicationException {
	setVersion(getVersion()); // speichert automatisch die Startnummer
	super.insert();
    }

    /**
     * @see Version#setName(java.lang.String)
     */
    @Override
    public void setName(final String name) throws RemoteException {
	setAttribute(VersionImpl.NAME, name);
    }

    /**
     * @see Version#setVersion(int)
     */
    @Override
    public void setVersion(final int newVersion) throws RemoteException {
	if (newVersion < 0) {
	    throw new RemoteException(I18N.tr("VersionImpl.versionSmallerZero")); //$NON-NLS-1$
	}
	setAttribute(VersionImpl.VERSION, Integer.valueOf(newVersion));
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {
	return VersionImpl.VERSION;
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {

	try {
	    if (getName() == null || getName().length() == 0) {
		throw new ApplicationException(I18N.tr("VersionImpl.noVersionGiven")); //$NON-NLS-1$
	    }
	} catch (final RemoteException re) {
	    Logger.error(I18N.tr("VersionImpl.errorCheckVersion"), re); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(I18N.tr("VersionImpl.errorCheckVersion")) //$NON-NLS-1$
		    .initCause(re);
	}
	super.insertCheck();
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {
	insertCheck();
    }

}
