/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.ValueHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung eines Wertpapier-Datensatzes.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.LongVariable", "PMD.ExcessivePublicCount" })
public class WertpapierImpl extends AbstractDBObject implements Wertpapier {

    /**
     * Name of ARD_id column in database
     */
    private static final String ARD_ID_ATTRIBUTE = "symbol"; //$NON-NLS-1$

    /**
     * Aktien_ID Text fr SQl-Abfragen
     */
    private static final String AKTIEN_ID_SQL = "aktie_id = "; //$NON-NLS-1$

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -4540084930902107527L;

    /**
     * Wertpapier in der ARD Brese
     */
    private transient Boolean inArd;

    /**
     * Implementierung eines Wertpapaier-Datensatzes.
     *
     * @throws RemoteException
     *             {@link java.rmi.RemoteException}
     */
    public WertpapierImpl() throws RemoteException {

        super();
    }

    /**
     * @see Wertpapier#getActualTotalValue()
     */
    @Override
    public BigDecimal getActualTotalValue() throws RemoteException {

        final BigDecimal total = this.getAnzahl().multiply(this.getLastTradePriceOnly());
        return total.setScale(2, RoundingMode.HALF_EVEN);

    }

    /**
     * @see Wertpapier#getAdditionalCosts()
     */
    @Override
    public BigDecimal getAdditionalCosts() throws RemoteException {

        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            BigDecimal additionalCosts = BigDecimal.ZERO;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                additionalCosts = additionalCosts
                        .add(order.getAdditionalCosts() == null ? BigDecimal.ZERO : order.getAdditionalCosts());
            }
            return additionalCosts.setScale(2, RoundingMode.HALF_EVEN);
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("WertpapierImpl.calculatingAdditionalCostsFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getAnzahl()
     */
    @Override
    public BigDecimal getAnzahl() throws RemoteException {

        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            BigDecimal anzahl = BigDecimal.ZERO;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                anzahl = anzahl.add(order.getOrderAmount());
            }
            return anzahl;
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("WertpapierImpl.calculatingNumberOfSharesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getArdId()
     */
    @Override
    public String getArdId() throws RemoteException {

        return (String) this.getAttribute(WertpapierImpl.ARD_ID_ATTRIBUTE);
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getAttribute(java.lang.String)
     */
    @Override
    public Object getAttribute(final String attribute) throws RemoteException {

        Object attributObject;
        switch (attribute) {
            case "gesamtWert": //$NON-NLS-1$
                attributObject = this.getGesamtWert();
                break;
            case "anzahl": //$NON-NLS-1$
                attributObject = this.getAnzahl();
                break;
            default:
                attributObject = super.getAttribute(attribute);
                break;
        }
        return attributObject;
    }

    /**
     * @see Wertpapier#getChange()
     */
    @Override
    public BigDecimal getChange() throws RemoteException {

        final Double change = (Double) this.getAttribute("change"); //$NON-NLS-1$
        return change == null || Double.isNaN(change) ? BigDecimal.ZERO : BigDecimal.valueOf(change);
    }

    /**
     * @see Wertpapier#getChangeInPercentage()
     */
    @Override
    public double getChangeInPercentage() throws RemoteException {

        final Double changeInPercentage = (Double) this.getAttribute("changeInPercentage"); //$NON-NLS-1$
        return changeInPercentage == null || Double.isNaN(changeInPercentage) ? 0.0 : changeInPercentage.doubleValue();
    }

    /**
     * @see Wertpapier#getComment()
     */
    @Override
    public String getComment() throws RemoteException {

        return (String) this.getAttribute("comment"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getCurrency()
     */
    @Override
    public String getCurrency() throws RemoteException {

        return (String) this.getAttribute("currency"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getExchange()
     */
    @Override
    public String getExchange() throws RemoteException {

        return (String) this.getAttribute("exchange"); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getFfmInstrument()
     */
    @Override
    public String getFfmInstrument() throws RemoteException {

        return (String) this.getAttribute("ffmInstrument"); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getFfmName()
     */
    @Override
    public String getFfmName() throws RemoteException {

        return (String) this.getAttribute("ffmName"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getGesamtWert()
     */
    @Override
    public BigDecimal getGesamtWert() throws RemoteException {

        return this.getLastTradePriceOnly().multiply(this.getAnzahl()).setScale(2, RoundingMode.HALF_EVEN);
    }

    @Override
    public String getIsin() throws RemoteException {

        return (String) this.getAttribute("isin"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getKauf()
     */
    @Override
    @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
    public BigDecimal getKauf() throws RemoteException {

        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            BigDecimal costs = BigDecimal.ZERO;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                if (order.getOrderAmount().compareTo(BigDecimal.ZERO) > 0) {
                    costs = order.getOrderPrice().multiply(order.getOrderAmount()).add(costs);
                }
            }
            return costs.setScale(2, RoundingMode.HALF_EVEN);
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("WertpapierImpl.calculatingExpensesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getLastTradeDate()
     */
    @Override
    public Date getLastTradeDate() throws RemoteException {

        return (Date) this.getAttribute("lastTradeDate"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getLastTradePriceOnly()
     */
    @Override
    public BigDecimal getLastTradePriceOnly() throws RemoteException {

        // AbstractDBObject will create a java.lang.Double.
        // We only have to cast it.
        final Double lastTradePriceOnly = (Double) this.getAttribute("lastTradePriceOnly"); //$NON-NLS-1$
        return lastTradePriceOnly == null || Double.isNaN(lastTradePriceOnly) ? BigDecimal.ZERO
                : BigDecimal.valueOf(lastTradePriceOnly);
    }

    /**
     * @see Wertpapier#getLastTradeTime()
     */
    @Override
    public Time getLastTradeTime() throws RemoteException {

        return (Time) this.getAttribute("lastTradeTime"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getName()
     */
    @Override
    public String getName() throws RemoteException {

        // We can cast this directly to String, the method getField() knows the
        // meta data of this sql table ;)
        return (String) this.getAttribute("name"); // "name" ist //$NON-NLS-1$
        // the
        // sql field name
    }

    /**
     * @see Wertpapier#getOrderHistory()
     */
    @Override
    public DBIterator<OrderHistory> getOrderHistory() throws RemoteException {

        try {
            final DBService service = this.getService();
            final DBIterator<OrderHistory> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            orderHistory.setOrder("ORDER BY orderDate desc"); //$NON-NLS-1$
            return orderHistory;
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("WertpapierImpl.loadOrderHistoryFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getPreviousClose()
     */
    @Override
    public BigDecimal getPreviousClose() throws RemoteException {

        // AbstractDBObject will create a java.lang.Double.
        // We only have to cast it.
        final Double previousClose = (Double) this.getAttribute("previousClose"); //$NON-NLS-1$
        return previousClose == null || Double.isNaN(previousClose) ? BigDecimal.ZERO
                : BigDecimal.valueOf(previousClose.doubleValue());
    }

    /**
     * Sometimes you can display only one of the projects attributes (in combo boxes). Here you can define the name of
     * this field. Please don't confuse this with the "primary KEY".
     *
     * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {

        return WertpapierImpl.ARD_ID_ATTRIBUTE;
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getQuandlCode()
     */
    @Override
    public String getQuandlCode() throws RemoteException {

        return (String) this.getAttribute("quandlCode"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getTotalGewinn()
     */
    @Override
    public BigDecimal getTotalGewinn() throws RemoteException {

        BigDecimal gewinn = BigDecimal.ZERO;
        final DBIterator<?> orderHistory = this.getOrderHistory();
        while (orderHistory.hasNext()) {
            final OrderHistory order = (OrderHistory) orderHistory.next();
            final BigDecimal orderWin = this.getLastTradePriceOnly().subtract(order.getOrderPrice())
                    .multiply(order.getOrderAmount());
            if (Logger.isLogging(Level.DEBUG)) {
                if (order.getOrderAmount().compareTo(BigDecimal.ZERO) > 0) {
                    Logger.debug(I18N.tr("WertpapierImpl.orderProfitPurchase", orderWin.toString())); //$NON-NLS-1$
                } else if (order.getOrderAmount().compareTo(BigDecimal.ZERO) < 0) {
                    Logger.debug(I18N.tr("WertpapierImpl.orderProfitSale", orderWin.toString())); //$NON-NLS-1$
                }
            }
            gewinn = gewinn.add(orderWin);
        }
        return gewinn;
    }

    /**
     * @see Wertpapier#getType()
     */
    @Override
    public String getType() throws RemoteException {

        return (String) this.getAttribute("type"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getValueHistory()
     */
    @Override
    public DBIterator<ValueHistory> getValueHistory() throws RemoteException {

        try {
            final DBService service = this.getService();
            final DBIterator<ValueHistory> valueHistory = service.createList(ValueHistory.class);
            valueHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            return valueHistory;
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("WertpapierImpl.loadHistoryValuesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getVerkauf()
     */
    @Override
    public BigDecimal getVerkauf() throws RemoteException {

        BigDecimal realisiert = BigDecimal.ZERO;
        final DBIterator<?> orderHistory = this.getOrderHistory();
        while (orderHistory.hasNext()) {
            final OrderHistory order = (OrderHistory) orderHistory.next();
            if (order.getOrderAmount().compareTo(BigDecimal.ZERO) < 0) {
                // Nur Verkufe bercksichtigen
                realisiert = order.getOrderPrice().multiply(order.getOrderAmount().negate()).add(realisiert);
            }
        }
        return realisiert.setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see Wertpapier#getYearHigh()
     */
    @Override
    public BigDecimal getYearHigh() throws RemoteException {

        final Double yearHigh = (Double) this.getAttribute("yearHigh"); //$NON-NLS-1$
        return yearHigh == null || Double.isNaN(yearHigh) ? BigDecimal.ZERO
                : BigDecimal.valueOf(yearHigh.doubleValue());
    }

    /**
     * @see Wertpapier#getYearLow()
     */
    @Override
    public BigDecimal getYearLow() throws RemoteException {

        final Double yearLow = (Double) this.getAttribute("yearLow"); //$NON-NLS-1$
        return yearLow == null || Double.isNaN(yearLow) ? BigDecimal.ZERO : BigDecimal.valueOf(yearLow.doubleValue());
    }

    /**
     * @see Wertpapier#isInArd()
     */
    @Override
    public boolean isInArd() throws RemoteException {

        if (this.inArd == null) {
            // Aktienabfrage starten
            try {
                ArdStockInfoBean stockInfo = new ArdStockInfoBean();
                stockInfo.setId(this.getArdId());
                stockInfo = BoerseArdApi.getInstance().getActualValue(stockInfo);
                // wenn wir hier sind, gibt es das Wertpapier
                if (stockInfo != null) {
                    this.inArd = true;
                }
            } catch (final RemoteException | FinanceException e) {
                this.inArd = false;
            }
        }
        return this.inArd;
    }

    /**
     * @see Wertpapier#parseWertpapier(StockInfoBean)
     */
    @Override
    public void parseWertpapier(final StockInfoBean wpi) throws RemoteException {

        this.setChange(wpi.getChangeRate());
        if (StringUtils.isEmpty(wpi.getCurrency())) {
            wpi.setCurrency("IDX"); //$NON-NLS-1$
        } else {
            this.setCurrency(wpi.getCurrency());
        }
        this.setExchange(wpi.getExchange());
        this.setLastTradeDate(wpi.getDate());
        this.setLastTradeTime(new Time(wpi.getDate().getTime()));
        this.setLastTradePriceOnly(wpi.getActValue());
        this.setPreviousClose(null);
        this.setChangeInPercentage(wpi.getChangeRate().doubleValue());
        this.setIsin(wpi.getIsin());
    }

    /**
     * @see Wertpapier#setArdId(String)
     */
    @Override
    public void setArdId(final String ardId) throws RemoteException {

        this.setAttribute(WertpapierImpl.ARD_ID_ATTRIBUTE, ardId);
    }

    /**
     * @see Wertpapier#setChange(BigDecimal)
     */
    @Override
    public void setChange(final BigDecimal change) throws RemoteException {

        this.setAttribute("change", change.doubleValue()); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setChangeInPercentage(double)
     */
    @Override
    public void setChangeInPercentage(final double changeInPercentage) throws RemoteException {

        this.setAttribute("changeInPercentage", changeInPercentage); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setComment(String)
     */
    @Override
    public void setComment(final String comment) throws RemoteException {

        this.setAttribute("comment", comment); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setCurrency(String)
     */
    @Override
    public void setCurrency(final String currency) throws RemoteException {

        this.setAttribute("currency", currency); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setExchange(String)
     */
    @Override
    public void setExchange(final String exchange) throws RemoteException {

        this.setAttribute("exchange", exchange); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setFfmInstrument(java.lang.String)
     */
    @Override
    public void setFfmInstrument(final String ffmInstrument) throws RemoteException {

        this.setAttribute("ffmInstrument", ffmInstrument); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setFfmName(java.lang.String)
     */
    @Override
    public void setFfmName(final String ffmName) throws RemoteException {

        this.setAttribute("ffmName", ffmName); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setInArd()
     */
    @Override
    public void setInArd() {

        this.inArd = null; // NOPMD
    }

    @Override
    public void setIsin(final String isin) throws RemoteException {

        this.setAttribute("isin", isin); //$NON-NLS-1$

    }

    /**
     * @see Wertpapier#setLastTradeDate(Date)
     */
    @Override
    public void setLastTradeDate(final Date lastTradeDate) throws RemoteException {

        this.setAttribute("lastTradeDate", lastTradeDate); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setLastTradePriceOnly(BigDecimal)
     */
    @Override
    public void setLastTradePriceOnly(final BigDecimal lastTradePriceOnly) throws RemoteException {

        this.setAttribute("lastTradePriceOnly", lastTradePriceOnly.doubleValue()); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setLastTradeTime(Time)
     */
    @Override
    public void setLastTradeTime(final Time lastTradeTime) throws RemoteException {

        this.setAttribute("lastTradeTime", lastTradeTime); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setName(String)
     */
    @Override
    public void setName(final String name) throws RemoteException {

        this.setAttribute("name", name == null ? StringUtils.EMPTY : name); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setPreviousClose(BigDecimal)
     */
    @Override
    public void setPreviousClose(final BigDecimal previousClose) throws RemoteException {

        this.setAttribute("previousClose", previousClose == null ? Double.NaN : previousClose.doubleValue()); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setQuandlCode(java.lang.String)
     */
    @Override
    public void setQuandlCode(final String quandlCode) throws RemoteException {

        this.setAttribute("quandlCode", quandlCode); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setType(String)
     */
    @Override
    public void setType(final String type) throws RemoteException {

        this.setAttribute("type", type); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setYearHigh(BigDecimal)
     */
    @Override
    public void setYearHigh(final BigDecimal yearHigh) throws RemoteException {

        this.setAttribute("yearHigh", Double.valueOf(yearHigh.doubleValue())); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setYearLow(BigDecimal)
     */
    @Override
    public void setYearLow(final BigDecimal yearLow) throws RemoteException {

        this.setAttribute("yearLow", yearLow.doubleValue()); //$NON-NLS-1$
    }

    /**
     * This method will be called, before delete() is executed. Here you can make some dependency checks. If you don't
     * want to delete the project (in case of failed dependencies) you have to throw an ApplicationException. The
     * message of this one will be shown in users UI. So please translate the text into the users language.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
     */
    @Override
    protected void deleteCheck() throws ApplicationException {

        // Maybe used later
    }

    /**
     * We have to return the name of the SQL table here.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {

        return "wertpapier"; //$NON-NLS-1$
    }

    /**
     * This method is invoked before executing insert(). So lets check the entered data.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {

        try {
            if (this.getArdId() == null || this.getArdId().length() == 0) {
                throw new ApplicationException(I18N.tr("WertpapierImpl.symbolNullOrEmpty")); //$NON-NLS-1$
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("WertpapierImpl.insertCheckFailed"), e); //$NON-NLS-1$
            throw (ApplicationException) new ApplicationException(I18N.tr("WertpapierImpl.insertCheckFailedMsg")) //$NON-NLS-1$
                    .initCause(e);
        }
    }

    /**
     * This method is invoked before every update().
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {

        // we simply call the insertCheck here ;)
        this.insertCheck();
    }
}
