/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen;

import java.rmi.RemoteException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmWertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlWertpapierHistory;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.WertpapierHistory;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class WertpapierHistoryBean {

    /**
     * DAX ID bei der ARD Brse
     */
    public static final String DAX_ID = "159096"; //$NON-NLS-1$

    /**
     * Datenquelle
     */
    private String dataSource;

    /**
     * DatenAbfrage erfolgreich
     */
    private boolean dataRequestOK;

    /**
     * The period of vlaues
     */
    private DateRange period;

    /**
     * Die Wertpapier ID
     */
    private String wpId;

    /**
     * Liefert eine Liste mit den historischen Werten des Wertpapiers
     */
    private List<HistoryValueBean> wphv = new ArrayList<>();

    /**
     * Konstruktor mit Angabe der ARD ID, des Quandl-Codes und der FFM
     * Brsennames
     *
     * @param wpId
     * @param quandlCode
     * @param ffmInfoBean
     * @throws FinanceException
     */
    public WertpapierHistoryBean(final String wpId, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
            throws FinanceException {
        // In privater Methode, damit sie vom 2.ten Konstruktor mit verwendet
        // werden kann.
        this.init(wpId, quandlCode, ffmInfoBean);
    }

    /**
     * Konstruktor mit Angabe eines Wertpapier-Okjektes
     *
     * @param wertpapier
     * @throws FinanceException
     */
    public WertpapierHistoryBean(final Wertpapier wertpapier) throws FinanceException {
        String wertPapierId = null;
        try {
            wertPapierId = wertpapier.getArdId();
        } catch (final RemoteException e) { // NOPMD
            // Hier machen wir nix
        }
        String quandlCode = null;
        try {
            quandlCode = wertpapier.getQuandlCode();
        } catch (final RemoteException e) { // NOPMD
            // Und hier auch nix
        }
        FfmStockInfoBean ffmInfoBean = null;
        try {
            ffmInfoBean = new FfmStockInfoBean(wertpapier.getFfmInstrument(), wertpapier.getFfmName(), null, null);
        } catch (final RemoteException e) { // NOPMD
            // Und hier auch nix
        }
        // Und nun die eigentliche Arbeit
        this.init(wertPapierId, quandlCode, ffmInfoBean);
    }

    /**
     * Liefert die Datenquelle
     *
     * @return Datenquelle
     */
    public String getDataSource() {

        return this.dataSource;
    }

    /**
     * Liefert eine Liste mit den historischen Werten des Wertpapiers.
     *
     * @return wphv Liste mit den HistoryValueBean
     */
    public List<HistoryValueBean> getHistoryValues() {

        return this.wphv;
    }

    /**
     * Returns the period of the values as a <code>DateRange</code> object.
     *
     * @return the period
     */
    public DateRange getPeriod() {

        return new DateRange(this.period.getStartDate(), this.period.getEndDate());
    }

    /**
     * Liefert die historischen Werte f&uuml;r eine angegeben Zeitraum.
     * 
     * @param timeRange
     *            the time range
     * @return Die Liste mit HistoryValueBean
     */
    public List<HistoryValueBean> getTimeRangeValues(final DateRange timeRange) {

        final LocalDate fromDate = timeRange.getStartDate();
        final LocalDate toDate = timeRange.getEndDate();
        final List<HistoryValueBean> rangeValues = new ArrayList<>();
        for (final HistoryValueBean value : this.wphv) {
            final LocalDate valueDate = value.getLocalDate();
            if (valueDate.compareTo(fromDate) >= 0 && valueDate.compareTo(toDate) <= 0) {
                rangeValues.add(value);
            }
        }
        return rangeValues;
    }

    /**
     * Liefert die historischen Werte f&uuml;r ein angegebenes Datum. Falls kein
     * Daten f&uuml; das angegeben Datum zur Verf&uuml;gung stehen, wird der
     * n&auml;chste davorliegende Handelstag genommen. Liegt kein Tage vor
     * DATUM, werden die Daten des ersten zur Verf&uuml;gung stehenden Tages
     * verwendet. Liegen gar keine Daten vor wird NULL zur&uuml;ckgegeben.
     *
     * @param date
     *            das Datum
     * @return Der HistoryValueBean
     */
    public HistoryValueBean getValuesAtDate(final LocalDate date) {

        HistoryValueBean returnValue = null;
        if (!this.wphv.isEmpty()) {
            returnValue = this.wphv.get(0);
            for (final HistoryValueBean value : this.wphv) {
                final LocalDate dateValue = value.getLocalDate();
                if (dateValue.isEqual(date)) {
                    // Gesuchtes Datum gefunden
                    returnValue = value;
                    break;
                } else if (date.isBefore(dateValue)) {
                    // Wir sind am gesuchten Datum vorbei
                    break;
                }
                returnValue = value;
            }
        }
        return returnValue;
    }

    /**
     * Liefert die Wertpapier ID
     *
     * @return wpId
     */
    public String getwpId() {

        return this.wpId;
    }

    /**
     * War der Datenabruf OK.
     *
     * @return the dataRequestOK
     */
    public Boolean isDataRequestOK() {

        return this.dataRequestOK;
    }

    /**
     * @param wpId
     * @param quandlCode
     * @param ffmInfoBean
     * @throws FinanceException
     */
    private void init(final String wpId, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
            throws FinanceException {

        this.wpId = wpId;
        this.dataRequestOK = false;
        this.dataSource = StringUtils.EMPTY;
        // Mal bei Quandl versuchen
        if (StringUtils.isNotEmpty(quandlCode)) {
            this.loadHistory(quandlCode);
        }
        if (this.wphv.isEmpty() && ffmInfoBean != null) {
            // probieren wir es mal mit der Brse FFM
            this.loadFfm(ffmInfoBean);
        }
        if (this.wphv.isEmpty() && StringUtils.isNotEmpty(wpId)) {
            // probieren wir es mal mit der ARD Brse
            this.loadArdBoerse();
        }
        if (this.wphv.isEmpty()) {
            this.dataSource = StringUtils.EMPTY;
            throw new FinanceException("Keine historischen Daten gefunden", FinanceExceptionType.DATA_NOT_FOUND); //$NON-NLS-1$
        }
        this.dataRequestOK = true;
        // Dann sortieren.
        Collections.sort(this.wphv);
        // und das Anfangs- und Enddatum setzten
        final LocalDate fromDate = this.wphv.get(0).getLocalDate();
        final LocalDate toDate = this.wphv.get(this.wphv.size() - 1).getLocalDate();
        this.period = new DateRange(fromDate, toDate);
    }

    /**
     *
     */
    private void loadArdBoerse() {

        try {
            final ArdStockInfoBean stockInfo = new ArdStockInfoBean();
            stockInfo.setId(this.wpId);
            BoerseArdApi.getInstance().loadHistoryValues(stockInfo);
            this.wphv = stockInfo.getHistoryValues().getValues();
            this.dataSource = stockInfo.getHistoryValues().getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage(), ex);
        }
    }

    /**
     * @param ffmInfoBean
     */
    private void loadFfm(final FfmStockInfoBean ffmInfoBean) {

        try {
            final WertpapierHistory fwh = new FfmWertpapierHistory(ffmInfoBean);
            this.wphv = fwh.getValues();
            this.dataSource = fwh.getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage(), ex);
        }
    }

    /**
     * @param quandlCode
     */
    private void loadHistory(final String quandlCode) {

        try {
            final WertpapierHistory qwh = new QuandlWertpapierHistory(quandlCode, Settings.getQuandlApiKey());
            this.wphv = qwh.getValues();
            this.dataSource = qwh.getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage());
        }
    }
}
