/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class FfmWebAPI {

    /**
     * Jackson Mapper
     */
    public static final ObjectMapper MAPPER = new ObjectMapper();

    /**
     * B&OUML;rese FFM URL
     */
    public static final String MAIN_URL = "http://www.boerse-frankfurt.de/"; //$NON-NLS-1$

    /**
     * B&OUML;rese FFM URL proxy
     */
    public static final String PROXY_URL = "http://proxy.boerse-frankfurt.de/"; //$NON-NLS-1$

    /**
     * Such URL
     */
    public static final String SEARCH_URL = FfmWebAPI.MAIN_URL + "Ajax/SearchController_JsonSuggest?Query="; //$NON-NLS-1$

    /**
     * Such URL Parameter
     */
    public static final String SEARCH_URL_PARAMS = "&SearchType=false"; //$NON-NLS-1$

    /**
     * Unsere Instanz
     */
    private static FfmWebAPI instance;

    /**
     * Liste der schon abgefragte URLs
     */
    private final Map<URI, String> cacheUrlResponse = new ConcurrentHashMap<>();

    /**
     * Liefert die FfmWebAPI Instanz. Wird bei bedarf erstellt.
     *
     * @return die FfmWebAPI-Instanz
     */
    public static FfmWebAPI getInstance() {

        synchronized (FfmWebAPI.class) {
            if (FfmWebAPI.instance == null) {
                FfmWebAPI.instance = new FfmWebAPI();
            }
        }
        return FfmWebAPI.instance;
    }

    /**
     * Privater Konstruktor
     */
    private FfmWebAPI() {

    }

    /**
     * Liefert die Antwort eines Web-Services Aufrufs.
     *
     * @param url
     *            des Webservices.
     * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
     *         dem Aufruf alles in Ordnung war, aber keine Antwort kam.
     * @throws FinanceException
     *             falls w&auml;hrend des Aufrufs etwas schief ging.
     */
    public String getResonseGETRequest(final URL url) throws FinanceException {

        return this.getResonseGETRequest(url, false);
    }

    /**
     * Liefert die Antwort eines Web-Services Aufrufs.
     *
     * @param url
     *            des Webservices.
     * @param useCache
     *            <code>true</code> falls der interne Cache verwendet werden
     *            soll, sonst <code>false</code>.
     * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
     *         dem Aufruf alles in Ordnung war, aber keine Antwort kam.
     * @throws FinanceException
     *             falls w&auml;hrend des Aufrufs etwas schief ging.
     */
    public String getResonseGETRequest(final URL url, final boolean useCache) throws FinanceException {

        String response;
        try {
            final URI uri = url.toURI();
            if (useCache && this.cacheUrlResponse.containsKey(uri)) {
                Logger.debug(I18N.tr("WebAPI.loadfromCacheInfo")); //$NON-NLS-1$
                response = this.cacheUrlResponse.get(uri);
            } else {
                if (Level.DEBUG.equals(Logger.getLevel())) {
                    Logger.debug(I18N.tr("WebAPI.url", url.toString())); //$NON-NLS-1$
                }
                final HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET"); //$NON-NLS-1$
                conn.setConnectTimeout(2000);
                conn.setReadTimeout(5000);
                conn.setRequestProperty("Accept-Charset", StandardCharsets.UTF_8.name()); //$NON-NLS-1$
                conn.setRequestProperty("User-Agent", "JaDepot"); //$NON-NLS-1$ //$NON-NLS-2$
                Logger.info(conn.getResponseMessage());
                if (conn.getResponseCode() != HttpURLConnection.HTTP_OK) {
                    throw new FinanceException(I18N.tr("WebAPI.httpError" //$NON-NLS-1$
                            , Integer.toString(conn.getResponseCode())), FinanceExceptionType.NETWORK);
                }
                final BufferedReader bufferReader = new BufferedReader(
                        new InputStreamReader(conn.getInputStream(), StandardCharsets.UTF_8));
                final StringBuilder sBuffer = new StringBuilder();
                String output;
                while ((output = bufferReader.readLine()) != null) { // NOPMD
                    sBuffer.append(output).append(System.lineSeparator());
                }
                response = sBuffer.toString();
                conn.disconnect();
                if (useCache) {
                    this.cacheUrlResponse.put(uri, response);
                }
            }
        } catch (final URISyntaxException e) {
            throw new FinanceException(I18N.tr("WebAPI.malformedUrl", e.getMessage()), //$NON-NLS-1$
                    FinanceExceptionType.NETWORK, e);
        } catch (final IOException e) {
            throw new FinanceException(I18N.tr("WebAPI.ioException", e.getMessage()), //$NON-NLS-1$
                    FinanceExceptionType.NETWORK, e);
        }

        return response;
    }
}
