/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.ChartData;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.ChartDataResponse;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.Instrument;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.InstrumentType;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.SearchResponse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.WertpapierHistory;
import de.willuhn.logging.Logger;

/**
 * L&auml;d historisch Werte von Frankfurter B&ouml;se.
 *
 * @author Frank Mnster
 */
public class FfmWertpapierHistory implements WertpapierHistory {

    /**
     * Jackson Mapper
     */
    private static final ObjectMapper MAPPER = new ObjectMapper();

    /**
     * Daten URL
     */
    private static final String DATA_URL = FfmWebAPI.PROXY_URL + "cst/BoerseFrankfurt/Share/chart.json?instruments="; //$NON-NLS-1$

    /**
     * Daten URL Parameter
     */
    private static final String DATA_URL_PARAMS = "&period=TenYears"; //$NON-NLS-1$

    /**
     * Zahl eins
     */
    private static final int EXACT_ONE = 1;

    /**
     * Der eindeutige Name
     */
    private final FfmStockInfoBean infoBean;

    /**
     * Konstruktor mit Angabe des FFM InfoBeans
     *
     * @param infoBean
     */
    public FfmWertpapierHistory(final FfmStockInfoBean infoBean) {
	this.infoBean = infoBean;
    }

    /**
     * @see WertpapierHistory#getSourceName()
     */
    @Override
    public String getSourceName() {

	return "Brese Frankfurt"; //$NON-NLS-1$
    }

    /**
     * @see WertpapierHistory#getValues()
     */
    @Override
    public List<HistoryValueBean> getValues() throws FinanceException {

	List<HistoryValueBean> wphvList = new ArrayList<>();
	if (StringUtils.isNotEmpty(this.infoBean.getName())
		&& StringUtils.isNotEmpty(this.infoBean.getInstrumentId())) {
	    final URL url = this.getMainUrl();
	    if (url != null) {
		final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(url, true);
		final int pos = responseString.indexOf("preLoadedSeries[0] = {name:"); //$NON-NLS-1$
		if (pos > 0) {
		    Logger.debug("Super, gefunden!! "); //$NON-NLS-1$
		    final int start = responseString.indexOf("identifier:"); //$NON-NLS-1$
		    final int end = responseString.indexOf("',language:", pos); //$NON-NLS-1$
		    final String instrument = responseString.substring(start + 13, end);
		    wphvList = this.readData(instrument);
		}
	    }
	}
	return wphvList;
    }

    /**
     * Untersucht eine List von INstruments, ob das gesuchte darunter ist und
     * gibt die Abfrage URL zurck.
     *
     * @param inst
     *            Liste der Instruments
     * @return URL des passenden Instruments oder einen leerem StringBuilder
     */
    private StringBuilder checkFoundInstruments(final List<InstrumentType> inst) {

	final StringBuilder urlSB = new StringBuilder();
	for (final InstrumentType instrument : inst) {
	    Logger.debug(instrument.getName() + " - " + instrument.getInstrumentID()); //$NON-NLS-1$
	    if (this.infoBean.getInstrumentId() != null
		    && this.infoBean.getInstrumentId().equalsIgnoreCase(Integer.toString(instrument.getInstrumentID()))
		    || this.infoBean.getName().equalsIgnoreCase(instrument.getName())) {

		final String urlType = InstrumentTypeEnum.getUrl(instrument.getInstrumentType());
		if (urlType != null) {
		    if (instrument.getInstrumentType().equalsIgnoreCase("aktien")) { //$NON-NLS-1$
			urlSB.append("aktie/").append(instrument.getUrlName()).append("-Aktie"); //$NON-NLS-1$ //$NON-NLS-2$
		    } else {
			urlSB.append(urlType).append('/').append(instrument.getUrlName());
		    }
		}
		break;
	    }
	}
	return urlSB;
    }

    /**
     * Ermittelt die konkrete URL zu dem Wertpapier.
     *
     * @return URL zu dem Instrument
     * @throws FinanceException
     */
    private URL getMainUrl() throws FinanceException {

	URL url = null;
	final String name = this.infoBean.getName();
	if (name != null) {
	    try {
		final String encodedName = URLEncoder.encode(name, java.nio.charset.StandardCharsets.UTF_8.name());
		StringBuilder urlSB = new StringBuilder(130);
		urlSB.append(FfmWebAPI.SEARCH_URL).append(encodedName).append(FfmWebAPI.SEARCH_URL_PARAMS);
		final URL searchUrl = new URL(urlSB.toString());
		final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(searchUrl, true);
		final SearchResponse searchResponse = FfmWertpapierHistory.MAPPER.readValue(responseString,
			SearchResponse.class);
		final List<InstrumentType> inst = searchResponse.getInstruments();
		urlSB.setLength(0);
		urlSB = this.checkFoundInstruments(inst);
		if (urlSB.length() > 0) {
		    urlSB.insert(0, FfmWebAPI.MAIN_URL);
		    url = new URL(urlSB.toString());
		}
	    } catch (final IOException e) {
		Logger.error(I18N.tr("FfmWertpapierHistory.createSearchURLFailed"), e); //$NON-NLS-1$
		throw new FinanceException(I18N.tr("FfmWertpapierHistory.createSearchURLFailed"), //$NON-NLS-1$
			FinanceExceptionType.NETWORK, e);
	    }
	}
	return url;
    }

    private List<HistoryValueBean> readData(final String instrumentID) throws FinanceException {

	final List<HistoryValueBean> wphv = new ArrayList<>();
	final StringBuilder urlString = new StringBuilder(FfmWertpapierHistory.DATA_URL).append(instrumentID)
		.append(FfmWertpapierHistory.DATA_URL_PARAMS);
	URL url;
	try {
	    url = new URL(urlString.toString());
	    Logger.debug("URL: " + url.toString()); //$NON-NLS-1$
	    final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(url, false);
	    Logger.trace(responseString);
	    final ChartDataResponse dataResponse = FfmWertpapierHistory.MAPPER.readValue(responseString,
		    ChartDataResponse.class);
	    if (dataResponse.getInstruments().size() != FfmWertpapierHistory.EXACT_ONE) {
		throw new FinanceException(I18N.tr("FfmWertpapierHistory.stockNotFound"), //$NON-NLS-1$
			FinanceExceptionType.DATA_NOT_FOUND);
	    }
	    final Instrument instrument = dataResponse.getInstruments().get(0);
	    for (final ChartData data : instrument.getData()) {
		final HistoryValueBean historyBean = new HistoryValueBean(instrumentID); // NOPMD
		historyBean.setLocalDate(data.getDate());
		historyBean.setOpen(data.getOpen());
		historyBean.setClose(data.getAdjClose());
		historyBean.setAdjClose(data.getAdjClose());
		historyBean.setHigh(data.getHigh());
		historyBean.setLow(data.getLow());
		historyBean.setVolume(data.getVolume());
		historyBean.setIdString(this.infoBean.getName());
		wphv.add(historyBean);
	    }
	} catch (final IOException e) {
	    Logger.error(I18N.tr("FfmWertpapierHistory.dataURLWrong"), e); //$NON-NLS-1$
	    throw new FinanceException(I18N.tr("FfmWertpapierHistory.dataURLWrong"), //$NON-NLS-1$
		    FinanceExceptionType.DATA_NOT_FOUND, e);
	}
	return wphv;
    }
}
