/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.time.DateUtils;

import de.frankmuenster.jameica.depot.finanzen.currate.json.CurRates;
import de.frankmuenster.jameica.depot.finanzen.currate.json.Rate;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.CurrencyConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.willuhn.logging.Logger;

/**
 * Webclient for CurRate service. Dealing also with http code returned by the
 * service.
 *
 * @author Frank Mnster
 */
public final class CurRateConverter implements CurrencyConverter {

    /**
     * Datumsformat bei CurRate
     */
    private static final String PATTERN = "yyyy-MM-dd"; //$NON-NLS-1$

    /**
     * CurRate API key
     */
    private static final String API_KEY = "f94b4736-456d-11e8-842f-0ed5f89f718b"; //$NON-NLS-1$

    /**
     * Die interne Instanz
     */
    private static CurRateConverter instance;

    /**
     * Oldest available date are from
     */
    private static final Date dateFirst;
    static {
        final Calendar cal = Calendar.getInstance(Locale.getDefault());
        cal.set(1999, 0, 4, 0, 0, 0);
        dateFirst = cal.getTime();
    }

    /**
     * Liefert die Instanz des CurRate-W&auml;hrungsumrechner.
     *
     * @return die Instanz des CurRate-W&auml;hrungsumrechner.
     */
    public static CurRateConverter getInstance() {

        synchronized (CurRateConverter.class) {
            if (CurRateConverter.instance == null) {
                CurRateConverter.instance = new CurRateConverter();
            }
        }
        return CurRateConverter.instance;
    }

    /**
     * Privater Konstruktor um externe Instanziierung zu verhindern.
     */
    private CurRateConverter() {

    }

    /**
     * @see CurrencyConverter#exchangeRate(String, String)
     */
    @Override
    public ExchangeRate exchangeRate(final String currencyFrom, final String currencyTo) throws FinanceException {

        final ExchangeRate exRate = this.exchangeRate(currencyFrom, currencyTo, null);
        exRate.setExcatDate(true);
        return exRate;
    }

    /**
     * @see CurrencyConverter#exchangeRate(String, String, java.util.Date)
     */
    @Override
    public ExchangeRate exchangeRate(final String currencyFrom, final String currencyTo, final Date date)
            throws FinanceException {

        if (date != null && date.before(CurRateConverter.dateFirst)) {
            throw new FinanceException(I18N.tr("CurrencyConverter.dateTooOld"), //$NON-NLS-1$
                    FinanceExceptionType.DATE_TOO_OLD);
        }

        Date requestDate = date;
        if (requestDate == null) {
            requestDate = new Date();
        }

        ExchangeRate rateCurFrom;
        ExchangeRate rateCurTo;
        if ("EUR".equalsIgnoreCase(currencyFrom)) { //$NON-NLS-1$
            rateCurTo = this.getEuroRate(currencyTo, requestDate);
            rateCurFrom = new ExchangeRate(rateCurTo.getDate(), BigDecimal.ONE);
            rateCurFrom.setExcatDate(rateCurTo.isExcatDate());
        } else if ("EUR".equalsIgnoreCase(currencyTo)) { //$NON-NLS-1$
            rateCurFrom = this.getEuroRate(currencyFrom, requestDate);
            rateCurTo = new ExchangeRate(rateCurFrom.getDate(), BigDecimal.ONE);
            rateCurTo.setExcatDate(rateCurFrom.isExcatDate());
        } else {
            rateCurFrom = this.getEuroRate(currencyFrom, requestDate);
            rateCurTo = this.getEuroRate(currencyTo, rateCurFrom.getDate());
        }
        if (DateUtils.isSameDay(rateCurFrom.getDate(), rateCurTo.getDate())) {

            final BigDecimal rate = BigDecimal.ONE.divide(rateCurFrom.getRate(), MathContext.DECIMAL128)
                    .multiply(rateCurTo.getRate()).setScale(5, BigDecimal.ROUND_HALF_UP);
            final ExchangeRate exRate = new ExchangeRate(rateCurFrom.getDate(), rate);
            exRate.setExcatDate(DateUtils.isSameDay(requestDate, rateCurFrom.getDate()));

            return exRate;

        }

        throw new FinanceException("Rate dates of from and to currencies are not the same", //$NON-NLS-1$
                FinanceExceptionType.OTHER);

    }

    private ExchangeRate getEuroRate(final String currency, final Date date) throws FinanceException {

        final SimpleDateFormat sdf = new SimpleDateFormat(CurRateConverter.PATTERN, Locale.GERMAN);

        String request = ""; //$NON-NLS-1$
        URL url;

        try {
            request = CurRateWebAPI.MAIN_URL + "getRate?curTo=" + currency + //$NON-NLS-1$
                    "&date=" + sdf.format(date) + //$NON-NLS-1$
                    "&apiKey=" + CurRateConverter.API_KEY; //$NON-NLS-1$
            url = new URL(request);
            final String responseString = CurRateWebAPI.getInstance().getResonseGETRequest(url, true);
            final CurRates curRate = CurRateWebAPI.MAPPER.readValue(responseString, CurRates.class);
            if (!curRate.getRates().isEmpty()) {
                final Rate rate = curRate.getRates().get(0);
                final Date rateDay = sdf.parse(rate.getDate());
                final ExchangeRate exRate = new ExchangeRate(rateDay, rate.getRateValue());
                exRate.setExcatDate(DateUtils.isSameDay(date, rateDay));
                return exRate;
            }
            Logger.error("CurRate answered with an unexpected failue."); //$NON-NLS-1$
            throw new FinanceException("CurRate answered with an unexpected failue.", //$NON-NLS-1$
                    FinanceExceptionType.OTHER);

        } catch (final MalformedURLException e) {
            throw new FinanceException(I18N.tr("WebAPI.malformedUrl", request), //$NON-NLS-1$
                    FinanceExceptionType.OTHER, e);
        } catch (final IOException e) {
            Logger.error(I18N.tr("CurrencyConverter.jsonDeserialisationFailed"), e); //$NON-NLS-1$
            throw new FinanceException(I18N.tr("CurrencyConverter.jsonDeserialisationFailed"), //$NON-NLS-1$
                    FinanceExceptionType.DATA_NOT_FOUND, e);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("CurrencyConverter.changeDateNotReadable"), e); //$NON-NLS-1$
            throw new FinanceException(I18N.tr("CurrencyConverter.changeDateNotReadable"), //$NON-NLS-1$
                    FinanceExceptionType.DATA_NOT_FOUND, e);
        }

    }

}
