/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.frankmuenster.jameica.depot.finanzen.currate.json.CurRateError;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * HTTP layer of the web client for the CurRate service. Deals also with http error codes (>= 300).
 *
 * @author Frank Mnster
 */
public final class CurRateWebAPI {

    /**
     * Jackson Mapper
     */
    public static final ObjectMapper MAPPER = new ObjectMapper();

    /**
     * CurRate URL
     */
    public static final String MAIN_URL = "https://services.frankmuenster.de/currency/"; //$NON-NLS-1$

    /**
     * Standardantwort, falls nichts gefunden wurde.
     */
    private static final String NOT_A_RESOURCE = "NaR"; //$NON-NLS-1$

    /**
     * Unser Instanz
     */
    private static CurRateWebAPI instance;

    /**
     * Liste der schon abgefragte URLs
     */
    private final Map<URI, String> cacheUrlResponse = new ConcurrentHashMap<>();

    /**
     * Liefert die FixerIO-API Instanz. Wird bei bedarf erstellt.
     *
     * @return die FixerIO-Instanz
     */
    public static CurRateWebAPI getInstance() {

        synchronized (CurRateWebAPI.class) {
            if (CurRateWebAPI.instance == null) {
                CurRateWebAPI.instance = new CurRateWebAPI();
            }
        }
        return CurRateWebAPI.instance;
    }

    /**
     * Privater Konstruktor
     */
    private CurRateWebAPI() {

    }

    /**
     * Liefert die Antwort eines Web-Services Aufrufs.
     *
     * @param url
     *            des Webservices.
     * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit dem Aufruf alles in Ordnung war, aber
     *         keine Antwort kam.
     * @throws FinanceException
     *             falls w&auml;hrend des Aufrufs etwas schief ging.
     */
    public String getResonseGETRequest(final URL url) throws FinanceException {

        return this.getResonseGETRequest(url, false);
    }

    /**
     * Liefert die Antwort eines Web-Services Aufrufs.
     *
     * @param url
     *            des Webservices.
     * @param useCache
     *            <code>true</code> falls der interne Cache verwendet werden soll, sonst <code>false</code>.
     * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit dem Aufruf alles in Ordnung war, aber
     *         keine Antwort kam.
     * @throws FinanceException
     *             falls w&auml;hrend des Aufrufs etwas schief ging.
     */
    public String getResonseGETRequest(final URL url, final boolean useCache) throws FinanceException {

        String response;
        try {
            final URI uri = url.toURI();
            if (useCache && this.cacheUrlResponse.containsKey(uri)) {
                Logger.debug("Lese aus Cache"); //$NON-NLS-1$
                response = this.cacheUrlResponse.get(uri);
            } else {
                final HttpRespone httpResponse = this.getRespone(url);
                if (useCache) {
                    this.cacheUrlResponse.put(uri, httpResponse.getResponse());
                }
                if (httpResponse.getCode() != HttpURLConnection.HTTP_OK) {
                    this.inspectError(httpResponse);
                }
                response = httpResponse.getResponse();
            }
        } catch (final URISyntaxException e) {
            throw new FinanceException(I18N.tr("WebAPI.malformedUrl", e.getMessage()), //$NON-NLS-1$
                    FinanceExceptionType.NETWORK, e);
        }

        return response;
    }

    /**
     * @param url
     *            the URL to query
     * @return the HttpResonse of the request
     * @throws FinanceException
     */
    private HttpRespone getRespone(final URL url) throws FinanceException {

        final HttpRespone response = new HttpRespone(HttpURLConnection.HTTP_BAD_REQUEST, CurRateWebAPI.NOT_A_RESOURCE);
        try {
            if (Level.DEBUG.equals(Logger.getLevel())) {
                Logger.debug("CurRate.url " + url.toString()); //$NON-NLS-1$
            }
            final HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            Logger.debug("Using proxy:  " + conn.usingProxy()); //$NON-NLS-1$
            conn.setRequestMethod("GET"); //$NON-NLS-1$
            conn.setConnectTimeout(2000);
            conn.setRequestProperty("Accept", "application/json"); //$NON-NLS-1$ //$NON-NLS-2$
            // //$NON-NLS-1$ //$NON-NLS-2$
            conn.setRequestProperty("Accept-Charset", StandardCharsets.UTF_8.name()); //$NON-NLS-1$
            conn.setRequestProperty("User-Agent", "JaDepot"); //$NON-NLS-1$ //$NON-NLS-2$
            Logger.info(conn.getResponseMessage());
            Logger.debug("Using proxy:  " + conn.usingProxy()); //$NON-NLS-1$

            BufferedReader bufferReader;
            if (HttpURLConnection.HTTP_OK <= conn.getResponseCode()
                    && conn.getResponseCode() < HttpURLConnection.HTTP_MULT_CHOICE) {
                bufferReader = new BufferedReader(new InputStreamReader(conn.getInputStream(), StandardCharsets.UTF_8));
            } else if (conn.getResponseCode() >= HttpURLConnection.HTTP_BAD_REQUEST) {
                bufferReader = new BufferedReader(new InputStreamReader(conn.getErrorStream(), StandardCharsets.UTF_8));
            } else {
                bufferReader = new BufferedReader(new StringReader(StringUtils.EMPTY));
            }
            final StringBuilder sBuffer = new StringBuilder();
            String output;
            while ((output = bufferReader.readLine()) != null) { // NOPMD
                sBuffer.append(output).append(System.lineSeparator());
            }

            response.setCode(conn.getResponseCode());
            response.setResponse(sBuffer.toString());
            conn.disconnect();
        } catch (final MalformedURLException e) {
            throw (FinanceException) new FinanceException(I18N.tr("WebAPI.malformedUrl", e.getMessage()), //$NON-NLS-1$
                    FinanceExceptionType.NETWORK).initCause(e);
        } catch (final IOException e) {
            throw new FinanceException(I18N.tr("WebAPI.ioException", e.getMessage()), // NOPMD //$NON-NLS-1$
                    FinanceExceptionType.NETWORK);
        }
        return response;

    }

    /**
     * Check an httpResonse if a http error (code >= 300) occurred and throws a FinanzException
     *
     * @param httpResponse
     *            the httpResonse object to inspect
     * @throws FinanceException
     */
    private void inspectError(final HttpRespone httpResponse) throws FinanceException {

        try {

            int errorCode;
            String message;
            if (StringUtils.isNotEmpty(httpResponse.getResponse())) {
                final CurRateError curError = CurRateWebAPI.MAPPER.readValue(httpResponse.getResponse(),
                        CurRateError.class);
                errorCode = curError.getErrorCode();
                message = curError.getMessage();
            } else {
                errorCode = httpResponse.getCode();
                message = StringUtils.EMPTY;

            }

            switch (httpResponse.getCode()) {
                case HttpURLConnection.HTTP_BAD_REQUEST:

                    switch (CurRateServiceErrors.getError(errorCode)) {
                        case CURRENCY_NOT_SUPPORTED:
                            throw new FinanceException(I18N.tr("WebAPI.currencyNotSupported"), //$NON-NLS-1$
                                    FinanceExceptionType.CURRENCY_NOT_SUPPORTED);
                        case NO_RATE_FOR_DATE:
                            throw new FinanceException(I18N.tr("WebAPI.dataNotFound"), //$NON-NLS-1$
                                    FinanceExceptionType.DATA_NOT_FOUND);
                        case DATE_TOO_OLD:
                            throw new FinanceException(I18N.tr("WebAPI.dateTooOld"), //$NON-NLS-1$
                                    FinanceExceptionType.DATE_TOO_OLD);
                        default:
                            throw new FinanceException(I18N.tr("WebAPI.otherError" //$NON-NLS-1$
                                    , Integer.toString(errorCode)), FinanceExceptionType.OTHER);
                    }
                case HttpURLConnection.HTTP_INTERNAL_ERROR:
                    throw new FinanceException(I18N.tr("WebAPI.serverError" //$NON-NLS-1$
                            , Integer.toString(errorCode), message), FinanceExceptionType.OTHER);
                default:
                    throw new FinanceException(I18N.tr("WebAPI.unknownHttpError" //$NON-NLS-1$
                            , Integer.toString(httpResponse.getCode()), message), FinanceExceptionType.OTHER);

            }

        } catch (final IOException e) {
            Logger.error(I18N.tr("WebAPI.jsonDeserialisationFailed"), e); //$NON-NLS-1$
            throw new FinanceException(I18N.tr("WebAPI.jsonDeserialisationFailed"), //$NON-NLS-1$
                    FinanceExceptionType.DATA_NOT_FOUND, e);

        }
    }

    /**
     * Represent an http response composed out of the http code and the response body message
     *
     * @author Frank Mnster
     */
    private class HttpRespone {

        /**
         * The http code
         */
        private int code;

        /**
         * The response http body
         */
        private String response;

        /**
         * Creates a new HttpResonse object
         *
         * @param code
         * @param reponse
         */
        public HttpRespone(final int code, final String reponse) {

            this.code = code;
            this.response = reponse;

        }

        /**
         * @return the code
         */
        public int getCode() {

            return this.code;
        }

        /**
         * @return the response
         */
        public String getResponse() {

            return this.response;
        }

        /**
         * @param code
         *            the code to set
         */
        public void setCode(final int code) {

            this.code = code;
        }

        /**
         * @param response
         *            the response to set
         */
        public void setResponse(final String response) {

            this.response = response;
        }

    }

}
