/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.time.LocalDate;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.jimmoores.quandl.HeaderDefinition;
import com.jimmoores.quandl.Row;
import com.jimmoores.quandl.TabularResult;

import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.willuhn.logging.Logger;

/**
 * Reads the history values of a Quandl response into a
 * <code>List<HistoryValueBean></code> list. Different column names can be
 * configured to find the correct values.
 *
 * @author Frank Mnster
 */
public final class HistoryReader {

    /**
     * Date Namen
     */
    private static final List<String> DATE = new ArrayList<>(Arrays.asList("Date")); //$NON-NLS-1$

    /**
     * Open Namen
     */
    private static final List<String> OPEN = new ArrayList<>(Arrays.asList("Open", "Previous Day Price", "USD (AM)")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    /**
     * Close Namen
     */
    private static final List<String> CLOSE = new ArrayList<>(Arrays.asList("Close", "Last", "Value", "USD (PM)")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

    /**
     * High Namen
     */
    private static final List<String> HIGH = new ArrayList<>(Arrays.asList("High")); //$NON-NLS-1$

    /**
     * Low Namen
     */
    private static final List<String> LOW = new ArrayList<>(Arrays.asList("Low")); //$NON-NLS-1$

    /**
     * Volume Namen
     */
    private static final List<String> VOLUME = new ArrayList<>(Arrays.asList("Trade Volume", "Volume")); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * List die historischen Werte aus einem TabularResult
     *
     * @param idString
     * @param result
     * @return Liste mit den Werten
     */
    public static List<HistoryValueBean> resultReader(final String idString, final TabularResult result) {

	final List<HistoryValueBean> values = new ArrayList<>();
	if (result != null) {
	    final Map<ColNames, Integer> indexMap = HistoryReader.findIndexes(result.getHeaderDefinition());

	    for (int i = 0; i < result.size(); i++) {
		// Reader row as HistoryValue and add to list
		final Row row = result.get(i);
		final HistoryValueBean value = HistoryReader.getValueFromRow(idString, row, indexMap);
		values.add(value);
	    }
	}
	return values;
    }

    /**
     * Wandelt ein <b>Double</b> Objekt in einen <b>double</b> Wert um. Gibt
     * <b>Double.NaN</b> wenn <i>doubleValue</i> <b>NULL</b> ist.
     *
     * @param doubleValue
     *            das Double-Objekt
     * @return der double Wert oder Double.NaN, wenn doubleValue NULL ist.
     */
    private static double doubelOrNan(final Double doubleValue) {

	return doubleValue == null ? Double.NaN : doubleValue;
    }

    /**
     * Ermittelt die Spaltenindexes
     *
     * @param headerInfo
     * @return Map mit den Indexes
     */
    private static Map<ColNames, Integer> findIndexes(final HeaderDefinition headerInfo) {

	final Map<ColNames, Integer> map = new ConcurrentHashMap<>();
	map.put(ColNames.DATE, HistoryReader.getIndexOf(headerInfo, HistoryReader.DATE));
	map.put(ColNames.OPEN, HistoryReader.getIndexOf(headerInfo, HistoryReader.OPEN));
	map.put(ColNames.CLOSE, HistoryReader.getIndexOf(headerInfo, HistoryReader.CLOSE));
	map.put(ColNames.HIGH, HistoryReader.getIndexOf(headerInfo, HistoryReader.HIGH));
	map.put(ColNames.LOW, HistoryReader.getIndexOf(headerInfo, HistoryReader.LOW));
	map.put(ColNames.VOLUME, HistoryReader.getIndexOf(headerInfo, HistoryReader.VOLUME));
	return map;
    }

    /**
     * Ermittelt den Index der Spalte
     *
     * @param headerInfo
     *            die HeaderInfos
     * @param titels
     *            der mglichen Spaltennamen
     * @return den Index null basiert oder -1, falls die Spalte nicht existiert.
     */
    private static int getIndexOf(final HeaderDefinition headerInfo, final List<String> titels) {

	int index = -1;
	for (final String columnTitel : titels) {
	    try {
		index = headerInfo.columnIndex(columnTitel);
	    } catch (final IllegalArgumentException e) { // NOPMD
		// Nix zu tun hier
	    }
	}
	return index;
    }

    /**
     * Returns a <code>HistoryValueBean</code> object with the values from the
     * row mapped with the given indexMap.
     *
     * @param idString
     *            the idString of the valueBean
     * @param row
     *            the row to take the data from
     * @param indexMap
     *            the indexMap to find the correct table columns for a value
     * @return the <code>HistoryValueBean</code> object with the values of the
     *         row
     */
    private static HistoryValueBean getValueFromRow(final String idString, final Row row,
	    final Map<ColNames, Integer> indexMap) {

	final HistoryValueBean value = new HistoryValueBean(idString);
	try {
	    final LocalDate date = LocalDate.parse(row.getString(indexMap.get(ColNames.DATE)));
	    value.setLocalDate(date);
	} catch (final DateTimeParseException e) {
	    Logger.error("Fehler beim Lesen des Datums.", e); //$NON-NLS-1$
	}
	if (indexMap.get(ColNames.OPEN) >= 0) {
	    final Double open = row.getDouble(indexMap.get(ColNames.OPEN));
	    value.setOpen(HistoryReader.doubelOrNan(open));
	}
	if (indexMap.get(ColNames.CLOSE) >= 0) {
	    final Double close = row.getDouble(indexMap.get(ColNames.CLOSE));
	    value.setClose(HistoryReader.doubelOrNan(close));
	}
	if (indexMap.get(ColNames.HIGH) >= 0) {
	    final Double high = row.getDouble(indexMap.get(ColNames.HIGH));
	    value.setHigh(HistoryReader.doubelOrNan(high));
	}
	if (indexMap.get(ColNames.LOW) >= 0) {
	    final Double low = row.getDouble(indexMap.get(ColNames.LOW));
	    value.setLow(HistoryReader.doubelOrNan(low));
	}
	if (indexMap.get(ColNames.VOLUME) >= 0) {
	    final Double low = row.getDouble(indexMap.get(ColNames.VOLUME));
	    value.setVolume(HistoryReader.doubelOrNan(low));
	}
	return value;
    }

    /**
     * Standardkonstruktor
     */
    private HistoryReader() {
	// Absichtlich leer
    }

    /**
     * Aufzhlung der Spaltennamen
     */
    private enum ColNames {
	DATE, OPEN, CLOSE, HIGH, LOW, VOLUME
    }
}
