/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.time.LocalDate;
import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.List;

import com.jimmoores.quandl.MetaDataResult;
import com.jimmoores.quandl.SearchRequest;
import com.jimmoores.quandl.SearchResult;
import com.jimmoores.quandl.classic.ClassicQuandlSession;
import com.jimmoores.quandl.util.QuandlRequestFailedException;

import de.frankmuenster.jameica.depot.finanzen.SearchWertpapier;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class QuandlSearch implements SearchWertpapier<QuandlStockInfoBean> {

    /**
     * Text fr boolean false
     */
    private static final String FALSE_TEXT = "false"; //$NON-NLS-1$

    /**
     * Unsere Instanz
     */
    private static QuandlSearch instance;

    /**
     * Quandl Attribute fr Frequency-Typ
     */
    private static final String QUANDL_FREQUENCY_ID = "frequency"; //$NON-NLS-1$ //NOPMD

    /**
     * Quandl Attribute fr Premiumseries
     */
    private static final String QUANDL_PREMIUM_ID = "premium"; //$NON-NLS-1$

    /**
     * Quandl Attribute fr Serientyp
     */
    private static final String QUANDL_TYPE_ID = "type"; //$NON-NLS-1$

    /**
     * Quandl Wert fr tgliche Series
     */
    private static final String QUNADL_FREQUENCY_DAILY = "daily"; //$NON-NLS-1$ //NOPMD

    /**
     * Der Quandl-Name
     */
    private static final String QUNADL_NAME = "name"; //$NON-NLS-1$

    /**
     * Quandl Wert fr Zeitreihen
     */
    private static final String QUNADL_TYPE_TIME_SERIES = "Time Series"; //$NON-NLS-1$ //NOPMD

    /**
     * Die zugrundeliegende Quandl4J session
     */
    private ClassicQuandlSession session;

    /**
     * Liefert die QuandlSearch Instanz.
     *
     * @param authToken
     *            Der API-KEY, kann NULL sein.
     * @return die QuandlSearch Instanz.
     */
    public static SearchWertpapier<QuandlStockInfoBean> getIntance(final String authToken) {

        synchronized (QuandlSearch.class) {
            if (QuandlSearch.instance == null) {
                QuandlSearch.instance = new QuandlSearch();
            }
        }
        QuandlSearch.instance.session = MyQuandlSession.getSession(authToken);
        return QuandlSearch.instance;
    }

    /**
     * privater Standardkonstruktor
     */
    private QuandlSearch() {
        // absichtlich leer
    }

    /**
     * @see de.frankmuenster.jameica.depot.finanzen.SearchWertpapier#search(java.lang.String)
     */
    @Override
    public List<QuandlStockInfoBean> search(final String name) {

        final List<QuandlStockInfoBean> list = new ArrayList<>();
        final SearchRequest sRequest = SearchRequest.Builder.ofQuery(name).withMaxPerPage(100).build();

        try {
            final SearchResult searchResult = this.session.search(sRequest);
            for (final MetaDataResult document : searchResult.getMetaDataResultList()) {
                // Datenbanken hinzufgen
                this.addDatabases(document, list);
            }
            Logger.debug("Usable codes: " + list.size() + " out of " + searchResult.getTotalDocuments()); //$NON-NLS-1$ //$NON-NLS-2$
        } catch (final QuandlRequestFailedException e) {
            // TODO: Fehler ermitteln (zu viele Requests...)
            Logger.info("Search request fails"); //$NON-NLS-1$
        }
        return list;
    }

    /**
     * Add the database as <code>QuandlStockInfoBean</code> to list of stocks.
     *
     * @param document
     *            <code>MetaDataResult</code> from the Quandl search
     * @param list
     *            to add the <code>QuandlStockInfoBean</code> into
     */
    private void addDatabases(final MetaDataResult document, final List<QuandlStockInfoBean> list) {

        try {
            // Nur freie Datenbank und Zeitreihen
            if (QuandlSearch.FALSE_TEXT.equals(document.getString(QuandlSearch.QUANDL_PREMIUM_ID))
                    && QuandlSearch.QUNADL_TYPE_TIME_SERIES.equals(document.getString(QuandlSearch.QUANDL_TYPE_ID))
                    && QuandlSearch.QUNADL_FREQUENCY_DAILY
                            .equals(document.getString(QuandlSearch.QUANDL_FREQUENCY_ID))) {
                final QuandlStockInfoBean bean = new QuandlStockInfoBean(); // NOPMD
                bean.setCode(document.getQuandlCode());
                bean.setExchange(document.getString("database_code")); //$NON-NLS-1$
                bean.setName(document.getString(QuandlSearch.QUNADL_NAME));
                bean.setDescription(document.getString("description")); //$NON-NLS-1$
                String dateStr = document.getString("refreshed_at"); //$NON-NLS-1$
                final OffsetDateTime offset = OffsetDateTime.parse(dateStr);
                bean.setRefeshedAt(offset);
                dateStr = document.getString("newest_available_date"); //$NON-NLS-1$
                LocalDate date = LocalDate.parse(dateStr);
                bean.setNewestAvailable(date);
                dateStr = document.getString("oldest_available_date"); //$NON-NLS-1$
                date = LocalDate.parse(dateStr);
                bean.setOldestAvailable(date);
                list.add(bean);
            }
        } catch (final RuntimeException e) { // NOPMD
            // Loggen und einfach weiter machen
            Logger.error("Name: " + document.getString(QuandlSearch.QUNADL_NAME) + //$NON-NLS-1$
                    ". " + e.getLocalizedMessage(), e); //$NON-NLS-1$
        }

    }
}
