/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.action;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Aktion zum L&ouml;schen einer Order.
 *
 * @author Frank Mnster
 */
public class OrderDeleteAction implements Action {

    /**
     * Die Listener fr einen QrCodeFound-Event
     */
    private final transient Set<OrderDeleteListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Konstruktor mit Listener
     *
     * @param listener
     *            der OrderDeleteListener
     */
    public OrderDeleteAction(final OrderDeleteListener listener) {
	this.addListener(listener);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final OrderDeleteListener listener) {

	this.listeners.add(listener);
    }

    /**
     *
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

	// check if the context is a project
	if (!(context instanceof OrderHistory)) {
	    throw new ApplicationException(I18N.tr("OrderDeleteAction.chooseOrder")); //$NON-NLS-1$
	}
	final OrderHistory order = (OrderHistory) context;
	try {
	    // before deleting the order, we show up a confirm dialog ;)
	    final String question = I18N.tr("OrderDeleteAction.reallyDelete"); //$NON-NLS-1$
	    if (!Application.getCallback().askUser(question)) {
		return;
	    }
	    order.delete();
	    // Send Status update message
	    Application.getMessagingFactory()
		    .sendMessage(new StatusBarMessage(I18N.tr("OrderDeleteAction.deleteSuccessful"), //$NON-NLS-1$
			    StatusBarMessage.TYPE_SUCCESS));
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("OrderDeleteAction.deleteError"), e); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(I18N.tr("OrderDeleteAction.deleteError")) //$NON-NLS-1$
		    .initCause(e);
	}
	this.notifyListeners();
    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final OrderDeleteListener listener) {

	this.listeners.remove(listener);
    }

    /**
     * Benachrichtigen der registrieretn Listeners mit bergabe des gefundenen
     * Strings
     */
    private void notifyListeners() {

	for (final OrderDeleteListener listener : this.listeners) {
	    listener.notifyOrderDeleted();
	}
    }
}
