/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.action;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Aktualisiert die Wertpapiere
 *
 * @author Frank Mnster
 */
public class WertpapierAktualisiernAction implements Action {

    /**
     * Die Listener
     */
    private final transient Set<WertpapierAktualisiertListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Standard Konstruktor
     */
    public WertpapierAktualisiernAction() {
        // absichtlich leer
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final WertpapierAktualisiertListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final WertpapierAktualisiertListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen
     * Strings
     */
    private void notifyListeners() {

        for (final WertpapierAktualisiertListener listener : this.listeners) {
            listener.notifyWertpapiereAktualisiert();
        }
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

        try {
            if (context instanceof Wertpapier) {
                final Wertpapier wertpapier = (Wertpapier) context;
                this.reloadWertpapierFromWeb(wertpapier);
            } else {
                this.reloadAlleWerteFromWeb();
            }

            this.notifyListeners();
        } catch (final RemoteException e) {
            Logger.error(e.getMessage());
            throw new ApplicationException(I18N.tr("WertpapierAktualisiernAction.updateError"), e); //$NON-NLS-1$
        }
    }

    private boolean reloadWertpapierFromWeb(final Wertpapier wertpapier) {

        boolean isLoadOK = false;
        String symbol = null;

        try {
            symbol = wertpapier.getArdId();
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("WertpapierAktualisiernAction.symbolNotFound")); //$NON-NLS-1$
            if (Logger.isLogging(Level.DEBUG)) {
                Logger.debug(e.getMessage());
            }
        }
        if (StringUtils.isNotEmpty(symbol)) {
            try {

                ArdStockInfoBean wpInfoBean = new ArdStockInfoBean(symbol);
                wpInfoBean = BoerseArdApi.getInstance().getActualValue(wpInfoBean);

                if (wpInfoBean != null) {
                    wertpapier.parseWertpapier(wpInfoBean);
                    wertpapier.store();
                    Logger.debug(I18N.tr("WertpapierAktualisiernAction.success", symbol)); //$NON-NLS-1$
                    isLoadOK = true;
                }
            } catch (final ApplicationException e) {
                Logger.error(I18N.tr("WertpapierAktualisiernAction.storeError", //$NON-NLS-1$
                        symbol));
                Logger.debug(e.getMessage());
            } catch (final FinanceException e) {
                Logger.debug(e.getMessage());
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("WertpapierAktualisiernAction.remoteError", symbol)); //$NON-NLS-1$
                Logger.debug(e.getMessage());
            }
        }
        return isLoadOK;
    }

    private void reloadAlleWerteFromWeb() throws RemoteException {

        JaDepotDBService service;

        service = (JaDepotDBService) Settings.getDBService();
        final DBIterator<Wertpapier> list = service.createList(Wertpapier.class);

        final ArrayList<String> errMessage = new ArrayList<>();

        while (list.hasNext()) {
            final Wertpapier wertpapier = list.next();
            String ardId;
            try {
                ardId = wertpapier.getArdId();
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("WertpapierAktualisiernAction.symbolNotFound")); //$NON-NLS-1$
                errMessage.add(I18N.tr("WertpapierAktualisiernAction.symbolNotFoundMsg")); //$NON-NLS-1$
                continue;
            }
            if (!this.reloadWertpapierFromWeb(wertpapier)) {
                errMessage.add(ardId);
            }
        }
        if (errMessage.isEmpty()) {
            final String msg = I18N.tr("WertpapierAktualisiernAction.successful"); //$NON-NLS-1$
            Application.getMessagingFactory().sendMessage(new StatusBarMessage(msg, StatusBarMessage.TYPE_SUCCESS));
        } else {
            final StringBuilder msg = new StringBuilder(I18N.tr("WertpapierAktualisiernAction.stocks")); //$NON-NLS-1$
            msg.append(errMessage.get(0));
            for (int i = 1; i < errMessage.size(); i++) {
                msg.append(',').append(errMessage.get(i));
            }
            msg.append(I18N.tr("WertpapierAktualisiernAction.notFound")); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
        }
    }
}
