/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.controller;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.util.ApplicationException;

/**
 * A generic search dialog to search for a <code>StockInfoBean</code> in a
 * <code>AbstractStockSearchPart</code>.
 *
 * @author Frank Mnster
 * @param <I>
 * @param <S>
 */
public class SearchDialogController<I extends StockInfoBean, S extends AbstractStockSearchPart<I>>
	implements SearchResultSelectListener<I> {

    /**
     * Der Bescheibungstext
     */
    private FormTextPart description;

    /**
     * Das QuandlCode Input
     */
    private TextInput resultInput;

    /**
     * der Suchtext
     */
    private final String searchText;

    /**
     * Der InfoBean
     */
    private I bean;

    /**
     * Suchfeld Klasse
     */
    private final Class<S> searchPartClazz;

    /**
     * Suchfeld
     */
    private S searchPart;

    /**
     * Standardkonstruktor
     *
     * @param searchText
     * @param searchPartClazz
     */
    public SearchDialogController(final String searchText, final Class<S> searchPartClazz) {
	this.searchText = searchText;
	this.searchPartClazz = searchPartClazz;
    }

    /**
     * Liefert den Bereich, in der die Beschreibung angezeigt wird.
     *
     * @return FormTextPart fr die Beschreibung
     */
    public FormTextPart getDescription() {

	if (this.description == null) {
	    this.description = new FormTextPart();
	}
	this.description = new FormTextPart("<form><p></p><p></p><p></p></form>"); //$NON-NLS-1$
	return this.description;
    }

    /**
     * Liefert eine TextInput fr die Ausgabe
     *
     * @return resultInput fr die Ausgabe
     * @throws RemoteException
     */
    public TextInput getResultInput() throws RemoteException {

	if (this.resultInput == null) {
	    this.resultInput = new TextInput(this.searchText == null ? StringUtils.EMPTY : this.searchText);
	}
	return this.resultInput;
    }

    /**
     * Liefert das SearchPart
     *
     * @return das SearchPart
     * @throws ApplicationException
     */
    @SuppressWarnings("unchecked")
    public S getSearchPart() throws ApplicationException {

	if (this.searchPart == null) {
	    try {
		final Constructor<?> cons = this.searchPartClazz.getConstructor(SearchResultSelectListener.class);
		final Object instance = cons.newInstance(this);
		if (instance instanceof AbstractStockSearchPart<?>) {
		    this.searchPart = (S) cons.newInstance(this);
		} else {
		    throw new InstantiationException(I18N.tr("SearchDialogController.wrongInstanceType")); //$NON-NLS-1$
		}
	    } catch (InstantiationException | IllegalAccessException | NoSuchMethodException | SecurityException
		    | IllegalArgumentException | InvocationTargetException e) {
		throw new ApplicationException(I18N.tr("SearchDialogController.errorCreatingSearchFieldInstance"), e); //$NON-NLS-1$
	    }
	}
	return this.searchPart;
    }

    /**
     * Liefert das ausgewhlte Wertpapier.
     *
     * @return das ausgewhlte Wertpapier
     */
    public I getSelectedBean() {

	return this.bean;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(StockInfoBean)
     */
    @Override
    public void searchResultSelected(final I stockInfo) {

	this.bean = stockInfo;
	if (this.bean != null) {
	    this.resultInput.setValue(stockInfo.getName());
	    final StringBuilder text = new StringBuilder("<form><p>"); //$NON-NLS-1$
	    text.append(stockInfo.getDescription()).append("</p></form>"); //$NON-NLS-1$
	    try {
		this.description.setText(text.toString());
	    } catch (final IllegalArgumentException e) {
		this.description.setText(stockInfo.getDescription());
	    }
	    this.description.refresh();
	}
    }
}
