/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.input;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.Font;

/**
 * A multi input providing 6 buttons to choose from periods 1 week, 1, 3 or 6
 * month or 1, 2, 5 years.
 *
 * @author Frank Mnster
 *
 */
public class PeriodChooserInput implements Input {

    /**
     * Context-Parameter f&uuml;r die Wichtung der Breite.
     * (jameica.multiinput.weight)
     */
    private static final String DATA_WEIGHT = "jameica.multiinput.weight"; //$NON-NLS-1$

    /**
     * Gruppen ID f&uuml;r die Radio-Buttons (periodChooser)
     */
    public static final String GROUP_ID = "periodChooser"; //$NON-NLS-1$

    /**
     * Tooltip key f&uuml;r Dat Feld
     */
    private static final String TOOLTIP = "tooltip"; //$NON-NLS-1$

    /**
     * Periodenwert f&uuml;r ein Woche (7)
     */
    public static final int PERIOD_1W = 7;

    /**
     * Periodenwert f&uuml;r einen Monat (30)
     */
    public static final int PERIOD_1M = 30;

    /**
     * Periodenwert f&uuml;r 3 Monate (90)
     */
    public static final int PERIOD_3M = 90;

    /**
     * Periodenwert f&uuml;r 6 Monate (180)
     */
    public static final int PERIOD_6M = 180;

    /**
     * Periodenwert f&uuml;r ein Jahr (365)
     */
    public static final int PERIOD_1J = 365;

    /**
     * Periodenwert f&uuml;r 2 Jaher (730)
     */
    public static final int PERIOD_2J = 730;

    /**
     * Periodenwert f&uuml;r 5 Jahr5 (1825)
     */
    public static final int PERIOD_5J = 1825;

    /**
     * Map f&uuml; belibige Nutzer Daten
     */
    private final Map<String, Object> data = new ConcurrentHashMap<>();

    /**
     * Liste aller Input-Objeckte
     */
    private final transient List<Input> inputs = new ArrayList<>();

    /**
     * Unser Composite auf dem wir malen.
     */
    private transient Composite composite;

    /**
     * Die Labelbezeichung
     */
    private String name;

    /**
     * Kommentartext
     */
    private transient String comment;

    /**
     * Konstruktor
     *
     */
    public PeriodChooserInput() {

	RadioInput radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_1M);
	radioInput.setName(I18N.tr("PeriodChooserInput.oneMonthName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.oneMonthTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

	radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_3M);
	radioInput.setName(I18N.tr("PeriodChooserInput.threeMonthName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.threeMonthTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

	radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_6M);
	radioInput.setName(I18N.tr("PeriodChooserInput.sixMonthName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.sixMonthTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

	radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_1J);
	radioInput.setName(I18N.tr("PeriodChooserInput.oneYearName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.oneYearTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

	radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_2J);
	radioInput.setName(I18N.tr("PeriodChooserInput.twoYearName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.twoYearTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

	radioInput = new RadioInput(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_5J);
	radioInput.setName(I18N.tr("PeriodChooserInput.fiveYearName")); //$NON-NLS-1$
	radioInput.setData(PeriodChooserInput.TOOLTIP, I18N.tr("PeriodChooserInput.fiveYearTooltip")); //$NON-NLS-1$
	this.inputs.add(radioInput);

    }

    /**
     * @see Input#addListener(Listener)
     */
    @Override
    public void addListener(final Listener listener) {
	for (final Input input : this.inputs) {
	    input.addListener(listener);
	}
    }

    /**
     * @see Input#disable()
     */
    @Override
    public void disable() {
	this.setEnabled(false);
    }

    /**
     * @see Input#enable()
     */
    @Override
    public void enable() {
	this.setEnabled(true);
    }

    /**
     * @see Input#focus()
     */
    @Override
    public void focus() {
	if (!this.inputs.isEmpty()) {
	    this.inputs.get(0).focus();
	}
    }

    /**
     * @see Input#getControl()
     */
    @Override
    public Control getControl() {
	return this.composite;
    }

    /**
     * @see Input#getData(String)
     */
    @Override
    public Object getData(final String key) {
	return this.data.get(key);
    }

    /**
     * @see Input#getName()
     */
    @Override
    public String getName() {
	return this.name;
    }

    /**
     * @see Input#getValue()
     */
    @Override
    public List<Object> getValue() {
	// Wir liefern eine Liste aller Ergebnisse zurueck
	final List<Object> values = new ArrayList<>();
	for (final Input input : this.inputs) {
	    values.add(input.getValue());
	}
	return values;
    }

    /**
     * @see Input#hasChanged()
     */
    @Override
    public boolean hasChanged() {
	boolean hasChanged = false;

	for (final Input input : this.inputs) {
	    if (input.hasChanged()) {
		hasChanged = true;
		break;
	    }
	}
	return hasChanged;
    }

    /**
     * @see Input#isEnabled()
     */
    @Override
    public boolean isEnabled() {

	boolean isenabled = true;
	for (final Input input : this.inputs) {
	    if (!input.isEnabled()) {
		isenabled = false;
		break;
	    }
	}
	return isenabled;
    }

    /**
     * @see Input#isMandatory()
     */
    @Override
    public boolean isMandatory() {
	boolean isMandatory = false;
	for (final Input input : this.inputs) {
	    if (input.isMandatory()) {
		isMandatory = true;
		break;
	    }
	}
	return isMandatory;
    }

    /**
     * @see Input#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) {
	this.paint(parent, 250);

    }

    /**
     * @see Input#paint(Composite, int)
     */
    @Override
    public void paint(final Composite parent, final int width) {

	final Font small = Font.SMALL;

	int size = this.inputs.size();
	size++;

	this.composite = new Composite(parent, SWT.NONE);
	final GridLayout layout = new GridLayout(size, false);
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	layout.horizontalSpacing = 2;
	layout.verticalSpacing = 0;
	this.composite.setLayout(layout);
	final GridData gridDatad = new GridData(GridData.FILL_HORIZONTAL);
	gridDatad.widthHint = width;
	this.composite.setLayoutData(gridDatad);

	for (final Input input : this.inputs) {

	    // Checken, ob wir einen Breiten-Hint haben
	    final Number weightHint = (Number) input.getData(PeriodChooserInput.DATA_WEIGHT);
	    final int len = width / size + 0;
	    if (weightHint == null) {
		input.paint(this.composite, len);
	    } else {
		input.paint(this.composite, (int) (len * weightHint.doubleValue()));
	    }
	    input.getControl().setToolTipText(I18N.tr("PeriodChooserInput.show", //$NON-NLS-1$
		    (String) input.getData(PeriodChooserInput.TOOLTIP)));
	}

	// den Kommentar hinten dran fuegen
	final Label commentLabel = GUI.getStyleFactory().createLabel(this.composite, SWT.NONE);
	if (this.comment == null) {
	    // Einfach noch einen leeren Kommentar anhngen, dass sich
	    // ausbreiten kann
	    commentLabel.setText(""); //$NON-NLS-1$
	    commentLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	} else {
	    commentLabel.setFont(small.getSWTFont());
	    commentLabel.setText(this.comment);
	    commentLabel.setForeground(Color.COMMENT.getSWTColor());
	    commentLabel.setAlignment(SWT.LEFT);
	    commentLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	}
	RadioInput.select(PeriodChooserInput.GROUP_ID, PeriodChooserInput.PERIOD_1J);
    }

    /**
     * @see Input#setComment(String)
     */
    @Override
    public void setComment(final String comment) {
	this.comment = comment;
    }

    /**
     * @see Input#setData(String, Object)
     */
    @Override
    public void setData(final String key, final Object data) {
	this.data.put(key, data);

    }

    /**
     * @see Input#setEnabled(boolean)
     */
    @Override
    public void setEnabled(final boolean enabled) {
	for (final Input input : this.inputs) {
	    input.setEnabled(enabled);
	}
    }

    /**
     * @see Input#setMandatory(boolean)
     */
    @Override
    public void setMandatory(final boolean mandatory) {
	for (final Input input : this.inputs) {
	    input.setMandatory(mandatory);
	}
    }

    /**
     * @see Input#setName(String)
     */
    @Override
    public void setName(final String name) {
	this.name = name;

    }

    /**
     * Setzt den PeriodChooser auf die angegebene Periode
     *
     * @param period
     *            auf die derChooser gesetzt werden soll.
     */
    public void setPeriod(final int period) {
	if (period == PeriodChooserInput.PERIOD_1M || period == PeriodChooserInput.PERIOD_3M
		|| period == PeriodChooserInput.PERIOD_6M || period == PeriodChooserInput.PERIOD_1J
		|| period == PeriodChooserInput.PERIOD_2J || period == PeriodChooserInput.PERIOD_5J) {
	    RadioInput.select(PeriodChooserInput.GROUP_ID, period);
	}
    }

    /**
     * @see Input#setValue(Object)
     */
    @Override
    public void setValue(final Object value) {
	// Hier machen wir nix...
    }

}
