/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.input;

import java.time.LocalDate;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;
import org.mihalis.opal.rangeSlider.RangeSlider;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.input.AbstractInput;

/**
 * Ein Schieber mit 2 Markern zur Auswahl eines Zeitbereiches.
 *
 * @author Frank Mnster
 */
public class ZeitauswahlSchieberInput extends AbstractInput {

    /**
     * Haben wir den Fokus
     */
    private transient Boolean hasFocus = false;

    /**
     * Sind wir Enabled
     */
    private boolean enabled = true;

    /**
     * Minimaler Wert
     */
    private transient int minimun;

    /**
     * Maximaler Wert. Standard 1 Jahr (365 Tage).
     */
    private transient int maximum = 365;

    /**
     * Position des unteren Schiebers
     */
    private int lower;

    /**
     * Position des oberen Schiebers
     */
    private int upper;

    /**
     * Der RangeSlider
     */
    private transient RangeSlider slider;

    /**
     * Die Listener fr ein ZeitauswahlSchieberInputListener-Event
     */
    private final transient Set<ZeitauswahlSchieberInputListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Default-Konstruktor mit default Werten.
     */
    public ZeitauswahlSchieberInput() {
	this(DateUtils.subtractDays(365), LocalDate.now());
    }

    /**
     * Default-Konstruktor.
     *
     * @param lower
     *            unteres Datum.
     * @param upper
     *            oberes Datum.
     */
    public ZeitauswahlSchieberInput(final LocalDate lower, final LocalDate upper) {

	super();
	setLower(lower);
	setUpper(upper);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final ZeitauswahlSchieberInputListener listener) {
	this.listeners.add(listener);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#disable()
     */
    @Override
    public void disable() {
	this.setEnabled(false);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#enable()
     */
    @Override
    public void enable() {
	this.setEnabled(true);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#focus()
     */
    @Override
    public void focus() {
	this.hasFocus = true;
	if (this.slider != null && !this.slider.isDisposed()) {
	    this.slider.setFocus();
	}
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#getControl()
     */
    @Override
    public Control getControl() {

	if (this.slider == null || this.slider.isDisposed()) {

	    this.slider = new RangeSlider(getParent(), SWT.HORIZONTAL);
	    final GridData gridData = new GridData(GridData.FILL, GridData.CENTER, true, false, 1, 2);
	    gridData.widthHint = 250;
	    this.slider.setLayoutData(gridData);
	    this.slider.setMinimum(this.minimun);
	    this.slider.setMaximum(this.maximum);
	    this.slider.setUpperValue(this.upper);
	    this.slider.setLowerValue(this.lower);

	    final LocalDate fromDate = getLower();
	    final LocalDate toDate = getUpper();
	    final String lString = Settings.DATE_FORMATTER.format(fromDate);
	    final String uString = Settings.DATE_FORMATTER.format(toDate);
	    final int tage = (int) DateUtils.dateDiff(fromDate, toDate);

	    setComment(createCommentText(lString, uString, tage));

	    final ZeitauswahlSchieberInputChangedListener changed = new ZeitauswahlSchieberInputChangedListener(this);

	    this.slider.addSelectionListener(changed);
	    this.slider.addMouseMoveListener(changed);
	    this.slider.addKeyListener(changed);

	    if (this.hasFocus) {
		this.slider.setFocus();
	    }
	}
	return this.slider;

    }

    /**
     * @return lower
     */
    public LocalDate getLower() {
	return DateUtils.subtractDays(this.maximum - this.lower);
    }

    /**
     * Liefert den maximalen Wert des RangeSliders.
     *
     * @return dem Maximalwert
     */
    public int getMaximun() {
	return this.maximum;
    }

    /**
     * Liefert den minimalen Wert des RangeSliders.
     *
     * @return dem Minimalwert
     */
    public int getMinimum() {
	return this.minimun;
    }

    /**
     * @return upper
     */
    public LocalDate getUpper() {
	return DateUtils.subtractDays(this.maximum - this.upper);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#getValue()
     */
    @Override
    public Object getValue() {
	return new int[] { this.lower, this.upper };
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#isEnabled()
     */
    @Override
    public boolean isEnabled() {
	return this.enabled;
    }

    /**
     * Benachrichtigen der registrieretn Listener
     */
    public final void notifyListeners() {
	// erst mal unseren Kommentar aktualisieren
	updateComment();
	for (final ZeitauswahlSchieberInputListener listener : this.listeners) {
	    listener.rangeChanged();
	}
    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final ZeitauswahlSchieberInputListener listener) {
	this.listeners.remove(listener);
    }

    /**
     * Auswahl des letzten Jahren (sprich 365 Tage)
     */
    public void selectLastYear() {
	this.lower = this.maximum > 365 ? this.maximum - 365 : 0;
	this.upper = this.maximum;

	if (this.slider != null && !this.slider.isDisposed()) {
	    this.slider.setLowerValue(this.lower);
	    this.slider.setUpperValue(this.maximum);

	    final LocalDate fromDate = getLower();
	    final LocalDate toDate = getUpper();

	    final String lString = Settings.DATEFORMAT.format(fromDate);
	    final String uString = Settings.DATEFORMAT.format(toDate);

	    final int tage = (int) DateUtils.dateDiff(fromDate, toDate);

	    setComment(createCommentText(lString, uString, tage));
	    notifyListeners();
	}

    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
     */
    @Override
    public void setEnabled(final boolean enabled) {
	this.enabled = enabled;
	if (this.slider != null && !this.slider.isDisposed()) {
	    this.slider.setEnabled(this.enabled);
	}

    }

    /**
     * @param lower
     *            das zu setzende Objekt lower
     */
    public void setLower(final int lower) {
	this.lower = lower > this.minimun && lower < this.upper ? lower : 0;
	if (this.slider != null && this.lower != this.slider.getLowerValue()) {
	    this.slider.setLowerValue(this.lower);
	    updateComment();
	}
    }

    /**
     * @param lower
     *            das zu setzende Objekt lower
     */
    public final void setLower(final LocalDate lower) {
	final int low = (int) DateUtils.dateDiff(lower);
	setLower(low);
    }

    /**
     * Setzt die Werte f&Uuml;r Skalierung und Schrittweite.
     *
     * @param fromDate
     *            Minimum-Wert (Default: 0).
     * @param toDate
     *            Maximum-Wert (Default: 100).
     * @param increment
     *            Schrittweite (Default: 1).
     * @param pageIncrement
     *            seitenweise Schrittweite (Default: 10).
     */
    public void setScaling(final LocalDate fromDate, final LocalDate toDate, final int increment,
	    final int pageIncrement) {

	this.minimun = 0;
	this.maximum = (int) DateUtils.dateDiff(fromDate, toDate);
	if (this.slider != null && !this.slider.isDisposed()) {
	    this.slider.setMinimum(this.minimun);
	    this.slider.setMaximum(this.maximum);
	    this.slider.setIncrement(increment);
	    this.slider.setPageIncrement(pageIncrement);
	}
    }

    /**
     * @param upper
     *            das zu setzende Objekt upper
     */
    public void setUpper(final int upper) {
	this.upper = upper < this.maximum && upper > this.lower ? upper : this.maximum;
	if (this.slider != null && this.upper != this.slider.getUpperValue()) {
	    this.slider.setUpperValue(this.upper);
	    updateComment();
	}
    }

    /**
     * @param upper
     *            das zu setzende Objekt upper
     */
    public final void setUpper(final LocalDate upper) {
	final int upperDay = (int) DateUtils.dateDiff(upper);
	setUpper(upperDay);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
     */
    @Override
    public void setValue(final Object value) {
	// check if int array with two values
	if (value instanceof int[] && ((int[]) value).length == 2) {
	    // OK, zwei int Werte, dann versuchen wir es mal
	    this.setLower(((int[]) value)[0]);
	    this.setUpper(((int[]) value)[1]);
	}
    }

    /**
     * Erstellt den Kommentartext.
     *
     * @param lValue
     * @param uValue
     * @param days
     * @return den Text
     */
    private String createCommentText(final String lValue, final String uValue, final long days) {
	String comment;
	if (days == 1) { // NOPMD
	    comment = I18N.tr("ZeitauswahlSchieberInput.commentTextDays", lValue, uValue, //$NON-NLS-1$
		    Long.toString(days));
	} else {
	    comment = I18N.tr("ZeitauswahlSchieberInput.commentTextDay", lValue, uValue, //$NON-NLS-1$
		    Long.toString(days));
	}
	return comment;
    }

    private void updateComment() {

	final int lValue = this.slider.getLowerValue();
	final int uValue = this.slider.getUpperValue();

	final LocalDate fromDate = DateUtils.subtractDays(this.maximum - lValue);
	final LocalDate toDate = DateUtils.subtractDays(this.maximum - uValue);

	final String lString = Settings.DATE_FORMATTER.format(fromDate);
	final String uString = Settings.DATE_FORMATTER.format(toDate);
	final long tage = DateUtils.dateDiff(fromDate, toDate);

	setComment(createCommentText(lString, uString, tage));
    }

}
