/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.time.LocalDate;
import java.time.temporal.TemporalAccessor;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.analysegrafik.AnalyseGrafik;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.gui.util.ScrolledContainer;
import de.willuhn.jameica.gui.util.TabGroup;

/**
 * Zeit dem Kursverlauf als Diagramm. Bietet verschiedene Anzeigeoptionen, wie
 * DAX Overlay oder Kerzendiagramm an. Oder zeigt die Kursdaten in einer
 * Tabelle. Dazu werden dann noch Analysegrafiken angezeigt, falls vorhanden.
 *
 * @author Frank Mnster
 */
public class KursHistoryGrafikPart extends AbstractGrafik {

    /**
     * Der Diagramm Titel
     */
    private String titel;

    /**
     * Methoden add/remove Listener
     */
    private final MethodeAddRemoveListener listener;

    /**
     * Hier kommt die Grafik rein, falls zu wenig platz ist.
     */
    private ScrolledContainer scroll;

    /**
     * Die Analysediagramme, die auch noch angezeigt werden sollen
     */
    private AnalyseGrafik anaG;

    /**
     * Konstruktor mit Grafik Tietel und den zu benachrichtigenden Listner zum
     * entfernen von Analysediagrammen.
     *
     * @param titel
     * @param listener
     */
    public KursHistoryGrafikPart(final String titel, final MethodeAddRemoveListener listener) {
	super();
	this.titel = titel;
	this.listener = listener;
    }

    /**
     * @return titel
     */
    public String getTitel() {

	return this.titel;
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	// Wir initialisieren die Tabelle.
	final TablePart table = new TablePart(this.wpHistoryBean.getHistoryValues(), null);
	table.addColumn(I18N.tr("KursHistoryGrafikPart.dateCol"), "localDate", new Formatter() { //$NON-NLS-1$ //$NON-NLS-2$

	    /**
	     * Formatter fr Datumsanzeige
	     *
	     * @param object
	     *            (LocalDate) das zu formatierende Datum
	     */
	    @Override
	    public String format(final Object object) {

		String text = StringUtils.EMPTY;
		if (object instanceof LocalDate) {
		    synchronized (KursHistoryGrafikPart.class) {
			text = FormatterUtils.GERMAN_DATE_TIME_FORMATER.get().format((TemporalAccessor) object);
		    }
		}
		return text;
	    }
	});
	table.addColumn(I18N.tr("KursHistoryGrafikPart.openCol"), "open", new CurrencyFormatter( //$NON-NLS-1$ //$NON-NLS-2$
		StringUtils.EMPTY, FormatterUtils.DECIMAL_FORMAT_6.get()));
	table.addColumn(I18N.tr("KursHistoryGrafikPart.lowCol"), "low", new CurrencyFormatter(null, //$NON-NLS-1$ //$NON-NLS-2$
		FormatterUtils.DECIMAL_FORMAT_6.get()));
	table.addColumn(I18N.tr("KursHistoryGrafikPart.highCol"), "high", new CurrencyFormatter(null, //$NON-NLS-1$ //$NON-NLS-2$
		FormatterUtils.DECIMAL_FORMAT_6.get()));
	table.addColumn(I18N.tr("KursHistoryGrafikPart.closeCol"), "close", new CurrencyFormatter( //$NON-NLS-1$ //$NON-NLS-2$
		null, FormatterUtils.DECIMAL_FORMAT_6.get()));
	table.addColumn(I18N.tr("KursHistoryGrafikPart.volumeCol"), "volume"); //$NON-NLS-1$ //$NON-NLS-2$
	table.addColumn(I18N.tr("KursHistoryGrafikPart.adjustedCol"), "adj-closed", //$NON-NLS-1$ //$NON-NLS-2$
		new CurrencyFormatter(null, FormatterUtils.DECIMAL_FORMAT_6.get()));
	table.setRememberOrder(true);
	table.setRememberColWidths(true);
	final TabFolder folder = new TabFolder(parent, SWT.NONE);
	folder.setLayoutData(new GridData(GridData.FILL_BOTH));
	final TabGroup grafiken = new TabGroup(folder, I18N.tr("KursHistoryGrafikPart.chartTab"), //$NON-NLS-1$
		false, 1);
	final GridLayout gridLayout = new GridLayout(1, false);
	grafiken.getComposite().setLayout(gridLayout);
	final TabGroup tabGroup = new TabGroup(folder, I18N.tr("KursHistoryGrafikPart.tableTab")); //$NON-NLS-1$
	table.paint(tabGroup.getComposite());
	this.scroll = new ScrolledContainer(grafiken.getComposite(), 1);
	this.anaG = new AnalyseGrafik(this.listener);
	this.scroll.addPart(this.anaG);
	this.anaG.setTitle(this.titel);
	this.anaG.setKursDaxDaten(this.wpHistoryBean, this.daxHistory);
	this.anaG.setRange(this.displayPeriod);
	this.refresh();
    }

    /**
     * Aktualisiert die Anzeige
     */
    public void refresh() {

	this.anaG.setShowDax(this.showDax);
	this.anaG.setShowAsCandleStick(this.showCandle);
	this.anaG.reload();
	this.scroll.update();
    }

    /**
     * @see AbstractGrafik#reload()
     */
    @Override
    public void reload() {
	// Nothing to here right now
    }

    /**
     * Sets the range to the main graphics as well as to all sub analysis
     * graphics.
     *
     * @see AbstractGrafik#setRange(DateRange)
     *
     */
    @Override
    public void setRange(final DateRange period) {

	super.setRange(period);
	if (this.anaG != null) {
	    this.anaG.setRange(period);
	}
    }

    /**
     * @param titel
     *            das zu setzende Objekt titel
     */
    public void setTitel(final String titel) {

	this.titel = titel;
    }

}
