/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.text.DecimalFormat;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.TableItem;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.gui.formatter.DecimalFormatter;
import de.frankmuenster.jameica.depot.gui.parts.column.OrderTotal;
import de.frankmuenster.jameica.depot.gui.parts.column.OrderTotalInStandartWaehrung;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.formatter.TableFormatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.logging.Logger;

/**
 * Ein Tabelle mit den Ordern eines Wertpapieres.
 *
 * @author Frank Mnster
 */
public class OrderListePart extends TablePart implements Part {

    /**
     * Pattern f&uuml;r die W&auml;hrungsdarstellung
     */
    private static final String PATTERN = "###,###,##0.00"; //$NON-NLS-1$

    /**
     * Konstruktor mit Angabe der Orderliste und der Action fr den Doppelklick.
     *
     * @param list
     * @param action
     * @throws RemoteException
     */
    public OrderListePart(final GenericIterator<?> list, final Action action) throws RemoteException {
        super(list, action);
        String waehrung = null;
        // suche Wertpapier
        if (list.hasNext()) {
            final GenericObject nextListObject = list.next();
            if (nextListObject instanceof OrderHistory) {
                waehrung = ((OrderHistory) nextListObject).getAktie().getCurrency();
            }
        }
        if (waehrung == null) {
            waehrung = "%"; //$NON-NLS-1$
        }
        final boolean showWaehrungsCol = !waehrung.equals(Settings.getDepotWaehrung());
        final DecimalFormat decimalFormat = new DecimalFormat(OrderListePart.PATTERN);
        final CurrencyFormatter currencyFormatter = new CurrencyFormatter(waehrung, decimalFormat);
        final DateFormatter dateFormatter = new DateFormatter();
        final DecimalFormatter decimalFormatter = new DecimalFormatter(FormatterUtils.DECIMAL_FORMAT_6.get());
        this.setRememberColWidths(true);
        this.setRememberOrder(true);
        this.setRememberState(true);
        this.setFormatter(new MyTableFormatter());
        this.addColumn(I18N.tr("OrderListePart.flagCol"), "orderAmount"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("OrderListePart.dateCol"), "orderDate", dateFormatter); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("OrderListePart.amountCol"), "orderAmount", decimalFormatter); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("OrderListePart.tradeValueCol"), "orderPrice", //$NON-NLS-1$ //$NON-NLS-2$
                currencyFormatter);
        if (showWaehrungsCol) {
            this.addColumn(new OrderTotalInStandartWaehrung());
        } else {
            this.addColumn(new OrderTotal());
        }
        this.addColumn(I18N.tr("OrderListePart.additionalCostsCol"), "additionalCosts", //$NON-NLS-1$ //$NON-NLS-2$
                currencyFormatter);
        if (showWaehrungsCol) {
            this.addColumn(I18N.tr("OrderListePart.inStandardCurrency", Settings.getDepotWaehrung()), //$NON-NLS-1$
                    "additionalCostsInMainCurrency"); //$NON-NLS-1$
        }
        this.addColumn(I18N.tr("OrderListePart.orderNumber"), "orderNumber"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderListePart.commentCol"), "comment"); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * Einen netten Formatierer f&uuml;r unser Tabelle.
     *
     * @author munstfr
     * @version 1.0
     */
    private class MyTableFormatter implements TableFormatter {

        /**
         * Standardkonstruktor
         */
        public MyTableFormatter() {
            // Absichtlich leer
        }

        /**
         * @see TableFormatter#format(TableItem)
         */
        @Override
        public void format(final TableItem item) {
            if (item.getData() instanceof OrderHistory) {
                final OrderHistory orderHistory = (OrderHistory) item.getData();
                try {
                    item.setText(0, StringUtils.EMPTY);
                    if (orderHistory.getOrderAmount().compareTo(BigDecimal.ZERO) < 0) {
                        item.setForeground(Color.SUCCESS.getSWTColor());
                        // Zusatzkosten nicht einfrben
                        item.setForeground(5, Color.FOREGROUND.getSWTColor());
                        item.setImage(0, SWTUtil.getImage("list-remove.png")); //$NON-NLS-1$
                    } else {
                        item.setForeground(Color.FOREGROUND.getSWTColor());
                        item.setImage(0, SWTUtil.getImage("list-add.png")); //$NON-NLS-1$
                    }
                } catch (final RemoteException e) {
                    Logger.error(I18N.tr("OrderListePart.formatingError"), e); //$NON-NLS-1$
                }
            }
        }
    }
}
