/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.sql.Time;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.controller.WertpapierDetailController;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Liefert ein Part, das Details zu einem Wertpapier anzeigt. Dazu geh&ouml;ren
 * der Name, the ISIN, die ARD-ID, die Anzahl, das letzte Abfragedatum und der
 * Kurs, sowie die Gesamtsumme des Papieres im Depot. Ist das Papier nicht bei
 * der ARD gelistet, knnen Datum, Kurs und W&auml;hrung gesetzt werden.
 *
 * @author Frank Mnster
 */
public class WertpapierDetailsPart implements Part {

    /**
     * Veraltertes Handelsdatum in Millisekunden.
     */
    private static final long OLD_TRADE_DAYS = 7;

    /**
     * Veraltertes Handelsdatum Anzeigentext
     */
    private static final String OLD_TRADE_TEXT = I18N.tr("WertpapierDetailsPart.toOldLastTrade"); //$NON-NLS-1$

    /**
     * Pfeil nach oben Zeichen
     */
    private static final char ARROW_UP = '\u2191';

    /**
     * Pfeil nach unten Zeichen
     */
    private static final char ARROW_DOWN = '\u2193';

    /**
     * WertpapierDetail-Kontroller, damit auch Eingabefelder verwendet werden
     * k&ouml;nnen.
     */
    private final transient WertpapierDetailController control;

    /**
     * Das Wertpapier
     */
    private final transient Wertpapier wertpapier;

    /**
     * Der WP-Name
     */
    private transient LabelInputExtended nameLbl;

    /**
     * Die ARD ID
     */
    private transient LabelInputExtended lblArdId;

    /**
     * The ISIN of teh security
     */
    private transient LabelInputExtended isinLbl;

    /**
     * Die WP-St&uuml;ckanzahl
     */
    private transient LabelInputExtended anzahlLbl;

    /**
     * Das letzte Handelsdatum
     */
    private transient LabelInputExtended datumLbl;

    /**
     * Der letzte Handelswert
     */
    private transient LabelInputExtended letzterWertLbl;

    /**
     * Der Gesamtwert
     */
    private transient LabelInputExtended totalLbl;

    /**
     * Erstellt ein neues WertpapierDetailPart.
     *
     * @param wertpapier
     *            das Papier dessen Details angezeigt werden sollen.
     * @param control
     *            der WertpapierDeatils-View Kontrolleur
     */
    public WertpapierDetailsPart(final Wertpapier wertpapier, final WertpapierDetailController control) {

        this.control = control;
        this.wertpapier = wertpapier;
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        final ColumnLayout col = new ColumnLayout(parent, 2);

        final GridLayout layout = (GridLayout) col.getComposite().getLayout();
        layout.makeColumnsEqualWidth = true;

        if (this.wertpapier == null) {
            final SimpleContainer firstLineLeft = new SimpleContainer(col.getComposite(), false, 2);
            firstLineLeft.addText(I18N.tr("WertpapierDetailsPart.noStockInfoAvailable"), //$NON-NLS-1$
                    true);
        } else {
            // create containers
            final SimpleContainer firstLineLeft = new SimpleContainer(col.getComposite(), false, 2);
            final SimpleContainer firstLineRight = new SimpleContainer(col.getComposite(), false, 4);
            // add security name
            this.nameLbl = new LabelInputExtended(this.wertpapier.getName());
            this.nameLbl.setToolTip(this.wertpapier.getName());
            firstLineLeft.addLabelPair(I18N.tr("WertpapierDetailsPart.lblName"), this.nameLbl); //$NON-NLS-1$
            // add security ISIN
            this.isinLbl = new LabelInputExtended(this.wertpapier.getIsin());
            firstLineRight.addLabelPair(I18N.tr("WertpapierDetailsPart.lblIsin"), this.isinLbl); //$NON-NLS-1$

            // add ARD id
            this.lblArdId = new LabelInputExtended(this.wertpapier.getArdId());
            firstLineRight.addLabelPair(I18N.tr("WertpapierDetailsPart.lblArdId"), this.lblArdId); //$NON-NLS-1$

            this.anzahlLbl = new LabelInputExtended(
                    this.wertpapier.getAnzahl().setScale(6, RoundingMode.HALF_EVEN).toPlainString(),
                    I18N.tr("WertpapierDetailsPart.lblNumberStocks")); //$NON-NLS-1$
            if (this.wertpapier.isInArd()) {
                final SimpleContainer secondLine = new SimpleContainer(parent, false, 8);
                this.wpInArd(secondLine);
            } else {
                this.wpNotInArd(parent);
            }
        }
    }

    /**
     * Aktualisiert die Anzeige.
     */
    public void update() {

        try {
            this.nameLbl.setValue(this.wertpapier.getName());
            this.nameLbl.setToolTip(this.wertpapier.getName());
            this.lblArdId.setValue(this.wertpapier.getArdId());
            this.anzahlLbl.setValue(this.wertpapier.getAnzahl().setScale(6, RoundingMode.HALF_EVEN).toPlainString());
            this.isinLbl.setValue(this.wertpapier.getIsin());

            // Datum und LetzerHandelsWert nur wenn nicht als Inputs
            if (this.datumLbl != null) {
                this.datumLbl
                        .setValue(FormatterUtils.GERMAN_DATE_FORMAT.get().format(this.wertpapier.getLastTradeDate()));
                final LocalDate date = DateUtils.asLocalDate(this.wertpapier.getLastTradeDate());
                final Time time = this.wertpapier.getLastTradeTime();
                if (DateUtils.dateDiff(date) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
                    this.datumLbl.setColor(Color.ERROR);
                    this.datumLbl.setToolTip(WertpapierDetailsPart.OLD_TRADE_TEXT);
                    Application.getMessagingFactory().sendMessage(
                            new StatusBarMessage(WertpapierDetailsPart.OLD_TRADE_TEXT, StatusBarMessage.TYPE_INFO));
                } else {
                    this.datumLbl.setColor(Color.FOREGROUND);
                    if (time != null) {
                        this.datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT.get().format(time));
                    }
                }
            }
            if (this.letzterWertLbl != null) {
                this.letzterWertLbl
                        .setValue(this.wertpapier.getLastTradePriceOnly().setScale(2, RoundingMode.HALF_EVEN).toString()
                                + StringUtils.SPACE + this.wertpapier.getCurrency());

                final StringBuilder change = new StringBuilder(
                        FormatterUtils.DECIMAL_FORMAT.get().format(this.wertpapier.getChangeInPercentage()));
                if (this.wertpapier.getChangeInPercentage() < 0) {
                    change.append(WertpapierDetailsPart.ARROW_DOWN);
                    this.letzterWertLbl.setColor(Color.ERROR);
                } else {
                    change.append(WertpapierDetailsPart.ARROW_UP);
                    this.letzterWertLbl.setColor(Color.FOREGROUND);
                }
                this.letzterWertLbl.setComment(change + "%"); //$NON-NLS-1$
                this.letzterWertLbl.setToolTip(I18N.tr("WertpapierDetailsPart.lblValueTooltip")); //$NON-NLS-1$
            }

            this.totalLbl.setValue(this.wertpapier.getActualTotalValue().toString() + StringUtils.SPACE
                    + this.wertpapier.getCurrency());
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("WertpapierDetailsPart.refreshStockDeatilPartError") //$NON-NLS-1$
                    + e.getMessage());
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(I18N.tr("WertpapierDetailsPart.refreshStockDeatilPartErrorMsg"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_ERROR));
        }
    }

    /**
     * Zeichnet die WP Daten mit Ard Bresen Daten.
     *
     * @param container
     *            der schon erstellte Container.
     * @throws RemoteException
     */
    private void wpInArd(final Container container) throws RemoteException {

        if (this.wertpapier.getLastTradeDate() == null) {
            this.datumLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierDetailsPart.lblDatePleaseRefesh")); //$NON-NLS-1$
        } else {
            final LocalDateTime date = DateUtils.asLocalDateTime(this.wertpapier.getLastTradeDate());
            this.datumLbl = new LabelInputExtended(FormatterUtils.GERMAN_DATE_TIME_FORMATER.get().format(date));
            if (DateUtils.dateDiff(date) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
                this.datumLbl.setColor(Color.ERROR);
                this.datumLbl.setToolTip(WertpapierDetailsPart.OLD_TRADE_TEXT);
                Application.getMessagingFactory().sendMessage(
                        new StatusBarMessage(WertpapierDetailsPart.OLD_TRADE_TEXT, StatusBarMessage.TYPE_INFO));
            } else {
                // Uhrzeit anzeigen
                final Time time = this.wertpapier.getLastTradeTime();
                if (time != null) {
                    this.datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT.get().format(time));
                }
            }
        }
        container.addLabelPair(I18N.tr("WertpapierDetailsPart.lblDate"), this.datumLbl); //$NON-NLS-1$
        this.letzterWertLbl = new LabelInputExtended(
                this.wertpapier.getLastTradePriceOnly().setScale(2, RoundingMode.HALF_EVEN).toString()
                        + StringUtils.SPACE + this.wertpapier.getCurrency());
        // Platz machen fr die nderungsanzeige
        this.letzterWertLbl.setComment(StringUtils.EMPTY);
        container.addLabelPair(I18N.tr("WertpapierDetailsPart.lblValue"), this.letzterWertLbl); //$NON-NLS-1$
        // Anzahl hinzufgen
        container.addLabelPair(I18N.tr("WertpapierDetailsPart.lblCount"), this.anzahlLbl); //$NON-NLS-1$
        // Gesammtsumme
        this.totalLbl = new LabelInputExtended(
                this.wertpapier.getActualTotalValue().toString() + StringUtils.SPACE + this.wertpapier.getCurrency(),
                I18N.tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
        container.addLabelPair(I18N.tr("WertpapierDetailsPart.lblTotal"), this.totalLbl); //$NON-NLS-1$
    }

    /**
     * Zeichnet die WP Daten ohne ARD Brse.
     *
     * @param parent
     * @throws RemoteException
     */
    private void wpNotInArd(final Composite parent) throws RemoteException {

        final SimpleContainer notArd = new SimpleContainer(parent, false, 10);
        notArd.addLabelPair(I18N.tr("WertpapierDetailsPart.lblDate"), //$NON-NLS-1$
                this.control.getDatum());
        notArd.addLabelPair(I18N.tr("WertpapierDetailsPart.lblValue"), //$NON-NLS-1$
                this.control.getLetzterKursWert());
        notArd.addPart(this.control.getWaehrung());
        // Anzahl hinzufgen
        notArd.addLabelPair(I18N.tr("WertpapierDetailsPart.lblCount"), this.anzahlLbl); //$NON-NLS-1$
        // Gesamtsumme
        this.totalLbl = new LabelInputExtended(
                this.wertpapier.getActualTotalValue().setScale(2, RoundingMode.HALF_EVEN).toString() + StringUtils.SPACE
                        + this.wertpapier.getCurrency(),
                I18N.tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
        notArd.addLabelPair(I18N.tr("WertpapierDetailsPart.lblTotal"), this.totalLbl); //$NON-NLS-1$
        final StringBuilder message = new StringBuilder(I18N.tr("WertpapierDetailsPart.quoteNotinArd")); //$NON-NLS-1$
        final LocalDate datum = DateUtils.asLocalDate((Date) this.control.getDatum().getValue());
        if (this.control.getDatum().getValue() != null
                && DateUtils.dateDiff(datum) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
            message.append(" | ").append(WertpapierDetailsPart.OLD_TRADE_TEXT); //$NON-NLS-1$
        }
        Application.getMessagingFactory()
                .sendMessage(new StatusBarMessage(message.toString(), StatusBarMessage.TYPE_INFO));
    }
}
