/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.currate.CurRateConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.gui.formatter.DecimalFormatter;
import de.frankmuenster.jameica.depot.gui.menu.WertpapierListeMenu;
import de.frankmuenster.jameica.depot.gui.parts.column.ExchangeInfo;
import de.frankmuenster.jameica.depot.gui.parts.column.OrderTotalInStandartWaehrung;
import de.frankmuenster.jameica.depot.gui.parts.column.WertpapierInStandardWaehrung;
import de.frankmuenster.jameica.depot.gui.parts.column.ZusatzkostenInStandardWaehrung;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.currency.Currency;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.gui.parts.table.Feature;
import de.willuhn.jameica.gui.parts.table.Feature.Context;
import de.willuhn.jameica.gui.parts.table.FeatureSummary;
import de.willuhn.logging.Logger;

/**
 * @author munstfr
 */
public class WertpapierListePart extends TablePart implements Part {

    /**
     * W&auml;hrungspalte anzeigen
     */
    private transient Boolean showWaehrSpalte;

    /**
     * Splate f&uuml; die Standardw&auml;hrung ???
     */
    private transient WertpapierInStandardWaehrung wpInStdWaehrung;

    /**
     * Die Summe aller WP.
     */
    private transient BigDecimal gesamtSumme;

    /**
     * Liefert die sortierte Wertpapierliste. Das ist nur deshalb in einer extra
     * Funktion ausgegliedert, weil der Aufruf des super-Konstruktors zwingend
     * in der ersten Zeile erfolgen muss.
     *
     * @return Liste mit dem WP.
     * @throws RemoteException
     */
    private static DBIterator<Wertpapier> init() throws RemoteException {

        final JaDepotDBService service = (JaDepotDBService) Settings.getDBService();
        final DBIterator<Wertpapier> list = service.createList(Wertpapier.class);
        list.setOrder("ORDER BY name asc"); //$NON-NLS-1$
        return list;
    }

    /**
     * Konstruktor mit Angabe der Action beim Doppelklick.
     *
     * @param action
     * @throws RemoteException
     */
    public WertpapierListePart(final Action action) throws RemoteException {
        this(WertpapierListePart.init(), action);
    }

    /**
     * Konstruktor mit Angabe der Liste und der Action beim Doppelklick.
     *
     * @param list
     * @param action
     * @throws RemoteException
     */
    public WertpapierListePart(final DBIterator<Wertpapier> list, final Action action) throws RemoteException {
        super(list, action);
        // Berechnet die Gesamtsumme
        this.calculateSum();
        this.setRememberColWidths(true);
        this.setRememberOrder(true);
        this.setRememberState(true);
        // Whrungsspalte?
        this.showWaehrSpalte = this.showWaehrungsSpalte(Settings.getDepotWaehrung(), list);
        final DateFormatter daf = new DateFormatter();
        final DecimalFormatter def = new DecimalFormatter(FormatterUtils.DECIMAL_FORMAT_6.get());
        this.addColumn(I18N.tr("WertpapierListePart.colName"), "name"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.colIsin"), "isin"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.colSymbole"), "symbol"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.number"), "anzahl", def); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.tradeDate"), "lastTradeDate", daf); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.tradePrice"), "lastTradePriceOnly", //$NON-NLS-1$ //$NON-NLS-2$
                FormatterUtils.CURRENCY_FORMAT.get());
        this.addColumn(I18N.tr("WertpapierListePart.currency"), "currency"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("WertpapierListePart.value"), "gesamtWert", //$NON-NLS-1$ //$NON-NLS-2$
                FormatterUtils.CURRENCY_FORMAT.get());
        if (this.showWaehrSpalte) {
            this.wpInStdWaehrung = new WertpapierInStandardWaehrung();
            this.addColumn(this.wpInStdWaehrung);
        }
        // Zustzliche Kosten
        final ZusatzkostenInStandardWaehrung additionalCosts = new ZusatzkostenInStandardWaehrung(true);
        additionalCosts.setName(I18N.tr("WertpapierListePart.additionalCosts")); //$NON-NLS-1$
        this.addColumn(additionalCosts);
        // Gesamtkosten
        final OrderTotalInStandartWaehrung kaufKosten = new OrderTotalInStandartWaehrung();
        kaufKosten.setName(I18N.tr("WertpapierListePart.TotalCosts")); //$NON-NLS-1$
        this.addColumn(kaufKosten);
        // Kommentar
        this.addColumn(I18N.tr("WertpapierListePart.comment"), "comment"); //$NON-NLS-1$ //$NON-NLS-2$
        // Kontext Men hinzufgen
        this.setContextMenu(new WertpapierListeMenu());
    }

    /**
     * Liefert Umtauschraten von WP, die nicht in der Standardw&auml;hrung sind.
     *
     * @return Liste vom Typ <i>ExchangeInfo</i> mit den Umtauschraten.
     */
    public List<ExchangeInfo> getExchangeInfos() {

        return this.wpInStdWaehrung.getExchangeInfos();
    }

    /**
     * Liefert die Gesamtsumme aller WP.
     *
     * @return die Gesamtsumme
     */
    public BigDecimal getGesamtSumme() {

        this.calculateSum();
        return this.gesamtSumme;
    }

    /**
     * Liefert <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
     * sonst <i>false</i>.
     *
     * @return <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
     *         sonst <i>false</i>
     */
    public boolean isShowWaehrungsSpalte() {

        return this.showWaehrSpalte;
    }

    /**
     * @see TablePart#createFeatureEventContext(Feature.Event, Object )
     */
    @SuppressWarnings("unchecked")
    @Override
    protected Context createFeatureEventContext(final Feature.Event event, final Object data) {

        final Context ctx = super.createFeatureEventContext(event, data);
        final String summery = I18N.tr("WertpapierListePart.totalValue", //$NON-NLS-1$
                FormatterUtils.DECIMAL_FORMAT.get().format(this.gesamtSumme), Settings.CURRENCY);
        if (this.hasEvent(FeatureSummary.class, event)) {
            ctx.addon.put(FeatureSummary.CTX_KEY_TEXT, summery);
        }
        return ctx;
    }

    private void calculateSum() {

        BigDecimal sumUp = new BigDecimal("0.0"); //$NON-NLS-1$
        try {
            @SuppressWarnings("unchecked")
            final Iterator<Wertpapier> list = this.getItems().iterator();

            // Berechnet die Gesamtsumme aller Wertpapiere
            while (list.hasNext()) {
                final Wertpapier wertpapier = list.next();
                final BigDecimal stockValue = this.getStockValue(wertpapier);
                sumUp = sumUp.add(stockValue); // NOPMD
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("WertpapierListePart.calculateTotalSumFailed"), e); //$NON-NLS-1$
        }
        this.gesamtSumme = sumUp;
    }

    /**
     * Calculates the stock value in EUR. If the stock currency is not in EUR
     * the Currency is converter is used with the last available exchange rate.
     * If the an error occurs during enumerating the exchange rate the total
     * value is set to ZERO.
     *
     * @param wertpapier
     *            the stock
     * @return the total value in EUR
     * @throws RemoteException
     *             if some stock parameter could not be read from the database.
     */
    private BigDecimal getStockValue(final Wertpapier wertpapier) throws RemoteException {

        Logger.debug(I18N.tr("WertpapierListePart.logInfoStockValue", //$NON-NLS-1$
                wertpapier.getName(), wertpapier.getGesamtWert().toString()));

        BigDecimal gesamtWert;
        // skip if Index
        if (Currency.IDX.getCode().equals(wertpapier.getCurrency())) {
            gesamtWert = BigDecimal.ZERO;
        } else {
            gesamtWert = wertpapier.getGesamtWert();

            if (wertpapier.getCurrency() != null && !Currency.EUR.getCode().equals(wertpapier.getCurrency())) {
                try {
                    final ExchangeRate changeRateEur = CurRateConverter.getInstance()
                            .exchangeRate(wertpapier.getCurrency(), "EUR"); //$NON-NLS-1$
                    gesamtWert = gesamtWert.multiply(changeRateEur.getRate());
                } catch (final FinanceException e) {
                    gesamtWert = BigDecimal.ZERO;
                    Logger.error(I18N.tr("WertpapierListePart.getExchangeRateError", //$NON-NLS-1$
                            e.getMessage()));
                }
            }
        }
        return gesamtWert;
    }

    /**
     * Checked, ob die Currency-Spalte angezeigt werden muss.
     *
     * @param waehrung
     * @param list
     * @return <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
     *         sonst <i>false</i>
     * @throws RemoteException
     */
    private boolean showWaehrungsSpalte(final String waehrung, final DBIterator<Wertpapier> list)
            throws RemoteException {

        boolean mustShow = false;
        list.begin();
        while (list.hasNext()) {
            final Wertpapier wertpapier = list.next();
            if (!waehrung.equals(wertpapier.getCurrency())) {
                // Andere Whrung gefunden, also anzeigen.
                mustShow = true;
                break;
            }
        }
        list.begin();
        // Alles in einer Waehrung, also nicht anzeigen.
        return mustShow;
    }
}
