/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Part zum Anzeigen der Wertpapierperformanz. dazu geh&ouml;ren die Anzahl der
 * K&auml;ufe, Verk&auml;ufe, Zusatzkosten und die Ergebnissaufstellung
 * &uuml;ber alle Kosten und ertr&auml;ge.
 *
 * @author Frank Mnster
 *
 */
public class WertpapierPerformanzPart implements Part {

    /**
     * Nicht verf&uuml;gbar String
     */
    private static final String NOT_AVAILABLE = I18N.tr("WertpapierPerformanzPart.notAvailable"); //$NON-NLS-1$

    /**
     * Das Wertpapier des Performanz angezeigt werden soll
     */
    private final transient Wertpapier wertpapier;

    /**
     * Gesamtwert aller Papiere-Label
     */
    private transient LabelInputExtended gesamtWertLbl;

    /**
     * Kosten aller Verk&auml;ufe-Label
     */
    private transient LabelInputExtended verkaufLbl;

    /**
     * Einnahmen aller K&auml;ufe-Label
     */
    private transient LabelInputExtended kaufLbl;

    /**
     * Gesamtwert aller zus&auml;ztlichen Kosten-Label
     */
    private transient LabelInputExtended kostenLbl;

    /**
     * Summe aller Einnahmen/Kosten-Label
     */
    private transient LabelInputExtended ergebnisLbl;

    /**
     * Konstruktor mit Angabe des Wertpapiers.
     *
     * @param wertpapier
     */
    public WertpapierPerformanzPart(final Wertpapier wertpapier) {
        this.wertpapier = wertpapier;
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        final SimpleContainer simpleContainer = new SimpleContainer(parent, false, 2);

        if (this.wertpapier != null) {

            // Gesamtwert
            this.gesamtWertLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierPerformanzPart.lblgesamtWertTooltip")); //$NON-NLS-1$

            // Einnahmen aus Verkufen
            this.verkaufLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierPerformanzPart.lblVerkaufTooltip")); //$NON-NLS-1$
            this.verkaufLbl.setColor(Color.SUCCESS);

            // Ausgaben fr Kufe
            this.kaufLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierPerformanzPart.lblKaufTooltip")); //$NON-NLS-1$
            this.kaufLbl.setColor(Color.ERROR);

            // Summe aller zustzlichen Kosten
            this.kostenLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierPerformanzPart.lblKostenTooltip")); //$NON-NLS-1$
            this.kostenLbl.setColor(Color.ERROR);

            // Das Gesamtergbnis
            this.ergebnisLbl = new LabelInputExtended(StringUtils.EMPTY,
                    I18N.tr("WertpapierPerformanzPart.lblErgebnisTooltip")); //$NON-NLS-1$

            // Felder aktualisieren
            this.update();

            // Jetzt alle Felder in den Container einfgen
            simpleContainer.addLabelPair(I18N.tr("WertpapierPerformanzPart.lblgesamtWert"), //$NON-NLS-1$
                    this.gesamtWertLbl);
            simpleContainer.addLabelPair(I18N.tr("WertpapierPerformanzPart.lblVerkauf"), this.verkaufLbl); //$NON-NLS-1$
            simpleContainer.addLabelPair(I18N.tr("WertpapierPerformanzPart.lblKauf"), this.kaufLbl); //$NON-NLS-1$
            simpleContainer.addLabelPair(I18N.tr("WertpapierPerformanzPart.lblKosten"), this.kostenLbl); //$NON-NLS-1$
            simpleContainer.addLabelPair(I18N.tr("WertpapierPerformanzPart.lblErgebnis"), this.ergebnisLbl); //$NON-NLS-1$
        }

    }

    /**
     * Aktualisiert die Werte aus der Datenbank
     */
    public void update() {

        String currency;
        BigDecimal bestand = null;
        BigDecimal verkauf = null;
        BigDecimal kauf = null;
        BigDecimal zusatzKosten = null;
        try {
            currency = this.wertpapier.getCurrency();
        } catch (final RemoteException e) {
            // Hmm, keine Whrung, also nehmen wir mal Euro an
            currency = "EUR"; //$NON-NLS-1$
        }

        // Gesamtwert
        try {
            bestand = this.wertpapier.getGesamtWert();
            this.gesamtWertLbl.setValue(bestand.toString() + StringUtils.SPACE + currency);
        } catch (final RemoteException e) {
            this.gesamtWertLbl.setValue(WertpapierPerformanzPart.NOT_AVAILABLE);
        }

        // Einnahmen aus Verkufen
        try {
            verkauf = this.wertpapier.getVerkauf();
            this.verkaufLbl.setValue(verkauf.toString() + StringUtils.SPACE + currency);
        } catch (final RemoteException e) {
            this.verkaufLbl.setValue(WertpapierPerformanzPart.NOT_AVAILABLE);
        }

        // Ausgaben fr Kufe
        try {
            kauf = this.wertpapier.getKauf();
            this.kaufLbl.setValue(kauf.toString() + StringUtils.SPACE + currency);
        } catch (final RemoteException e) {
            this.kaufLbl.setValue(WertpapierPerformanzPart.NOT_AVAILABLE);
        }

        // Summe aller zustzlichen Kosten
        try {
            zusatzKosten = this.wertpapier.getAdditionalCosts();
            this.kostenLbl.setValue(zusatzKosten.toString() + StringUtils.SPACE + currency);
            this.kostenLbl.setColor(Color.ERROR);
        } catch (final RemoteException e) {
            this.kostenLbl.setValue(WertpapierPerformanzPart.NOT_AVAILABLE);
        }

        if (bestand == null || verkauf == null || kauf == null || zusatzKosten == null) {
            this.ergebnisLbl.setValue(WertpapierPerformanzPart.NOT_AVAILABLE);
            this.ergebnisLbl.setColor(Color.BACKGROUND);
        } else {
            final BigDecimal erg = bestand.add(verkauf).subtract(kauf.add(zusatzKosten)).setScale(2,
                    RoundingMode.HALF_EVEN);
            this.ergebnisLbl.setValue(erg + StringUtils.SPACE + currency);
            if (erg.compareTo(BigDecimal.ZERO) < 0) {
                this.ergebnisLbl.setColor(Color.ERROR);
            } else {
                this.ergebnisLbl.setColor(Color.SUCCESS);
            }
        }
    }
}
