/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.column;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.currate.CurRateConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.CurrencyConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.Column;
import de.willuhn.logging.Logger;

/**
 * Klasse die eine Spalte mit Betr&auml;gen in der Standartw&auml;hrung
 * erstellt. Dabei kann entweder das aktuelle Datum oder das Datum der Order
 * genommen werden.
 *
 * @author Frank Mnster
 */
public abstract class AbstractColumnInStandardWaehrung extends Column {

    /**
     * Unser serial UID
     */
    private static final long serialVersionUID = -5340194795194610509L;

    /**
     * Eine Liste mit Umtauschraten der Wertpapiere, die nicht in der
     * Standardw&auml;hrung sind.
     */
    private final transient List<ExchangeInfo> exchangeInfos = new ArrayList<>();

    /**
     * Soll das Datum der Order benutzt werden, um die Umtauschrate zu
     * ermitteln. Falls nein, wird die aktuellste Umtauschrate verwendet.
     */
    private transient boolean useOrderDate;

    /**
     * Der W&auml;hrungsumrechner
     */
    private final transient CurrencyConverter ccf = CurRateConverter.getInstance();

    /**
     * Kleine Funktion die den Titel ein f&uuml;r Kurswertspalte mit W&ahrung
     * erzeugt.
     *
     * @return den Spaltentitel
     */
    private static String getStandardWaehrung() {

        return I18N.tr("_in") + Settings.getDepotWaehrung(); //$NON-NLS-1$
    }

    /**
     * Konstruktor mit Angabe eines Formatters und dem useOrderDate Value. Mit
     * der oderDate kann angegeben werden, ob die Umtauschrate zum Zeitpunkt der
     * Order oder die aktuellste zur Verf&uuml;gung stehendes angezeigt werden
     * soll.
     *
     * @param useOrderDate
     * @param formatter
     */
    public AbstractColumnInStandardWaehrung(final boolean useOrderDate, final Formatter formatter) {
        super("id", AbstractColumnInStandardWaehrung.getStandardWaehrung(), formatter, false, Column.ALIGN_RIGHT); //$NON-NLS-1$
        this.useOrderDate = useOrderDate;
    }

    /**
     * Konstruktor mit Angabe eines Formatters.
     *
     * @param formatter
     */
    public AbstractColumnInStandardWaehrung(final Formatter formatter) {
        this(false, formatter);
    }

    /**
     * Liefert eine Liste mit allen ermittelten Umtauschraten.
     *
     * @return Liste mit Umtauschraten.
     */
    public List<ExchangeInfo> getExchangeInfos() {

        return this.exchangeInfos;
    }

    /**
     * Liefert den formatieren Zeileneintrag.
     *
     * @param price
     * @param waehrung
     * @param date
     * @return den formatieren Zeileneintrag
     */
    protected String getFormattedValue(final BigDecimal price, final String waehrung, final Date date) {

        String display = StringUtils.EMPTY;

        final String defaultWaehrung = Settings.getDepotWaehrung();
        if (waehrung != null && !waehrung.equals(defaultWaehrung)) {

            try {
                final BigDecimal value = this.getValueInSdtCurrency(price, waehrung, date);
                // Formatter definiert?
                if (this.getFormatter() == null) {
                    display = FormatterUtils.CURRENCY_FORMAT.get().format(value.doubleValue()) + defaultWaehrung;
                } else {
                    display = this.getFormatter().format(value);
                }
            } catch (final FinanceException e) {
                Logger.error(I18N.tr("AbstractColumnInStandardWaehrung.calculatingError"), //$NON-NLS-1$
                        e);
            }
        }
        return display;
    }

    /**
     * Liefert den Betrag in der Standardwhrung.
     *
     * @param betrag
     * @param waehrung
     * @param date
     * @return den Betrag in der Standardwhrung.
     * @throws FinanceException
     */
    protected BigDecimal getValueInSdtCurrency(final BigDecimal betrag, final String waehrung, final Date date)
            throws FinanceException {

        BigDecimal value = betrag == null ? BigDecimal.ZERO : betrag;

        final String defaultWaehrung = Settings.getDepotWaehrung();
        if (waehrung != null && !waehrung.equals(defaultWaehrung) && !"IDX".equalsIgnoreCase(waehrung)) { //$NON-NLS-1$
            ExchangeRate changeRate;
            changeRate = this.ccf.exchangeRate(waehrung, defaultWaehrung, date);

            final ExchangeInfo exchangeInfo = new ExchangeInfo(waehrung, defaultWaehrung, date, changeRate.getRate());
            this.storeExchange(exchangeInfo);

            value = value.multiply(changeRate.getRate());
        }

        return value;

    }

    /**
     * Wird Orderdatum zur Anzeige der Umtauschrate verwendet?.
     *
     * @return useOrderDate
     */
    protected boolean isUseOrderDate() {

        return this.useOrderDate;
    }

    /**
     * F&uuml;gt eine Umtauschrate zur Liste hinzu und vermeidet Doppelungen.
     *
     * @param exchangeInfo
     */
    private void storeExchange(final ExchangeInfo exchangeInfo) {

        for (final ExchangeInfo eis : this.exchangeInfos) {
            if (eis.getWaehrungFrom().equals(exchangeInfo.getWaehrungFrom())
                    && eis.getWaehrungTo().equals(exchangeInfo.getWaehrungTo())) {
                // Schon in Liste --> aktualisieren
                eis.setDatum(exchangeInfo.getDatum());
                eis.setKurs(exchangeInfo.getKurs());
                return;
            }
        }

        // Nicht in Liste --> hinzufgen
        this.exchangeInfos.add(exchangeInfo);

    }
}
