/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.column;

import java.math.BigDecimal;
import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.logging.Logger;

/**
 * Zusatzkostenspalte in Standardw&auml;hrung f&uuml;r die Order-Tabelle
 *
 * @author Frank Mnster
 */
public class ZusatzkostenInStandardWaehrung extends AbstractColumnInStandardWaehrung {

    /**
     * Error text in column in case the formatting failed.
     */
    private static final String ERROR_TEXT = "Error"; //$NON-NLS-1$
    /**
     * Serial UID
     */
    private static final long serialVersionUID = -4356082272860968994L;

    /**
     * Konstruktor mit Angabe des useOrderDate Value. Mit der oderDate kann
     * angegeben werden, ob die Umtauschrate zum Zeitpunkt der Order oder die
     * aktuellste zur Verf&uuml;gung stehendes angezeigt werden soll.
     *
     * @param useOrderDate
     */
    public ZusatzkostenInStandardWaehrung(final Boolean useOrderDate) {
        this(useOrderDate, null);
    }

    /**
     * Konstruktor mit Angabe eines Formatters und dem useOrderDate Value. Mit
     * der oderDate kann angegeben werden, ob die Umtauschrate zum Zeitpunkt der
     * Order oder die aktuellste zur Verf&uuml;gung stehendes angezeigt werden
     * soll.
     *
     * @param useOrderDate
     * @param formatter
     */
    public ZusatzkostenInStandardWaehrung(final Boolean useOrderDate, final Formatter formatter) {
        super(useOrderDate, formatter);
    }

    /**
     * Konstruktor mit Angabe eines Formatters
     *
     * @param formatter
     */
    public ZusatzkostenInStandardWaehrung(final Formatter formatter) {
        this(false, formatter);
    }

    /**
     * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
     *      java.lang.Object)
     */
    @Override
    public String getFormattedValue(final Object objectId, final Object context) {

        String display = StringUtils.EMPTY;
        if (context instanceof Wertpapier) {
            try {
                display = this.getFormattedValueWerpapier((Wertpapier) context);
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("AbstractColumnInStandardWaehrung.formatError", //$NON-NLS-1$
                        (String) objectId, (String) context), e);
                GUI.getStatusBar().setErrorText(I18N.tr("AbstractColumnInStandardWaehrung.userMsg")); //$NON-NLS-1$
                display = ZusatzkostenInStandardWaehrung.ERROR_TEXT;
            } catch (final FinanceException e) {
                Logger.error(I18N.tr("AbstractColumnInStandardWaehrung.calculatingError"), e); //$NON-NLS-1$
                GUI.getStatusBar().setErrorText(I18N.tr("AbstractColumnInStandardWaehrung.calculatingError")); //$NON-NLS-1$
                display = ZusatzkostenInStandardWaehrung.ERROR_TEXT;
            }
        } else if (context instanceof OrderHistory) {
            try {
                display = this.getFormattedValueOrderHistoty((OrderHistory) context);
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("AbstractColumnInStandardWaehrung.formatError", //$NON-NLS-1$
                        (String) objectId, (String) context), e);
                GUI.getStatusBar().setErrorText(I18N.tr("AbstractColumnInStandardWaehrung.userMsg")); //$NON-NLS-1$
                display = ZusatzkostenInStandardWaehrung.ERROR_TEXT;
            }
        }
        return display;
    }

    /**
     * Formatiert die Zusatzkosten fr die Order-History-Ansicht
     *
     * @param orderHistory
     * @return die formatierten Zusatzkosten fr die Order-History-Ansicht
     * @throws RemoteException
     */
    private String getFormattedValueOrderHistoty(final OrderHistory orderHistory) throws RemoteException {

        String display;
        final BigDecimal betrag = orderHistory.getAdditionalCosts();
        if (this.isUseOrderDate()) {
            display = this.getFormattedValue(betrag, orderHistory.getAktie().getCurrency(),
                    orderHistory.getOrderDate());
        } else {
            display = this.getFormattedValue(betrag, orderHistory.getAktie().getCurrency(), null);
        }
        return display;
    }

    /**
     * Formatiert die Zusatzkosten fr die Wertpapier-Ansicht
     *
     * @param wertpapier
     * @return die formatierten Zusatzkosten fr die Wertpapier-Ansicht
     * @throws RemoteException
     * @throws FinanceException
     */
    private String getFormattedValueWerpapier(final Wertpapier wertpapier) throws RemoteException, FinanceException {

        String display;
        BigDecimal addCosts = BigDecimal.ZERO;
        final DBIterator<OrderHistory> ohList = wertpapier.getOrderHistory();
        while (ohList.hasNext()) {
            final OrderHistory orderHistory = ohList.next();
            BigDecimal betrag = orderHistory.getAdditionalCosts() == null ? BigDecimal.ZERO
                    : orderHistory.getAdditionalCosts();
            if (this.isUseOrderDate()) {
                betrag = this.getValueInSdtCurrency(betrag, orderHistory.getAktie().getCurrency(),
                        orderHistory.getOrderDate());
            } else {
                betrag = this.getValueInSdtCurrency(betrag, orderHistory.getAktie().getCurrency(), null);
            }
            addCosts = addCosts.add(betrag);
        }
        // Formatter definiert?
        if (this.getFormatter() == null) {
            display = FormatterUtils.CURRENCY_FORMAT.get().format(addCosts) + Settings.getDepotWaehrung();
        } else {
            display = this.getFormatter().format(addCosts);
        }
        return display;
    }
}
