/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.awt.Font;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.DefaultOHLCDataset;
import org.jfree.data.xy.OHLCDataItem;
import org.jfree.data.xy.OHLCDataset;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.willuhn.jameica.gui.Part;

/**
 * Abstrakte Klasse zur Anzeige des Charts eines Wertpapiers. Als Chart wird JFreeChart verwendet.
 *
 * @author Frank Mnster
 */
public abstract class AbstractGrafik implements Part {

    /**
     * Formatter f&uuml;r die Kurswert-Annotation
     */
    protected static final DecimalFormat DECIMAL_FOMAT = new DecimalFormat("  ###,##0.00"); //$NON-NLS-1$

    /**
     * Achsenbeschriftung Font
     */
    protected static final Font FONT_AXIS = new Font("Dialog", Font.PLAIN, 12); //$NON-NLS-1$

    /**
     * Der Titel im JFreeChart
     */
    protected String title;

    /**
     * Das JFreeChart das unten drunter benutzt wird.
     */
    protected transient JFreeChart chart;

    /**
     * Display period
     */
    protected DateRange displayPeriod;

    /**
     * Die Werte des Wertpapieres
     */
    protected transient WertpapierHistoryBean wpHistoryBean;

    /**
     * Die DAX-Werte
     */
    protected transient WertpapierHistoryBean daxHistory;

    /**
     * Datenquelle
     */
    protected transient String dataSource;

    /**
     * Zeige DAX im CHART
     */
    protected boolean showDax;

    /**
     * Zeige Chart als Kerzengrafik
     */
    protected transient boolean showCandle;

    /**
     * Standardkonstruktor
     */
    public AbstractGrafik() {

        // Absichtlich leer
    }

    /**
     * Liefert das JFreeChart Objekt.
     *
     * @return das JFreeChart Objekt
     */
    public JFreeChart getChart() {

        return this.chart;
    }

    /**
     * @return fromDate
     */
    public LocalDate getFromDate() {

        return this.displayPeriod.getStartDate();
    }

    /**
     * Liefert den Titel
     *
     * @return der Charttitel
     */
    public String getTitle() {

        return this.title;
    }

    /**
     * @return toDate
     */
    public LocalDate getToDate() {

        return this.displayPeriod.getEndDate();
    }

    /**
     * Gibt an, ob die Grafik als Kerzengrafik angezeigt wird.
     *
     * @return the showCandle
     */
    public boolean isShowCandle() {

        return this.showCandle;
    }

    /**
     * @return showDax
     */
    public boolean isShowDax() {

        return this.showDax;
    }

    /**
     * Neuladen den Grafik.
     */
    public abstract void reload();

    /**
     * Setzt die Datenquelle.
     *
     * @param dataSource
     *            die Datenquelle
     */
    public void setDataSource(final String dataSource) {

        this.dataSource = dataSource;
    }

    /**
     * Setzt die DAX Daten.
     *
     * @param wpHistoryBean
     *            the wertpapierHistoryBean to set
     * @param daxHistory
     * @throws IllegalArgumentException
     */
    public void setKursDaxDaten(final WertpapierHistoryBean wpHistoryBean, final WertpapierHistoryBean daxHistory) {

        if (wpHistoryBean == null || daxHistory == null) {
            throw new IllegalArgumentException(I18N.tr("AbstractGrafik.nullArgument")); //$NON-NLS-1$
        }
        this.wpHistoryBean = wpHistoryBean;
        this.dataSource = wpHistoryBean.getDataSource();
        this.daxHistory = daxHistory;
    }

    /**
     * Setzt den Zeitraum, der angezeigt werden soll.
     *
     * @param displayPeriod
     *            the <code>DateRange</code> to set
     */

    /**
     * Sets the date range to display.
     *
     * @param displayPeriod
     */
    public void setRange(final DateRange displayPeriod) {

        this.displayPeriod = new DateRange(displayPeriod.getStartDate(), displayPeriod.getEndDate());
    }

    /**
     * Schalten zwischen Kerzen- und normaler Grafik um.
     *
     * @param showCandle
     *            the showCandle to set
     */
    public void setShowAsCandleStick(final boolean showCandle) {

        this.showCandle = showCandle;
        if (this.showCandle) {
            this.showDax = false; // Beides geht nicht!!!
        }
    }

    /**
     * @param showDax
     *            das zu setzende Objekt showDax
     */
    public void setShowDax(final boolean showDax) {

        this.showDax = showDax;
        if (this.showDax) {
            this.showCandle = false; // Beides geht nicht!
        }
    }

    /**
     * Setzt den Titel des Charts
     *
     * @param title
     */
    public void setTitle(final String title) {

        this.title = title;
        if (this.chart != null) {
            this.chart.setTitle(this.title);
        }
    }

    /**
     * Liefert ein JFreeChart OHLCDataset. Beinhaltet alle ntigen Daten fr eine Kerzengrafik. Dazu geh&ouml;ren u.a.
     * Er&ouml;ffnungs, Schlusskurs, Tageshoch-tief, etc.
     *
     * @return {@link OHLCDataset} mit Kerzengrafikdaten.
     */
    protected OHLCDataset getDatasetCandleStick() {

        final List<OHLCDataItem> dataItems = new ArrayList<>();
        final List<HistoryValueBean> historyValueBean = this.wpHistoryBean.getTimeRangeValues(this.displayPeriod);
        for (final HistoryValueBean wphv : historyValueBean) {
            final Date date = DateUtils.asDate(wphv.getLocalDate());
            final double open = wphv.getOpen();
            final double close = wphv.getClose();
            double high = wphv.getHigh();
            if (Double.isNaN(high) || high == 0.0d) {
                high = open;
            }
            double low = wphv.getLow();
            if (Double.isNaN(low) || low == 0.0d) {
                low = open;
            }
            final double volume = wphv.getVolume();
            wphv.getAdjClose();
            final OHLCDataItem item = new OHLCDataItem(date, open, high, low, // NOPMD
                    close, volume);
            dataItems.add(item);
        }
        final OHLCDataItem[] data = dataItems.toArray(new OHLCDataItem[0]);
        return new DefaultOHLCDataset(this.wpHistoryBean.getwpId(), data);
    }

    /**
     * Liefert eine JFreeChart TimeSeriesCollection der Wertpapierdaten.
     *
     * @return JFreeChart TimeSeriesCollection der Wertpapierdaten
     */
    protected TimeSeriesCollection getDataSetKurs() {

        final TimeSeries seriesKurs = new TimeSeries(I18N.tr("AbstractGrafik.timeSeriesRate")); //$NON-NLS-1$
        final List<HistoryValueBean> historyValueBean = this.wpHistoryBean.getTimeRangeValues(this.displayPeriod);
        for (final HistoryValueBean wphv : historyValueBean) {
            final Date date = DateUtils.asDate(wphv.getLocalDate());
            double value = wphv.getClose();
            if (Double.isNaN(value)) {
                // versuchen wir es mit open
                value = wphv.getOpen();
            }
            // Neues Datumsobjekt erstellen (wirklich ntig?)
            // NUr hinzufgen, falls vorhanden
            final Day day = new Day(date); // NOPMD
            if (Double.isNaN(value)) {
                seriesKurs.add(day, Double.NaN);
            } else {
                seriesKurs.add(day, value);
            }
        }
        final TimeSeriesCollection collection = new TimeSeriesCollection();
        collection.addSeries(seriesKurs);
        return collection;
    }

    /**
     * Liefert eine JFreeChart TimeSeriesCollection den prozentualen Ver&auml;nderungen des Wertpapier und DAX Kurses.
     *
     * @return JFreeChart TimeSeriesCollection der Wertpapierdaten
     */
    protected TimeSeriesCollection getDatasetKursDaxVergleich() {

        final TimeSeries seriesKurs = new TimeSeries(I18N.tr("AbstractGrafik.timeSeriesRate")); //$NON-NLS-1$
        final TimeSeries seriesDax = new TimeSeries("DAX"); //$NON-NLS-1$
        final List<HistoryValueBean> daxHistoryValues = this.daxHistory.getTimeRangeValues(this.displayPeriod);
        // Dax in Prozent
        final double daxStart = daxHistoryValues.get(0).getClose();
        for (final HistoryValueBean dax : daxHistoryValues) {
            final Date date = DateUtils.asDate(dax.getLocalDate());
            final double daxValue = dax.getClose();
            final double closeDax = (daxValue - daxStart) * 100 / daxStart;
            final Day day = new Day(date); // NOPMD
            seriesDax.add(day, closeDax);
        }
        // Dax-History zu kurz?
        final LocalDate daxStartDate = daxHistoryValues.get(0).getLocalDate();
        final double kursStart = this.wpHistoryBean.getValuesAtDate(daxStartDate).getClose();
        final List<HistoryValueBean> historyValues = this.wpHistoryBean.getTimeRangeValues(this.displayPeriod);
        for (final HistoryValueBean kurs : historyValues) {
            final Date date = DateUtils.asDate(kurs.getLocalDate());
            final double kursValue = kurs.getClose();
            final double closeKurs = (kursValue - kursStart) * 100 / kursStart;
            final Day day = new Day(date); // NOPMD
            seriesKurs.add(day, closeKurs);
        }
        final TimeSeriesCollection collection = new TimeSeriesCollection();
        collection.addSeries(seriesKurs);
        collection.addSeries(seriesDax);
        return collection;
    }
}
