/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.utils;

import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Objects;

/**
 * Utiliy Klasse fr Datumsberechnungen
 *
 * @author Frank Mnster
 */
public final class DateUtils {

    /**
     * Exception string in case an argument is NULL.
     */
    private static final String NOT_NULL = " darf nicht NULL sein"; //$NON-NLS-1$

    /**
     * Exception string in case a fromDate argument is NULL.
     */
    private static final String FROM_DATE_NOT_NULL = "'fromDate'" + DateUtils.NOT_NULL; //$NON-NLS-1$

    /**
     * Exception string in case a fromDate argument is NULL.
     */
    private static final String LOCAL_DATE_NOT_NULL = "'localDate'" + DateUtils.NOT_NULL; //$NON-NLS-1$

    /**
     * Exception string in case a toDate argument is NULL.
     */
    private static final String TO_DATE_NOT_NULL = "'toDate'" + DateUtils.NOT_NULL; //$NON-NLS-1$

    /**
     * Exception string in case an date argumet is NULL.
     */
    private static final String DATE_NOT_NULL = "'date'" + DateUtils.NOT_NULL; //$NON-NLS-1$

    /**
     * Millisekunden eines Tages
     */
    private static final double DAY_MILLIS = 24.0 * 60.0 * 60.0 * 1000.0;

    /**
     * Addiert die Anzahl an Tagen und liefert ein neues Objekt basierend auf
     * Now()
     *
     * @param days
     *            Anzahl die addiert wird (kann negative sein)
     * @return das new Date Objekt mit der Anzahl addiert
     */
    public static Date addDays(final int days) {
	final Calendar cal = Calendar.getInstance();
	cal.add(Calendar.DATE, days);
	return cal.getTime();
    }

    /**
     * @param localDate
     * @return Date object from LocalDate Object
     */
    public static Date asDate(final LocalDate localDate) {
	Objects.requireNonNull(localDate, DateUtils.LOCAL_DATE_NOT_NULL);
	return Date.from(localDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * @param localDateTime
     *            not NULL
     * @return Date object from LocalDateTime object
     */
    public static Date asDate(final LocalDateTime localDateTime) {
	Objects.requireNonNull(localDateTime, DateUtils.LOCAL_DATE_NOT_NULL);
	return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * @param date
     *            not NULL
     * @return LocalDate object from Date object
     */
    public static LocalDate asLocalDate(final Date date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
    }

    /**
     * @param date
     * @return LocalDateTime object from Date object
     */
    public static LocalDateTime asLocalDateTime(final Date date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    /**
     * Berechnet die Differenz zwischen den beiden Date-Objekt
     *
     * @param fromDate
     *            das Anfangsdatum Date-Objekt, nicht Null
     * @param toDate
     *            das Enddatum Date-Objekt, nicht Null
     * @return die Differenz in Tagen
     */
    public static long dateDiff(final Date fromDate, final Date toDate) {
	Objects.requireNonNull(fromDate, DateUtils.FROM_DATE_NOT_NULL);
	Objects.requireNonNull(toDate, DateUtils.TO_DATE_NOT_NULL);
	return (long) ((toDate.getTime() - fromDate.getTime()) / DateUtils.DAY_MILLIS);
    }

    /**
     * Berechnet die Differenz zwischen Jetzt und dem Date-Objekt
     *
     * @param date
     *            das Date-Objekt, nicht Null
     * @return die Differenz Tagen
     */
    public static long dateDiff(final LocalDate date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	return ChronoUnit.DAYS.between(date, LocalDate.now());
    }

    /**
     * Berechnet die Differenz zwischen den beiden Date-Objekt
     *
     * @param fromDate
     *            das Anfangsdatum Date-Objekt, nicht Null
     * @param toDate
     *            das Enddatum Date-Objekt, nicht Null
     * @return die Differenz in Tagen
     */
    public static long dateDiff(final LocalDate fromDate, final LocalDate toDate) {
	Objects.requireNonNull(fromDate, DateUtils.FROM_DATE_NOT_NULL);
	Objects.requireNonNull(toDate, DateUtils.TO_DATE_NOT_NULL);
	return ChronoUnit.DAYS.between(fromDate, toDate);
    }

    /**
     * Berechnet die Differenz zwischen Jetzt und dem Date-Objekt
     *
     * @param date
     *            das Date-Objekt, nicht Null
     * @return die Differenz Tagen
     */
    public static long dateDiff(final LocalDateTime date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	return ChronoUnit.DAYS.between(date, LocalDateTime.now());
    }

    /**
     * Liefert den Tag des Datums als Integerwert.
     *
     * @param date
     *            das Datum
     * @return den Tag als Integerwert
     */
    public static int getDay(final Date date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	final Calendar calendar = new GregorianCalendar();
	calendar.setTime(date);
	return calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * Liefert den Monat des Datums als Integerwert. Januar gleich 1, Dezember
     * gleich 12.
     *
     * @param date
     *            das Datum
     * @return den Monat als Integerwert
     */
    public static int getMonth(final Date date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	final Calendar calendar = new GregorianCalendar();
	calendar.setTime(date);
	return calendar.get(Calendar.MONTH + 1);
    }

    /**
     * Liefert das Jahr des Datums als Integerwert.
     *
     * @param date
     *            das Datum
     * @return das Jahr als Integerwert
     */
    public static int getYear(final Date date) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	final Calendar calendar = new GregorianCalendar();
	calendar.setTime(date);
	return calendar.get(Calendar.YEAR);
    }

    /**
     * Checks, if a date is in rage of two other dates.
     *
     * @param date
     *            date to be checked, not null
     * @param fromDate
     *            first of the range, not null
     * @param toDate
     *            last of the range, not null
     * @return true, if fromDate <= date <= toDate, false else
     */
    public static boolean isInRange(final LocalDate date, final LocalDate fromDate, final LocalDate toDate) {
	Objects.requireNonNull(date, DateUtils.DATE_NOT_NULL);
	Objects.requireNonNull(fromDate, DateUtils.FROM_DATE_NOT_NULL);
	Objects.requireNonNull(toDate, DateUtils.TO_DATE_NOT_NULL);
	return fromDate.compareTo(date) <= 0 && toDate.compareTo(date) >= 0;
    }

    /**
     * Subtrahiert die Anzahl an Tagen und liefert ein neues Objekt basierend
     * auf Now()
     *
     * @param days
     *            Anzahl die addiert wird (kann negative sein)
     * @return das new Date Objekt mit der Anzahl subtrahiert
     */
    public static LocalDate subtractDays(final int days) {
	return LocalDate.now().minusDays(days);
    }

    /**
     * Standardkonstruktor
     */
    private DateUtils() {
	// absichlich leer
    }
}
