/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.io.BufferedReader;
import java.io.IOException;
import java.math.BigDecimal;

import org.apache.commons.lang3.StringUtils;

import de.frankmuenster.jameica.depot.importer.exceptions.OrderValueMissing;
import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.internal.objects.OrderImpl;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * Abstract class of a RegisterCsvOrderBean implementation to define common constants and methods
 *
 * @author Frank Mnster
 */
public abstract class AbstractCsvOrderBean implements CsvOrderBean {

    /**
     * de-DE local
     */
    public static final String DE_DE = "de-DE"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public AbstractCsvOrderBean() {

        // intentionally left blank
    }

    @Override
    public Order getOrder() throws OrderValueMissing {

        // date
        if (this.getOrderDate() == null) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.BookingDateMissing")); //$NON-NLS-1$
        }
        // ISIN
        if (StringUtils.isEmpty(this.getIsin())) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.isinMissing")); //$NON-NLS-1$
        }
        // name
        if (StringUtils.isEmpty(this.getName())) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.nameMissing")); //$NON-NLS-1$
        }
        // currency
        if (StringUtils.isEmpty(this.getOrderCurrency())) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.currencyMissing")); //$NON-NLS-1$
        }
        // value
        if (this.getOrderPrice() == null) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.rateMissing")); //$NON-NLS-1$
        }
        // value nominal
        if (this.getNominal() == null) {
            throw new OrderValueMissing(I18N.tr("AbstractCsvOrderBean.ammountMissing")); //$NON-NLS-1$
        }

        // is defined, now get the order from the implementation
        return this.getOrderFromBean();

    }

    @Override
    public boolean isReading(final BufferedReader csvExport) {

        boolean isSupporting = false;
        try {
            // mark the
            csvExport.mark(1000);
            // skip the first lines if needed
            for (int i = 0; i < this.getSkipLines(); i++) {
                Logger.debug(I18N.tr("AbstractCsvOrderBean.skippedLine") + csvExport.readLine()); //$NON-NLS-1$
            }
            // read test line
            final String line = csvExport.readLine();
            if (line != null) {
                isSupporting = this.isSupported(line);
            }
            // reset reader to beginning
            csvExport.reset();
        } catch (final IOException e) {
            Logger.error(I18N.tr("AbstractCsvOrderBean.errorReadingCsvFile"), e); //$NON-NLS-1$
        }

        // return what has been found out
        return isSupporting;

    }

    @Override
    public String toString() {

        final StringBuilder buffer = new StringBuilder(50);
        buffer.append("ISIN: ").append(this.getIsin()).append(", Name: ").append(this.getName()).append(", Stck: ") //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
                .append(this.getNominal()).append(", Kurs: ").append(this.getOrderPrice()).append(", Comment: ") //$NON-NLS-1$//$NON-NLS-2$
                .append(this.getComment());

        return buffer.toString();
    }

    /**
     * Returns the order nominal. Must be implemented from sub class.
     *
     * @return the order price
     */
    protected abstract BigDecimal getNominalForOrderBean();

    /**
     * Creates an order based on the bean data.
     *
     * @return the created order
     */
    protected Order getOrderFromBean() {

        final Order order = new OrderImpl();

        order.setComment(this.getComment());
        order.setName(this.getName());
        order.setOrderDate(this.getOrderDate());

        // get order price from sub class implementation
        order.setOrderPrice(this.getOrderPrice());

        order.setOrderVolume(this.getNominalForOrderBean());
        order.setIsin(this.getIsin());
        order.setCurrency(this.getOrderCurrency());
        order.setOrderNumber(this.getOrderNumber());

        return order;
    }

    /**
     * Implements the 'real check' if a CsvOrderBean supports the file or not based on the header line
     *
     * @param line
     *            the header line
     * @return <i>true</i> in case the CsvOrderBean implementation support this line, <i>false</i> else.
     */
    protected abstract boolean isSupported(String line);

}
