/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.io.BufferedReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import com.opencsv.bean.CsvToBean;
import com.opencsv.bean.CsvToBeanBuilder;
import com.opencsv.exceptions.CsvException;
import com.opencsv.exceptions.CsvRequiredFieldEmptyException;

import de.frankmuenster.jameica.depot.importer.exceptions.OrderProcessException;
import de.frankmuenster.jameica.depot.importer.exceptions.OrderProcessExceptionType;
import de.frankmuenster.jameica.depot.importer.exceptions.OrderValueMissing;
import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class OrderReader {

    /**
     * List with order read from the csv file
     */
    private List<Order> orderList;

    /**
     * List of parse execption
     */
    private List<CsvException> expections;

    /**
     * Constructs a new OrderReader with the suitable CsvOrderBean for the
     * provided BufferedReader.
     *
     * @throws OrderProcessException
     *             in case no suitable CsvOrderBean implementation that can read
     *             the buffer could be found.
     */
    public OrderReader(final BufferedReader csvExport) throws OrderProcessException {

        Objects.requireNonNull(csvExport, "CSVExport must not be null"); //$NON-NLS-1$

        // get the reader
        final CsvOrderBean imp = CsvOrderBeanRegistry.getCsvOrderBean(csvExport);

        if (imp == null) {
            throw new OrderProcessException(I18N.tr("OrderReader.NoReaderFound"), //$NON-NLS-1$
                    OrderProcessExceptionType.NO_IMPORTER_FOUND);
        }
        // now read the orders
        this.getOrders(csvExport, imp);

    }

    /**
     * @return the exceptions
     */
    public List<CsvException> getExpections() {

        return this.expections;
    }

    /**
     * Returns the number of orders with load error.
     *
     * @return
     */
    public int getLoadErrorNumber() {

        int errors = 0;

        if (this.expections != null) {
            errors = this.expections.size();
        }

        return errors;
    }

    /**
     * Returns the number of orders with load error.
     *
     * @return
     */
    public int getLoadSuccessfullNumber() {

        int count = 0;
        if (this.orderList != null) {
            count = this.orderList.size();
        }

        return count;
    }

    /**
     * @return the orderList
     */
    public List<Order> getOrderList() {

        return this.orderList;
    }

    /**
     * Returns the total number of orders in the input file.
     *
     * @return
     */
    public int getTotalOrders() {

        int count = 0;

        if (this.expections != null && this.orderList != null) {
            count = this.expections.size() + this.orderList.size();
        }

        return count;
    }

    /**
     * Reads the csv file with the
     *
     * @param csvExport
     * @param importer
     */
    private void getOrders(final Reader csvExport, final CsvOrderBean importer) {

        // TODO add parse error and throw OrderProcessException

        final int skipLines = importer.getSkipLines();
        final char separator = importer.getSeparator();

        final CsvToBean<CsvOrderBean> beanBuilder = new CsvToBeanBuilder<CsvOrderBean>(csvExport)
                .withType(importer.getClass()).withIgnoreLeadingWhiteSpace(true).withSkipLines(skipLines)
                .withSeparator(separator).withThrowExceptions(false).build();
        final List<CsvOrderBean> importOrderList = beanBuilder.parse();

        this.expections = beanBuilder.getCapturedExceptions();

        final Iterator<CsvOrderBean> orderIterator = importOrderList.iterator();

        this.orderList = new ArrayList<>();

        while (orderIterator.hasNext()) {
            final CsvOrderBean importOrder = orderIterator.next();
            Logger.info(importOrder.toString());
            try {
                this.orderList.add(importOrder.getOrder());
            } catch (final OrderValueMissing e) {
                Logger.info("Adding " + importOrder.getName() + " to exception list"); //$NON-NLS-1$//$NON-NLS-2$
                final CsvException exception = new CsvRequiredFieldEmptyException(e.getMessage()); // NOPMD
                exception.setLine(importOrder.toString().split(",")); //$NON-NLS-1$
                this.expections.add(exception);
            }
        }
    }

}
