/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.currate.CurRateConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.importer.exceptions.OrderProcessException;
import de.frankmuenster.jameica.depot.importer.exceptions.OrderProcessExceptionType;
import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * A write to write new orders into the database. <br/>
 * The write takes care to map the orders to a stock via ISIN. And also tries to
 * identify duplications.
 *
 * @author Frank Mnster
 */
public class OrderWriter {

    /**
     * Internal cache to speed-up searching for duplicates
     */
    private static Map<String, Wertpapier> stockCache = new ConcurrentHashMap<>();

    /**
     * Original list of order to process
     */
    private final List<Order> orders;

    /**
     * List of new order found
     */
    private final List<Order> newOrderList = new ArrayList<>();

    /**
     * List of orders with an error
     */
    private final Map<Order, OrderProcessException> errorOrderList = new LinkedHashMap<>(); // NOPMD

    /**
     * List of orders that could not be stored
     */
    private final Map<Order, Exception> storeOrderErrorList = new LinkedHashMap<>(); // NOPMD

    /**
     * List of duplications found
     */
    private final List<Order> exitingOrderList = new ArrayList<>();

    /**
     * The database service
     */
    private DBService service;

    /**
     * the number of stored orders
     */
    private int storedOrders;

    /**
     * Return a new OrderWriter. The list of orders is automatically
     * pre-processed during initialisation of the writer.
     */
    public OrderWriter(final List<Order> orders) throws OrderProcessException {

        Objects.requireNonNull(orders, "Orders must not be null"); //$NON-NLS-1$
        this.orders = orders;

        try {
            this.service = Settings.getDBService();

            // process the list
            this.processList();

        } catch (final RemoteException e) {
            Logger.error(I18N.tr("OrderWriter.dbConnectionFailed"), e); //$NON-NLS-1$
            throw new OrderProcessException(I18N.tr("OrderWriter.dbConnectionFailed"), //$NON-NLS-1$
                    OrderProcessExceptionType.DB_CONNETION_PROBLEM, e);

        }

    }

    /**
     * Gets all orders with a processing error
     *
     * @return
     */
    public Map<Order, OrderProcessException> getErrorOrders() {

        return this.errorOrderList;
    }

    /**
     * Gets all duplicated thus already stored orders
     *
     * @return
     */
    public List<Order> getExistingOrders() {

        return this.exitingOrderList;
    }

    /**
     * Gets all new orders found
     *
     * @return
     */
    public List<Order> getNewOrders() {

        return this.newOrderList;
    }

    /**
     * Gets all orders that could not be stored
     *
     * @return
     */
    public Map<Order, Exception> getStoreOrderErrors() {

        return this.storeOrderErrorList;
    }

    /**
     * Return the number of total errors
     */
    public int getTotalErrors() {

        int errors = 0;
        errors += this.storeOrderErrorList.size();
        errors += this.errorOrderList.size();

        return errors;
    }

    /**
     * Returns if all new found orders have been stored successfully in the
     * database.
     *
     * @return <i>true</i> in case orders list has elements and all new orders
     *         found have been stored, <i>false</i> else.
     */
    public boolean isAllStored() {

        boolean allStored = false;
        if (this.orders.isEmpty()) {
            allStored = true;
        } else {
            if (this.storedOrders == this.newOrderList.size()) {
                allStored = true;
            }
        }
        return allStored;
    }

    /**
     * Writes the new Orders in the database. Order that cannot be stored will
     * be put in the storeOrderError list.
     *
     * @return the number of stored orders
     */
    public int writeOrders() {

        this.storedOrders = 0;

        // Loop over all
        for (final Order order : this.newOrderList) {

            try {
                final OrderHistory dbOrder = (OrderHistory) this.service.createObject(OrderHistory.class, null);

                dbOrder.setAktie(this.getStockFromOrder(order));
                dbOrder.setOrderAmount(order.getOrderVolume());
                dbOrder.setOrderDate(order.getOrderDate());
                dbOrder.setOrderPrice(order.getOrderPrice());

                dbOrder.setAdditionalCosts(BigDecimal.ZERO);
                dbOrder.setAdditionalCostsCurrency(order.getCurrency());

                dbOrder.setOrderNumber(order.getOrderNumber());
                dbOrder.setComment(order.getComment());

                // checks and set prices in case order is not in main currency
                calculateForeignPrices(order, dbOrder);

                // save history
                dbOrder.store();
                this.storedOrders++;

            } catch (RemoteException | ApplicationException | OrderProcessException e) {
                Logger.error(I18N.tr("OrderWriter.errorSaving"), e); //$NON-NLS-1$
                this.storeOrderErrorList.put(order, e);
            }
        }

        return this.storedOrders;
    }

    /**
     * Calculates and sets the prices in main currency if necessary.
     *
     * @param order
     *            the gross order values
     * @param dbOrder
     *            the order to be saved in the database
     *
     * @throws RemoteException
     *             in case the price or error comment cannot be set
     */

    private void calculateForeignPrices(final Order order, final OrderHistory dbOrder) throws RemoteException {

        if (!Settings.getDepotWaehrung().equals(order.getCurrency())) {
            try {
                final ExchangeRate exRate = CurRateConverter.getInstance().exchangeRate(order.getCurrency(),
                        Settings.getDepotWaehrung(), order.getOrderDate());
                dbOrder.setOrderPriceInMainCurrency(exRate.getRate().multiply(order.getOrderPrice()));

            } catch (final FinanceException e) {
                dbOrder.setComment(I18N.tr("OrderWriter.errorCalculatingExchange", "EUR") + " " + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                        e.getType().toString());
                Logger.error(I18N.tr("OrderWriter.errorCalculatingExchange", "EUR"), e); //$NON-NLS-1$ //$NON-NLS-2$
            }

        }

    }

    /**
     * Returns the stock for this order. A cache list is maintained to speed-up
     * the lookup
     *
     * @param order
     *            which stock should be search
     * @return the stock
     * @throws OrderProcessException
     *             in case the stock could not be retrieved or a database error
     *             occurred
     */
    private Wertpapier getStockFromOrder(final Order order) throws OrderProcessException {

        if (StringUtils.isEmpty(order.getIsin())) {
            throw new OrderProcessException(OrderProcessExceptionType.ISIN_IS_NULL_OR_EMPTY);
        }

        // check cache
        if (OrderWriter.stockCache.containsKey(order.getIsin())) {
            return OrderWriter.stockCache.get(order.getIsin()); // NOPMD
        }

        DBIterator<?> list;
        try {

            list = this.service.createList(Wertpapier.class);

            // list.setOrder("ORDER BY name asc"); //$NON-NLS-1$
            while (list.hasNext()) {
                final Wertpapier stock = (Wertpapier) list.next();
                if (order.getIsin().equals(stock.getIsin())) {
                    if (!OrderWriter.stockCache.containsKey(order.getIsin())) {
                        OrderWriter.stockCache.put(order.getIsin(), stock);
                    }
                    return stock;
                }
            }

        } catch (final RemoteException e) {
            throw new OrderProcessException(order.getIsin() + I18N.tr("OrderWriter.instrumentNotFound"), //$NON-NLS-1$
                    OrderProcessExceptionType.STOCK_NOT_FOUND, e);
        }

        throw new OrderProcessException(order.getIsin() + I18N.tr("OrderWriter.instrumentNotFound"), //$NON-NLS-1$
                OrderProcessExceptionType.STOCK_NOT_FOUND);
    }

    /**
     * Check if the given order is already in the database.
     *
     * @param order
     * @param security
     * @return
     * @throws OrderProcessException
     */
    private boolean isDublicateOrder(final Order order, final Wertpapier security) throws OrderProcessException {

        boolean isEqual = false;

        try {

            // the orders of the stock
            final DBIterator<OrderHistory> stockOrders = security.getOrderHistory();

            // order number for new order
            final String orderNumber = order.getOrderNumber();

            // loop through all stored orders
            while (stockOrders.hasNext()) {

                final OrderHistory stockOrder = stockOrders.next();

                // order number from DB order
                final String ordernumberDb = stockOrder.getOrderNumber();

                if (StringUtils.isNoneEmpty(ordernumberDb)) {
                    // order number already in DB
                    if (orderNumber.equals(ordernumberDb)) {
                        isEqual = true;
                        break;
                    }
                } else {

                    // boolean to indicate same order
                    boolean deepCheckEqual = true;
                    // Check date
                    final Date date = order.getOrderDate();
                    if (date != null && !date.equals(stockOrder.getOrderDate())) {
                        deepCheckEqual = false;
                    }
                    if (deepCheckEqual) {
                        // Check volume
                        final BigDecimal volumeOrder = order.getOrderVolume();
                        final BigDecimal volumeStock = stockOrder.getOrderAmount();
                        if (volumeOrder.compareTo(volumeStock) != 0) {
                            deepCheckEqual = false;
                        }
                    }
                    if (deepCheckEqual) {
                        // check price
                        final BigDecimal price = order.getOrderPrice();
                        if (price != null && !price.equals(stockOrder.getOrderPrice())) {
                            deepCheckEqual = false;
                        }
                    }
                    if (deepCheckEqual) {
                        // Equal --> Update order number in DB
                        stockOrder.setOrderNumber(orderNumber);
                        stockOrder.store();
                        isEqual = true;
                    }

                }
            }

        } catch (

        final RemoteException e) {
            throw new OrderProcessException(I18N.tr("OrderWriter.dbErrorOccured"), //$NON-NLS-1$
                    OrderProcessExceptionType.EQUAL_TEST_FAILED, e);
        } catch (final ApplicationException e) {
            // Storage of order number failed. Bad luck but not critical
            Logger.error(I18N.tr("OrderWriter.failedUpdatingOrderNumber") + e.getMessage(), e); //$NON-NLS-1$
        }

        // return test result
        return isEqual;
    }

    /**
     * Process the list and creates three result lists: <br/>
     * <ol>
     * <li>new order that can be loaded</li>
     * <li>existing orders</li>
     * <li>orders with any kind of errors</li>
     * </ol>
     */
    private void processList() {

        // Loop over all
        for (final Order order : this.orders) {
            try {
                // get the related stock
                final Wertpapier stock = this.getStockFromOrder(order);
                // check if order exists already
                if (this.isDublicateOrder(order, stock)) {
                    this.exitingOrderList.add(order);
                } else {
                    this.newOrderList.add(order);
                }
            } catch (final OrderProcessException e) {
                this.errorOrderList.put(order, e);
            }
        }
    }

}
