/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.wizard;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;

import com.opencsv.exceptions.CsvException;

import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.parts.filedropper.FileAddedListener;
import de.frankmuenster.jameica.depot.gui.parts.filedropper.FileLoadDropPastePart;
import de.frankmuenster.jameica.depot.importer.OrderReader;
import de.frankmuenster.jameica.depot.importer.OrderWriter;
import de.frankmuenster.jameica.depot.importer.exceptions.OrderProcessException;
import de.frankmuenster.jameica.depot.importer.wizard.parts.OrderExceptionTablePart;
import de.frankmuenster.jameica.depot.importer.wizard.parts.OrderTablePart;
import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

@SuppressWarnings({ "PMD.ExcessiveImports" })
public class OrderImportWizardControl implements FileAddedListener {

    /**
     * I18n key for OrderImportWizardControl.errorAddOrderToTable
     */
    private static final String I18N_ERROR_ADD_TO_TABLE = "OrderImportWizardControl.errorAddOrderToTable"; //$NON-NLS-1$

    /**
     * Tab ID of the load page
     */
    private static final int LOAD_TAB_ID = 0;

    /**
     * Tab ID of the save page
     */
    private static final int SAVE_TAB_ID = 1;

    /**
     * Tab ID of the result page
     */
    private static final int RESULT_TAB_ID = 2;

    /**
     * The main container to hold the wizard pages
     */
    private TabFolder folder;

    /**
     * The back button
     */
    private Button back;

    /**
     * The next button
     */
    private Button next;

    /**
     * Info label for page1
     */
    private LabelInputExtended page1InfoLabel;

    /**
     * File load/paste/dropper to load the file
     */
    private FileLoadDropPastePart page1FileDropper;

    /**
     * Table with successfully loaded orders
     */
    private OrderTablePart page2OrderLoadedTbl;

    /**
     * Table to hold the orders with a load exception
     */
    private OrderExceptionTablePart p2OrderLoadExceptionTbl;

    /**
     * The cancel button
     */
    private Button cancel;

    /**
     * The wizard dialog
     */
    private final OrderImportWizard importWizard;

    /**
     * Wizard page2
     */
    private ImportWizardPage2Part wizardPage2;

    /**
     * The order reader
     */
    private OrderReader importer;

    /**
     * Field on page 3 to show number of saved orders
     */
    private LabelInputExtended page3SavedOrder;
    /**
     * Table on page 3 to hold the orders that failed to load
     */
    private OrderTablePart page3FailedLoadedTbl;

    /**
     * Wizard page 3
     */
    private ImportWizardPage3Part wizardPage3;

    /**
     * Field on page 3 to show the the total number of read orders
     */
    private LabelInputExtended page3TotalOrder;

    /**
     * Field on page 3 to show the number of orders with an problem/error
     */
    private LabelInputExtended page3FailedOrders;

    /**
     * Field on page 3 to show the number of already existing orders
     */
    private Input page3ExistingOrder;

    /**
     * Field on page 3 to show the number of orders with a load problem
     */
    private LabelInputExtended page3LoadErrorOrders;

    /**
     * Constructs a new controller for the order import wizard
     */
    public OrderImportWizardControl(final OrderImportWizard importWizard) {

        super();
        this.importWizard = importWizard;
    }

    /**
     * Prepares the tab folder to hold the wizard pages.
     *
     * @param parent
     *            the composite to paint the tab folder in
     *
     */
    public void createTabFolder(final Composite parent) {

        if (this.folder == null) {

            // Create the tab group
            this.folder = new TabFolder(parent, SWT.NONE);
            this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
            this.folder.addSelectionListener(new SelectionListener() {

                /**
                 * Add a listener to see get notified about active tab change to
                 * be able to update the button status
                 */
                @Override
                public void widgetDefaultSelected(final SelectionEvent arg0) {

                    // nothing to do
                }

                @Override
                public void widgetSelected(final SelectionEvent arg0) {

                    if (arg0.getSource() instanceof TabFolder) {
                        // update the button status
                        OrderImportWizardControl.this.updateButtons();
                    }
                }
            });

            // now add some pages
            this.addPages();

        }

    }

    /**
     * Return the back page button. It creates it if not yet available.
     *
     * @return the back page button
     */
    public Button getBackButton() {

        if (this.back == null) {
            this.back = new Button("Zurck", new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object context)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    OrderImportWizardControl.this.handleBackPage();
                }

            }, null, true, "go-previous.png"); //$NON-NLS-1$
            this.back.setEnabled(false);

        }
        return this.back;

    }

    /**
     * Return the cancel page button. It creates it if not yet available.
     *
     * @return the cancel page button
     */
    public Button getCancelButton() {

        if (this.cancel == null) {
            this.cancel = new Button("Abbrechen", new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object context)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    // Close the wizard
                    OrderImportWizardControl.this.importWizard.close();

                }

            }, null, true, "edit-undo.png"); //$NON-NLS-1$
            this.cancel.setEnabled(true);

        }
        return this.cancel;

    }

    /**
     * Return the next page button. It creates it if not yet available.
     *
     * @return the next page button
     */
    public Button getNextButton() {

        if (this.next == null) {
            this.next = new Button(" Weiter ", new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object context)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    OrderImportWizardControl.this.handleNextPage();
                }

            }, null, true, "go-next.png"); //$NON-NLS-1$
            this.next.setEnabled(true);

        }
        return this.next;

    }

    /**
     * Returns the FileDropper to load/paste/drop CSV files
     *
     * @return the FileDropper
     */
    public FileLoadDropPastePart getPage1FileDropper() {

        if (this.page1FileDropper == null) {
            this.page1FileDropper = new FileLoadDropPastePart();
            this.page1FileDropper.addListener(this);
        }
        return this.page1FileDropper;
    }

    /**
     * Return the info label for the fist page. It creates it if not yet
     * available.
     *
     * @return the info label for the first page.
     */
    public LabelInputExtended getPage1InfoLabel() {

        if (this.page1InfoLabel == null) {
            this.page1InfoLabel = new LabelInputExtended("\n\n\n", true); //$NON-NLS-1$
        }
        return this.page1InfoLabel;
    }

    /**
     * Return the table with loaded orders for page 2. It creates it if not yet
     * available.
     *
     * @return a table with loaded orders.
     */
    public OrderTablePart getPage2OrderLoadedTable() {

        if (this.page2OrderLoadedTbl == null) {
            this.page2OrderLoadedTbl = new OrderTablePart(new ArrayList<Order>(), null);
        }
        return this.page2OrderLoadedTbl;
    }

    /**
     * Return the table with load failures for page 2. It creates it if not yet
     * available.
     *
     * @return a table with loaded failures.
     */
    public OrderExceptionTablePart getPage2OrderLoadExeptionTable() {

        if (this.p2OrderLoadExceptionTbl == null) {
            this.p2OrderLoadExceptionTbl = new OrderExceptionTablePart(new ArrayList<CsvException>(), null);
        }
        return this.p2OrderLoadExceptionTbl;
    }

    /**
     * Return an info field to show the number of already existing orders. It
     * creates it if not yet available.
     *
     * @return an info field to show the number of already existing orders.
     */
    public Input getPage3ExistingOrders() {
        if (this.page3ExistingOrder == null) {
            this.page3ExistingOrder = new LabelInputExtended(StringUtils.EMPTY, SWT.LEFT);
            this.page3ExistingOrder.disable();
        }

        return this.page3ExistingOrder;
    }

    /**
     * Return the table with load failures for page 3. It creates it if not yet
     * available.
     *
     * @return a table with loaded failures
     */
    public OrderTablePart getPage3FailedLoadedTable() {

        if (this.page3FailedLoadedTbl == null) {
            this.page3FailedLoadedTbl = new OrderTablePart(new ArrayList<Order>(), null);
            this.page3FailedLoadedTbl.setHeight(220);

        }
        return this.page3FailedLoadedTbl;
    }

    /**
     * Return an info field to show the number of orders with an problem/errors.
     * It creates it if not yet available.
     *
     * @return an info field to show the number of orders with an problem/error.
     */
    public LabelInputExtended getPage3FailureOrders() {
        if (this.page3FailedOrders == null) {
            this.page3FailedOrders = new LabelInputExtended(StringUtils.EMPTY, SWT.LEFT);
            this.page3FailedOrders.disable();
        }

        return this.page3FailedOrders;

    }

    /**
     * Return an info field to show the number of orders with a load problem. It
     * creates it if not yet available.
     *
     * @return an info field to show the number of orders with a load problem.
     */
    public LabelInputExtended getPage3LoadErrorOrders() {
        if (this.page3LoadErrorOrders == null) {
            this.page3LoadErrorOrders = new LabelInputExtended(StringUtils.EMPTY, SWT.LEFT);
            this.page3LoadErrorOrders.disable();
        }

        return this.page3LoadErrorOrders;

    }

    /**
     * Return an info field to show the number of saved orders. It creates it if
     * not yet available.
     *
     * @return an info field to show the number of saved orders.
     */
    public LabelInputExtended getPage3SavedOrders() {

        if (this.page3SavedOrder == null) {
            this.page3SavedOrder = new LabelInputExtended(StringUtils.EMPTY, SWT.LEFT);
            this.page3SavedOrder.disable();
        }

        return this.page3SavedOrder;

    }

    /**
     * Return an info field to show the total number of read orders from the
     * input file. It creates it if not yet available.
     *
     * @return an info field to show the total number of read orders.
     */
    public LabelInputExtended getPage3TotalOrders() {
        if (this.page3TotalOrder == null) {
            this.page3TotalOrder = new LabelInputExtended(StringUtils.EMPTY, SWT.LEFT);
            this.page3TotalOrder.disable();
        }

        return this.page3TotalOrder;

    }

    @Override
    public void newFileAvailable(final File file) {

        // Load file
        this.loadCsv(file);

    }

    /**
     * Add the wizard pages to the tab folder. Existing ones are first disposed.
     */
    private void addPages() {

        // Clear all tabs from the tab folder
        if (this.folder != null) {
            final Control[] oldTabs = this.folder.getChildren();
            for (final Control tab : oldTabs) {
                tab.dispose();
            }

            final TabGroup page1 = new TabGroup(this.folder, I18N.tr("OrderImportWizardControl.startImport"), true, 1); //$NON-NLS-1$
            page1.addPart(new ImportWizardPage1Part(this));
            this.wizardPage2 = new ImportWizardPage2Part(this);
            final TabGroup page2 = new TabGroup(this.folder, I18N.tr("OrderImportWizardControl.readResult"), true, 1); //$NON-NLS-1$
            page2.addPart(this.wizardPage2);
            final TabGroup page3 = new TabGroup(this.folder, I18N.tr("OrderImportWizardControl.saveResult"), true, 1); //$NON-NLS-1$
            this.wizardPage3 = new ImportWizardPage3Part(this);
            page3.addPart(this.wizardPage3);
        }

    }

    /**
     * Handles a click on the back button according to the actual page.
     */
    private void handleBackPage() {

        if (0 < this.folder.getSelectionIndex()) {
            this.folder.setSelection(this.folder.getSelectionIndex() - 1);
        }
        // update (de)active buttons
        this.updateButtons();
    }

    /**
     * Handles the next button action. This can be to simply show the page2, to
     * start storing the orders or to close the wizard depending on the actual
     * TAB.
     */
    private void handleNextPage() {

        if (this.folder.getSelectionIndex() == OrderImportWizardControl.RESULT_TAB_ID) {
            // Close the wizard
            this.importWizard.close();
        } else {
            // if save tab is open trigger save
            if (this.folder.getSelectionIndex() == OrderImportWizardControl.SAVE_TAB_ID) {
                this.triggerSaveOrder();
            }
            // open next tab
            if (this.folder.getSelectionIndex() < this.folder.getChildren().length - 1) {
                this.folder.setSelection(this.folder.getSelectionIndex() + 1);
            }
            // update (de)active buttons
            this.updateButtons();
        }
    }

    private void loadCsv(final File file) {

        Logger.info("Starting loading file..." + file.getName()); //$NON-NLS-1$

        try (BufferedReader reader = Files.newBufferedReader(Paths.get(file.toURI()), StandardCharsets.ISO_8859_1)) {

            this.importer = new OrderReader(reader);

            // clear file dropper status as we can handle it in own status field
            this.page1FileDropper.setStatusText(StringUtils.EMPTY);

            // Successful read orders
            final List<CsvException> execptions = this.importer.getExpections();
            if (execptions.isEmpty()) {
                this.page1InfoLabel.setBackgroundColor(Color.SUCCESS);
                this.page1InfoLabel.setValue(I18N.tr("OrderImportWizardControl.fileLoaded", file.getName())); //$NON-NLS-1$
            } else {
                this.page1InfoLabel.setBackgroundColor(Color.MANDATORY_BG);
                this.page1InfoLabel.setValue(I18N.tr("OrderImportWizardControl.fileLoadedWithErrors", file.getName())); //$NON-NLS-1$
            }

            // update loaded order table
            this.updateOrderLoadedTable();

        } catch (final IOException e) {
            // TODO Auto-generated catch block
            Logger.error(e.getLocalizedMessage());
        } catch (final OrderProcessException e) {

            // TODO check type first
            this.page1FileDropper.setStatusText(e.getType().getDetailMsg());
            Logger.error(e.getLocalizedMessage(), e);

        }

    }

    /**
     * Handles saving of the orders
     */
    private void triggerSaveOrder() {

        Logger.debug("Start saving orders..."); //$NON-NLS-1$

        if (this.importer != null) {

            try {
                @SuppressWarnings("unchecked")
                final List<Order> ord = this.page2OrderLoadedTbl.getItems();
                final OrderWriter writer = new OrderWriter(ord);

                final int saveOk = writer.writeOrders();

                this.page3SavedOrder.setValue(String.valueOf(saveOk));
                this.page3ExistingOrder.setValue(String.valueOf(writer.getExistingOrders().size()));
                this.page3TotalOrder.setValue(String.valueOf(this.importer.getTotalOrders()));
                this.page3FailedOrders.setValue(String.valueOf(writer.getTotalErrors()));
                this.page3LoadErrorOrders.setValue(String.valueOf(this.importer.getLoadErrorNumber()));

                updateOrdersSavedTable(writer);

            } catch (RemoteException | OrderProcessException e) {
                this.wizardPage3.repaintSaveError();
                this.folder.setSelection(OrderImportWizardControl.SAVE_TAB_ID);
                Logger.error(e.getLocalizedMessage(), e);
            }
        }
    }

    /**
     * Updated the button status and texts according to the actual page of the
     * wizard.
     */
    private void updateButtons() {

        if (this.back != null && this.next != null && this.folder != null) {

            switch (this.folder.getSelectionIndex()) {
            case LOAD_TAB_ID:
                this.cancel.setEnabled(true);
                this.back.setEnabled(false);
                this.next.setEnabled(true);
                this.next.setText(I18N.tr("OrderImportWizardControl.next")); //$NON-NLS-1$
                this.next.setIcon("go-next.png"); //$NON-NLS-1$
                break;
            case SAVE_TAB_ID:
                this.cancel.setEnabled(true);
                this.back.setEnabled(true);
                this.next.setEnabled(this.importer != null);
                this.next.setText(I18N.tr("OrderImportWizardControl.save")); //$NON-NLS-1$
                this.next.setIcon("document-save.png"); //$NON-NLS-1$
                break;
            case RESULT_TAB_ID:
                this.cancel.setEnabled(false);
                this.back.setEnabled(false);
                this.next.setEnabled(true);
                this.next.setText(I18N.tr("OrderImportWizardControl.close")); //$NON-NLS-1$
                this.next.setIcon("window-close.png"); //$NON-NLS-1$
                break;

            default:
                break;
            }

            this.importWizard.redrawButtons();

        }

    }

    /**
     * Updates the tables on page 2 of the wizard
     */
    private void updateOrderLoadedTable() {

        // Clean tables
        try {
            this.page2OrderLoadedTbl.removeAll();
            this.p2OrderLoadExceptionTbl.removeAll();

            // First refresh the read result wizard page with correct table
            // layout
            if (this.importer.getExpections().isEmpty()) {
                this.p2OrderLoadExceptionTbl.setEnabled(false);
                this.page2OrderLoadedTbl.setHeight(330);
            } else {
                this.p2OrderLoadExceptionTbl.setEnabled(true);
                this.page2OrderLoadedTbl.setHeight(130);
            }
            this.wizardPage2.repaint();

            // Now update the table content
            // Update successfully loaded orders table
            this.importer.getOrderList().forEach(o -> {
                try {
                    this.page2OrderLoadedTbl.addItem(o);
                } catch (final RemoteException e1) {
                    Logger.error(I18N.tr(OrderImportWizardControl.I18N_ERROR_ADD_TO_TABLE, o.toString()));
                }
            });

            // Update exceptions table
            this.importer.getExpections().forEach(o -> {
                try {
                    this.p2OrderLoadExceptionTbl.addItem(o);
                } catch (final RemoteException e1) {
                    Logger.error(I18N.tr(OrderImportWizardControl.I18N_ERROR_ADD_TO_TABLE, o.toString()));
                }

            });

        } catch (final RemoteException e) {
            Logger.error(I18N.tr("OrderImportWizardControl.pageUpdateFailed", "2"), e); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * Updates the tables of failed stored on page 3 of the wizard
     *
     * @param writer
     */
    private void updateOrdersSavedTable(final OrderWriter writer) {

        // Clean tables
        try {
            this.page3FailedLoadedTbl.removeAll();

            // First refresh the read result wizard page with correct table
            // layout
            if (writer.getErrorOrders().isEmpty()) {
                this.page3FailedLoadedTbl.setEnabled(false);
            } else {
                this.page3FailedLoadedTbl.setEnabled(true);
                this.page2OrderLoadedTbl.setHeight(130);
            }
            this.wizardPage3.repaint();

            // Now update the table content

            // Update successfully loaded orders table
            writer.getErrorOrders().forEach((o, e) -> {
                o.setComment(e.getLocalizedMessage());
                try {
                    this.page3FailedLoadedTbl.addItem(o);
                } catch (final RemoteException e1) {
                    Logger.error(I18N.tr(OrderImportWizardControl.I18N_ERROR_ADD_TO_TABLE, o.toString()));
                }

            });

            writer.getStoreOrderErrors().forEach((o, e) -> {
                o.setComment(e.getLocalizedMessage());
                try {
                    this.page3FailedLoadedTbl.addItem(o);
                } catch (final RemoteException e1) {
                    Logger.error(I18N.tr(OrderImportWizardControl.I18N_ERROR_ADD_TO_TABLE, o.toString()));
                }

            });

        } catch (final RemoteException e) {
            Logger.error(I18N.tr("OrderImportWizardControl.pageUpdateFailed", "3"), e); //$NON-NLS-1$ //$NON-NLS-2$
        }

    }

}
