/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.List;

import org.eclipse.swt.graphics.Color;
import org.jfree.chart.plot.XYPlot;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;

/**
 * Abstrakte Klasse fr Analysemethoden.
 *
 * @param <P>
 *            Der Type des KursAnalyseParameters
 * @param <R>
 *            Der Ergebnistyp. Darf nur KursDaten[] oder XYPLot sein.
 * 
 * @author Frank Mnster
 */
public abstract class AbstractKursAnalyse<P, R> implements IKursAnalyse<P, R> {

    /**
     * Die Parameter fr die Analysemethode
     */
    protected KursAnalyseParameter<P> parameter;

    /**
     * Der Ergebnis XYPLot
     */
    protected XYPlot anaG;

    /**
     * Die historischen Werte des Wertpapiers.
     */
    protected WertpapierHistoryBean wertPapierHistory;

    /**
     * Die Schlusskurse
     */
    protected KursDaten closeDaten = new KursDaten();

    /**
     * Anzahl der Linien
     */
    private int lineCount;

    /**
     * Die Farben der Linien
     */
    private Color[] color;

    /**
     * Der Style der Linien
     */
    private KursAnalyseLineStyle[] lineStyle;

    /**
     * Soll der Bereich unterhalb ausgemalt werden.
     */
    private boolean enableArea;

    /**
     * Line las Steps zeichen
     */
    private boolean enableStep;

    /**
     * Der Typ der Analyse
     */
    private KursAnalyseTyp type;

    /**
     * Standardkonstruktor
     */
    public AbstractKursAnalyse() {
	// Absichlich leer
    }

    /**
     * Konstruktor mit Angaben von WertpapterHistory und den Analyseparametern.
     *
     * @param wpHistoryBean
     * @param parameter
     */
    public AbstractKursAnalyse(final WertpapierHistoryBean wpHistoryBean, final KursAnalyseParameter<P> parameter) {
	this.wertPapierHistory = wpHistoryBean;
	final List<HistoryValueBean> wphv = wpHistoryBean.getHistoryValues();
	// Loop thru all Values
	for (final HistoryValueBean historyValue : wphv) {
	    final DateValuePair value = new DateValuePair( // NOPMD
		    historyValue.getLocalDate(), historyValue.getClose());
	    this.closeDaten.add(value);
	}
	this.parameter = parameter;
    }

    /**
     * @return color
     */
    @Override
    public Color[] getColor() {
	return this.color.clone();
    }

    /**
     * Setzt die Anzahl der Linien im Kurstyp OVERLAY
     *
     * @return Anzahl Linien
     */
    @Override
    public int getLineCount() {
	return this.type == KursAnalyseTyp.SEPERATE_GRAPH ? 0 : this.lineCount;
    }

    /**
     * @return color
     */
    @Override
    public KursAnalyseLineStyle[] getLineStyle() {
	return this.lineStyle.clone();
    }

    /**
     * @see IKursAnalyse#getType()
     */
    @Override
    public KursAnalyseTyp getType() {
	return this.type;
    }

    /**
     * @return enableArea
     */
    public boolean isEnableArea() {
	return this.enableArea;
    }

    /**
     * @return enableStep
     */
    public boolean isEnableStep() {
	return this.enableStep;
    }

    /**
     * @param color
     *            das zu setzende Objekt color
     */
    @Override
    public void setColor(final Color... color) {
	this.color = color.clone();
    }

    /**
     * @param enableArea
     *            das zu setzende Objekt enableArea
     */
    public void setEnableArea(final boolean enableArea) {
	this.enableArea = enableArea;
    }

    /**
     * @param enableStep
     *            das zu setzende Objekt enableStep
     */
    public void setEnableStep(final boolean enableStep) {
	this.enableStep = enableStep;
    }

    /**
     * @param parameter
     */
    @Override
    public void setKursAnalyseParameter(final KursAnalyseParameter<P> parameter) {
	this.parameter = parameter;
    }

    /**
     * Setzt die Anzahl an Kurslinien im Kurstyp OVERLAY
     *
     * @param count
     *            die Anzahl an Linien
     */
    @Override
    public void setLineCount(final int count) {
	this.lineCount = count;
    }

    /**
     * Setzt den Linienstile
     */
    @Override
    public void setLineStyle(final KursAnalyseLineStyle... lineStyle) {
	this.lineStyle = lineStyle.clone();
    }

    /**
     * @see IKursAnalyse#setType(KursAnalyseTyp)
     */
    @Override
    public void setType(final KursAnalyseTyp type) {
	this.type = type;
    }

    /**
     * @see IKursAnalyse#setWertpapierHistory(WertpapierHistoryBean)
     */
    @Override
    public void setWertpapierHistory(final WertpapierHistoryBean wpHistoryBean) {
	this.wertPapierHistory = wpHistoryBean;
	this.closeDaten.clear();
	final List<HistoryValueBean> wphv = wpHistoryBean.getHistoryValues();
	// Loop thru all Values
	for (final HistoryValueBean historyValue : wphv) {
	    final DateValuePair value = new DateValuePair( // NOPMD
		    historyValue.getLocalDate(), historyValue.getClose());
	    this.closeDaten.add(value);
	}
    }
}
