/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.frankmuenster.jameica.depot.kursanalyse.bollingbaender.BollingBaender;
import de.frankmuenster.jameica.depot.kursanalyse.ema.ExponentialMovingAverage;
import de.frankmuenster.jameica.depot.kursanalyse.macd.MACD;
import de.frankmuenster.jameica.depot.kursanalyse.mom.Momentum;
import de.frankmuenster.jameica.depot.kursanalyse.rsi.RelativeStrengthIndex;
import de.frankmuenster.jameica.depot.kursanalyse.sma.SimpleMovingAverage;
import de.frankmuenster.jameica.depot.kursanalyse.volumen.Volumen;
import de.frankmuenster.jameica.depot.utils.I18N;

/**
 * Alle Analysemethoden als Liste.
 *
 * @author Frank Mnster
 *
 * @param <P>
 *            type of <code>KursAnalyseParameters</code>
 * @param <R>
 *            type of result. Only <code>KursDaten[]</code> or
 *            <code>XYPLot</code> is allowed.
 *
 */
public final class Analysen<P, R> extends ArrayList<IKursAnalyse<P, R>> {

    /**
     * die serial UID
     */
    private static final long serialVersionUID = -2102978330463004776L;

    /**
     * Die Liste mit allen verf&uuml;gbaren Analysemethoden
     */
    private static List<Class<?>> methods;

    /**
     * Unser Instanz
     */
    private static Analysen<Object, Object> instance;

    /**
     * Liefert eine SingletonObject der Klasse
     *
     * @return das SingletonObject der Klasse
     */
    public static Analysen<Object, Object> getInstance() {

        synchronized (Analysen.class) {
            if (Analysen.instance == null) {
                Analysen.instance = new Analysen<>();
            }
        }
        return Analysen.instance;

    }

    /**
     * Statische Methode die eine Liste aller Analysemethoden liefert.
     *
     * @return Liste aller Analysemethoden
     */
    public static List<Class<?>> getMethods() {

        synchronized (Analysen.class) {
            if (Analysen.methods == null) {
                Analysen.methods = new ArrayList<>();
                Analysen.methods.add(SimpleMovingAverage.class);
                Analysen.methods.add(ExponentialMovingAverage.class);
                Analysen.methods.add(BollingBaender.class);
                Analysen.methods.add(MACD.class);
                Analysen.methods.add(Momentum.class);
                Analysen.methods.add(Volumen.class);
                Analysen.methods.add(RelativeStrengthIndex.class);
            }
        }
        return Analysen.methods;
    }

    /**
     * Privater Konstruktor zum Vermeiden von mehrfache Instantiierung.
     */
    private Analysen() {
        super();
    }

    /**
     * @see java.util.ArrayList#add(java.lang.Object)
     */
    @Override
    public boolean add(final IKursAnalyse<P, R> iKursAnalyse) {
        boolean added = false;
        for (final IKursAnalyse<P, R> analyse : this) {
            if (iKursAnalyse.getLegendTitel().equals(analyse.getLegendTitel())) {
                added = true;
                break;
            }
        }

        // Add if not already done
        if (!added) {
            added = super.add(iKursAnalyse);
        }

        return added;
    }

    /**
     *
     * @see java.util.ArrayList#add(int, java.lang.Object)
     */
    @Override
    public void add(final int index, final IKursAnalyse<P, R> iKursAnalyse) {
        boolean alreadyAdded = false;
        for (final IKursAnalyse<P, R> analyse : this) {
            if (iKursAnalyse.getLegendTitel().equals(analyse.getLegendTitel())) {
                alreadyAdded = true;
                break;
            }
        }
        if (!alreadyAdded) {
            super.add(index, iKursAnalyse);
        }
    }

    /**
     * Liefert <i>true</i> falls es eine Analyse mit angegeben Titel gibt, sonst
     * <i>false</i>.
     *
     * @param legendTitel
     *            der Legendentitel, nach dem gesucht werden soll.
     * @return <i>true</i> falls es eine Analyse mit angegeben Titel gibt, sonst
     *         <i>false</i>
     */
    public boolean conatinsAnalyseWithLegendTitel(final String legendTitel) {

        boolean analysePresent = false;
        for (final IKursAnalyse<P, R> analyse : this) {
            if (analyse.getLegendTitel().equals(legendTitel)) {
                analysePresent = true;
                break;
            }
        }

        return analysePresent;

    }

    /**
     * Liefert die Analyse zum Angegebenen Titel.
     *
     * @param legendTitel
     * @return die Analyse
     * @throws AnalyseNotInListeException
     */
    public IKursAnalyse<P, R> getAnalyseWithLegendTitel(final String legendTitel) throws AnalyseNotInListeException {

        for (final IKursAnalyse<P, R> analyse : this) {
            if (analyse.getLegendTitel().equals(legendTitel)) {
                return analyse;
            }
        }

        throw new AnalyseNotInListeException(I18N.tr("Analysen.analysisNotFound", legendTitel)); //$NON-NLS-1$
    }

    /**
     * @return the anzahlBarAnalysen
     */
    public int getAnzahlSeperateAnalysen() {
        int numSeperateAna = 0;
        for (final Iterator<IKursAnalyse<P, R>> iterator = super.iterator(); iterator.hasNext();) {
            final IKursAnalyse<P, R> type = iterator.next();
            if (type.getType().equals(KursAnalyseTyp.SEPERATE_GRAPH)) {
                numSeperateAna++;
            }
        }
        return numSeperateAna;
    }
}
