/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.bollingbaender; // NOPMD

import java.time.LocalDate;
import java.util.LinkedList;
import java.util.Queue;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * Bolling B&auml;nder
 *
 * @author Frank Mnster
 */
public class BollingBaender extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private static final String NAME = "BOLLING"; //$NON-NLS-1$

    /**
     * Bollig-B&auml;nder mit Average 10
     */
    private static final int BOLLIG_10 = 10;

    /**
     * Bollig-B&auml;nder mit Average 50
     */
    private static final int BOLLIG_50 = 50;

    /**
     * Tempor&auml;rge Queue zum Zwischenspeichern von Werten
     */
    private final Queue<Double> temp = new LinkedList<>();

    /**
     * Summe
     */
    private double sum;

    /**
     * Daten der Bollig Mittellinie (Trigger)
     */
    private KursDaten bollingMitte;

    /**
     * Daten der oberen Bolliglinie
     */
    private KursDaten bollingOben;

    /**
     * Daten der unteren Bolliglinie
     */
    private KursDaten bollingUnten;

    /**
     * Liefert die AddAction
     *
     * @return die BollingBaenderHandleAddAction
     */
    public static Action getHandleAction() {

        return new BollingBaenderHandleAddAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

        return BollingBaender.NAME;
    }

    /**
     * Standardkonstruktor, ruft nur super auf.
     */
    public BollingBaender() {
        super();
    }

    /**
     * Konstruktor mit Angabe der Kursdaten und der zu benutzen Parameter.
     *
     * @param wpHistoryBean
     *            dir Kursdaten
     * @param parameter
     *            die Paramater
     */
    public BollingBaender(final WertpapierHistoryBean wpHistoryBean, final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel ()
     */
    @Override
    public String getLegendTitel() {

        return "Bolling [" + this.parameter.getValue() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public KursDaten[] getResult(final DateRange period) {

        final LocalDate fromDate = period.getStartDate();
        final LocalDate toDate = period.getEndDate();

        if (this.bollingMitte == null) {
            this.calculate();
        }
        // Ergbniss zusammenkrzen auf Zeitangebe
        final KursDaten bbMitte = new KursDaten();
        final KursDaten bbOben = new KursDaten();
        final KursDaten bbUnten = new KursDaten();
        for (int i = 0; i < this.bollingMitte.size(); i++) {
            final DateValuePair bbMitteValue = this.bollingMitte.get(i);
            if (DateUtils.isInRange(bbMitteValue.getDate(), fromDate, toDate)) {
                final DateValuePair bbObenValue = this.bollingOben.get(i);
                final DateValuePair bbUntenValue = this.bollingUnten.get(i);
                bbMitte.add(bbMitteValue);
                bbOben.add(bbObenValue);
                bbUnten.add(bbUntenValue);
            }
        }
        // nun das Ergebnis zusammenbasteln
        final KursDaten[] result = new KursDaten[3];
        result[0] = bbMitte;
        result[1] = bbOben;
        result[2] = bbUnten;
        return result;
    }

    /**
     * F&uuml;gt einen Wert in die Queue. Wurde die Zeitspane aus Parameter
     * erreicht, wird der &auml;lteste Wert aus der Queue gel&ouml;scht. Die
     * Summe sum wird ebenfalls immer aus letzetn n Werten (aus Parameter)
     * aktualisiert.
     *
     * @param wert
     */
    private void add(final double wert) {

        this.sum += wert;
        this.temp.add(wert);
        if (this.temp.size() > this.parameter.getValue()) {
            this.sum -= this.temp.remove();
        }
    }

    /**
     * Hier werden die Bollig-B&auml;nderdaten berechnet.
     */
    private void calculate() {

        this.bollingMitte = new KursDaten();
        this.bollingOben = new KursDaten();
        this.bollingUnten = new KursDaten();
        double faktor = 2.0;
        if (this.parameter.getValue() == BollingBaender.BOLLIG_10) {
            faktor = 1.9;
        } else if (this.parameter.getValue() == BollingBaender.BOLLIG_50) {
            faktor = 2.1;
        }
        for (int i = 0; i < this.closeDaten.size(); i++) {
            final DateValuePair orgValue = this.closeDaten.get(i);
            this.add(orgValue.getValue());
            final double bollingMitteValue = this.getAverage();
            final double stdAbweichung = this.getStandardAbweichung(bollingMitteValue);
            final double bollingObenValue = bollingMitteValue + stdAbweichung * faktor;
            final double bollingUntenValue = bollingMitteValue - stdAbweichung * faktor;
            if (!Double.isNaN(bollingMitteValue)) {
                this.bollingMitte.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingMitteValue));
                this.bollingOben.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingObenValue));
                this.bollingUnten.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingUntenValue));
            }
        }
    }

    /**
     * Liefert den Durchschnitt der letzten n-Werte (wenn auch mindestens
     * n-Werte vorhanden sind.
     *
     * @return der Durchschnitt der letzten n-Werte
     */
    private double getAverage() {

        Double durchschnitt = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            durchschnitt = this.sum / this.temp.size();
        }
        return durchschnitt;
    }

    /**
     * Berechnet die Standardabweichung des Mittelbandes f&uuml;r den angegeben
     * Zeitraum.
     *
     * @param bollingMitte
     * @return die Standardabweichung
     */
    private double getStandardAbweichung(final double bollingMitte) {

        Double stdAbweichung = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            double sumAbweichungen = 0.0;
            for (final double close : this.temp) {
                sumAbweichungen += (close - bollingMitte) * (close - bollingMitte);
            }
            stdAbweichung = Math.sqrt(sumAbweichungen / this.temp.size());
        }
        return stdAbweichung;
    }
}
