/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.macd;

import java.awt.Color;
import java.awt.Paint;
import java.time.LocalDate;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYBarPainter;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.time.Day;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;

import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.ItemUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * MACD
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.ExcessiveImports")
public class MACD extends AbstractKursAnalyse<MACDParameter, XYPlot> { // NOPMD

    /**
     * Methodenname
     */
    private static final String NAME = "MACD"; //$NON-NLS-1$

    /**
     * Daten der MACD Linie
     */
    private TimeSeries serieMacd;

    /**
     * Daten der Trigger Linie
     */
    private TimeSeries serieTrigger;

    /**
     * Der der Histogrambalken
     */
    private TimeSeries serieHistogram;

    /**
     * Der MACD Renderer
     */
    private XYItemRenderer rendererMACD;

    /**
     * Der Trigger Renderer
     */
    private XYItemRenderer rendererTrigger;

    /**
     * Der Histogram Renderer
     */
    private MyXYBarRenderer rendererHistogram;

    /**
     * Liefert die AddAction
     *
     * @return ie MACDHandleAddAction
     */
    public static Action getHandleAction() {

	return new MACDHandleAddAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

	return MACD.NAME;
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {

	return "MACD"; //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public XYPlot getResult(final DateRange period) {

	final LocalDate fromDate = period.getStartDate();
	final LocalDate toDate = period.getEndDate();

	if (this.anaG == null) {
	    final DateAxis dateAxisKurs = new DateAxis(I18N.tr("MACD.axisDate")); //$NON-NLS-1$
	    final NumberAxis rangeAxisKurs = new NumberAxis(this.getLegendTitel());
	    rangeAxisKurs.setAutoRangeIncludesZero(false);
	    this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
	    ((DateAxis) this.anaG.getDomainAxis()).setTimeline(SegmentedTimeline.newMondayThroughFridayTimeline());
	    this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
	    this.rendererMACD = new StandardXYItemRenderer();
	    this.rendererMACD.setSeriesPaint(0, Color.BLACK);
	    this.rendererTrigger = new StandardXYItemRenderer();
	    this.rendererTrigger.setSeriesPaint(0, Color.RED);
	    this.rendererHistogram = new MyXYBarRenderer();
	    this.rendererHistogram.setSeriesVisibleInLegend(0, false);
	    this.rendererHistogram.setShadowVisible(false);
	    this.rendererHistogram.setMargin(0.85);
	    this.rendererHistogram.setBarPainter(new StandardXYBarPainter());
	    ItemUtils.addLegendTitle(this.anaG);
	}
	if (this.serieMacd == null) {
	    this.calculate();
	}
	final Day fromDay = new Day(DateUtils.asDate(fromDate));
	final Day toDay = new Day(DateUtils.asDate(toDate));
	try {
	    final TimeSeriesCollection tmpMacd = new TimeSeriesCollection(this.serieMacd.createCopy(fromDay, toDay));
	    final TimeSeriesCollection tmpTrigger = new TimeSeriesCollection(
		    this.serieTrigger.createCopy(fromDay, toDay));
	    final TimeSeriesCollection tmpHistogram = new TimeSeriesCollection(
		    this.serieHistogram.createCopy(fromDay, toDay));
	    this.anaG.setDataset(0, tmpMacd);
	    this.anaG.setDataset(1, tmpTrigger);
	    this.anaG.setDataset(2, tmpHistogram);
	    this.anaG.setRenderer(0, this.rendererMACD);
	    this.anaG.setRenderer(1, this.rendererTrigger);
	    this.anaG.setRenderer(2, this.rendererHistogram);
	} catch (final CloneNotSupportedException e) {
	    Logger.error(I18N.tr("MACD.createTsDataError"), e); //$NON-NLS-1$
	}
	return this.anaG;
    }

    /**
     * Nun mal rechnen
     */
    private void calculate() {

	final MACDParameter para = this.parameter.getValue();
	this.serieMacd = new TimeSeries(this.getLegendTitel());
	this.serieTrigger = new TimeSeries(I18N.tr("MACD.seriesTriggerName")); //$NON-NLS-1$
	this.serieHistogram = new TimeSeries(I18N.tr("MACD.seriesHistogramName")); //$NON-NLS-1$
	// MACD Daten
	final KursDaten emaKurz = new KursDaten();
	final KursDaten emaLang = new KursDaten();
	double ewK;
	double ewL;
	ewL = 2.0 / (para.getEmaLang() + 1.0);
	ewK = 2.0 / (para.getEmaKurz() + 1.0);
	emaKurz.add(new DateValuePair(this.closeDaten.get(0).getDate(), this.closeDaten.get(0).getValue()));
	emaLang.add(new DateValuePair(this.closeDaten.get(0).getDate(), this.closeDaten.get(0).getValue()));
	Day day = new Day(DateUtils.asDate(this.closeDaten.get(0).getDate()));
	this.serieMacd.add(day, 0.0);
	for (int i = 1; i < this.closeDaten.size(); i++) {
	    final double emaKurzValue = emaKurz.get(i - 1).getValue()
		    + ewK * (this.closeDaten.get(i).getValue() - emaKurz.get(i - 1).getValue());
	    emaKurz.add(new DateValuePair(this.closeDaten.get(i).getDate(), // NOPMD
		    emaKurzValue));
	    final double emaLangValue = emaLang.get(i - 1).getValue()
		    + ewL * (this.closeDaten.get(i).getValue() - emaLang.get(i - 1).getValue());
	    emaLang.add(new DateValuePair(this.closeDaten.get(i).getDate(), // NOPMD
		    emaLangValue));
	    day = new Day(DateUtils.asDate(this.closeDaten.get(i).getDate())); // NOPMD
	    this.serieMacd.add(day, emaKurzValue - emaLangValue);
	}
	// Trigger und Histogram
	final double ewS = 2.0 / (para.getEmaSignal() + 1.0);
	// Startwerte
	this.serieTrigger.add(this.serieMacd.getDataItem(0).getPeriod(), this.serieMacd.getDataItem(0).getValue());
	this.serieHistogram.add(this.serieMacd.getDataItem(0).getPeriod(), 0.0);
	for (int i = 1; i < this.serieMacd.getItemCount(); i++) {
	    // Datum
	    final RegularTimePeriod date1 = this.serieMacd.getDataItem(i).getPeriod();
	    // Trigger
	    final double trigger0 = (Double) this.serieTrigger.getDataItem(i - 1).getValue();
	    final double macd1 = (Double) this.serieMacd.getDataItem(i).getValue();
	    final double trigger1 = trigger0 + ewS * (macd1 - trigger0);
	    this.serieTrigger.add(date1, trigger1);
	    // Histogram
	    final double histogram = macd1 - trigger1;
	    this.serieHistogram.add(date1, histogram);
	}
    }

    /**
     * Modifizierter XYBarRenderer. Positive Werte werden gr&uuml;n dargestellt,
     * negative rot.
     */
    /* default */ static class MyXYBarRenderer extends XYBarRenderer {

	/**
	 * serial UID
	 */
	private static final long serialVersionUID = -8978350984656959134L;

	/**
	 * Standardkonstruktor, ruft nur super() auf
	 */
	public MyXYBarRenderer() {
	    super();
	}

	/**
	 * @see org.jfree.chart.renderer.xy.XYItemRenderer#getItemPaint(int row,
	 *      int column)
	 */
	@Override
	public Paint getItemPaint(final int series, final int item) {

	    Paint color;
	    // here we assume we're working with the primary dataset
	    final XYDataset dataset = this.getPlot().getDataset(2);
	    final double value = dataset.getYValue(series, item);
	    if (value >= 0) {
		color = Color.green;
	    } else {
		color = Color.red;
	    }
	    return color;
	}
    }
}
