/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.rsi;

import java.awt.BasicStroke;
import java.awt.Color;
import java.time.LocalDate;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYDifferenceRenderer;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.ItemUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * Momentum Methode
 *
 * @author Frank Mnster
 */
public class RelativeStrengthIndex extends AbstractKursAnalyse<Integer, XYPlot> {

    /**
     * Methodenname
     */
    private static final String NAME = "RSI"; //$NON-NLS-1$

    /**
     * RSI max. Limit (100)
     */
    private static final int RSI_ABOVE_100 = 100;

    /**
     * Daten der RSI Serie
     */
    private TimeSeries serieRsi;

    /**
     * Daten der RSI Serie unter 30%
     */
    private TimeSeries serie30;

    /**
     * Daten der RSI Serie ber 70%
     */
    private TimeSeries serie70;

    /**
     * Renderer f&uuml; der Bereich &uuml;ber 70%
     */
    private XYDifferenceRenderer diff1;

    /**
     * Renderer f&uuml; der Bereich unter 30%
     */
    private XYDifferenceRenderer diff2;

    /**
     * Liefert die Add Aktion
     *
     * @return die RelativeStrengthIndexHandleAction
     */
    public static Action getHandleAction() {

	return new RelativeStrengthIndexHandleAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

	return RelativeStrengthIndex.NAME;
    }

    /**
     * Standardkonstruktor
     */
    public RelativeStrengthIndex() {
	super();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {

	final int period = this.parameter.getValue();
	return "RSI (" + period + ")"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public XYPlot getResult(final DateRange period) {

	final LocalDate fromDate = period.getStartDate();
	final LocalDate toDate = period.getEndDate();

	if (this.anaG == null) {
	    final DateAxis dateAxisKurs = new DateAxis(I18N.tr("RelativeStrengthIndex.axisDate")); //$NON-NLS-1$
	    final NumberAxis rangeAxisKurs = new NumberAxis("RSI"); //$NON-NLS-1$
	    rangeAxisKurs.setLowerBound(0);
	    rangeAxisKurs.setUpperBound(100);
	    this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
	    ((DateAxis) this.anaG.getDomainAxis()).setTimeline(SegmentedTimeline.newMondayThroughFridayTimeline());
	    this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
	    final StandardXYItemRenderer rendererRsi = new StandardXYItemRenderer();
	    rendererRsi.setSeriesPaint(0, Color.BLACK);
	    final Color lightgreen = new Color(170, 237, 169, 255);
	    final Color lightred = new Color(238, 170, 169, 255);
	    final Color yellow = new Color(254, 216, 61, 255);
	    final Color black = new Color(0, 0, 0, 0);
	    this.diff1 = new XYDifferenceRenderer(lightgreen, black, false);
	    this.diff1.setSeriesStroke(0, new BasicStroke(2.0f));
	    this.diff1.setSeriesStroke(1, new BasicStroke(1.0f));
	    this.diff1.setSeriesPaint(0, yellow);
	    this.diff1.setSeriesPaint(1, Color.lightGray);
	    this.diff2 = new XYDifferenceRenderer(black, lightred, false);
	    this.diff2.setSeriesStroke(0, new BasicStroke(2.0f));
	    this.diff2.setSeriesStroke(1, new BasicStroke(0.0f));
	    this.diff2.setSeriesPaint(0, yellow);
	    this.diff2.setSeriesPaint(1, Color.lightGray);
	    ItemUtils.addLegendTitle(this.anaG);
	}
	if (this.serieRsi == null) {
	    this.calculate();
	}
	final Day fromDay = new Day(DateUtils.asDate(fromDate));
	final Day toDay = new Day(DateUtils.asDate(toDate));
	try {
	    final TimeSeriesCollection col1 = new TimeSeriesCollection();
	    col1.addSeries(this.serieRsi.createCopy(fromDay, toDay));
	    col1.addSeries(this.serie70.createCopy(fromDay, toDay));
	    final TimeSeriesCollection col2 = new TimeSeriesCollection();
	    col2.addSeries(this.serieRsi.createCopy(fromDay, toDay));
	    col2.addSeries(this.serie30.createCopy(fromDay, toDay));
	    this.anaG.setDataset(0, col1);
	    this.anaG.setRenderer(0, this.diff1);
	    this.anaG.setDataset(1, col2);
	    this.anaG.setRenderer(1, this.diff2);
	    this.diff1.setSeriesVisibleInLegend(1, false);
	    this.diff2.setSeriesVisibleInLegend(0, false);
	    this.diff2.setSeriesVisibleInLegend(1, false);
	} catch (final CloneNotSupportedException e) {
	    Logger.error(I18N.tr("RelativeStrengthIndex.createTsDataError"), e); //$NON-NLS-1$
	}
	return this.anaG;
    }

    /**
     * Calculate series data
     */
    private void calculate() {

	final int period = this.parameter.getValue().intValue();
	this.serieRsi = new TimeSeries(this.getLegendTitel());
	this.serie30 = new TimeSeries("30"); //$NON-NLS-1$
	this.serie70 = new TimeSeries("70"); //$NON-NLS-1$
	for (int t = this.closeDaten.size() - 1; t > period; t--) {
	    double sumUp1 = 0.0d;
	    double sumDown1 = 0.0d;
	    // First calculate teh sum
	    for (int i = 1; i <= period; i++) {
		sumUp1 += Math.max(this.closeDaten.get(t - i + 1).getValue() - this.closeDaten.get(t - i).getValue(),
			0.0d);
		sumDown1 -= Math.min(this.closeDaten.get(t - i + 1).getValue() - this.closeDaten.get(t - i).getValue(),
			0.0d);
	    }
	    // Now build the averages
	    final double avgUp = sumUp1 / period;
	    final double avgDown = sumDown1 / period;
	    // Calculate the dividend
	    final double dividend = avgUp + avgDown;
	    double rsi;
	    // check for zero
	    if (dividend == 0.0d) {
		rsi = 0.0d;
	    } else {
		// calculate RSI and multiply with 100
		rsi = 100 * avgUp / dividend;
	    }
	    if (rsi >= RelativeStrengthIndex.RSI_ABOVE_100) {
		Logger.info("RSI: " + rsi); //$NON-NLS-1$
		rsi = 100.0d;
	    }

	    final Day day = new Day(DateUtils.asDate(this.closeDaten.get(t).getDate())); // NOPMD
	    Logger.trace("RSI (" + day.toString() + "): " + rsi); //$NON-NLS-1$ //$NON-NLS-2$
	    this.serieRsi.add(day, rsi);
	    this.serie30.add(day, 30);
	    this.serie70.add(day, 70);
	}
    }
}
