/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.sma;

import java.time.LocalDate;
import java.util.LinkedList;
import java.util.Queue;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * SimpleMovingAverage Methode
 *
 * @author Frank Mnster
 */
public class SimpleMovingAverage extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private static final String NAME = "SMA"; //$NON-NLS-1$

    /**
     * Queue Objekt zur einfachen Aufbewahrung der letzten n Werte.
     */
    private final Queue<Double> temp = new LinkedList<>();

    /**
     * Gloables Summen Objekt zur Berechnung der Durchschnittswerte.
     */
    private double sum;

    /**
     * Liefert die Add Aktion
     *
     * @return die SimpleMovingAverageHandleAddAction
     */
    public static Action getHandleAction() {

        return new SimpleMovingAverageHandleAddAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

        return SimpleMovingAverage.NAME;
    }

    /**
     * Standardkonstruktor
     */
    public SimpleMovingAverage() {
        super();
    }

    /**
     * @param wpHistoryBean
     * @param parameter
     */
    public SimpleMovingAverage(final WertpapierHistoryBean wpHistoryBean,
            final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {

        return "SMA " + this.parameter.getValue(); //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public KursDaten[] getResult(final DateRange period) {

        final LocalDate fromDate = period.getStartDate();
        final LocalDate toDate = period.getEndDate();

        final KursDaten[] result = new KursDaten[1];
        final KursDaten samSeries = new KursDaten();
        for (int i = 0; i < this.closeDaten.size(); i++) {
            final DateValuePair orgValue = this.closeDaten.get(i);
            this.add(orgValue.getValue());
            final double gdAverage = this.getAverage();
            if (DateUtils.isInRange(orgValue.getDate(), fromDate, toDate) && !Double.isNaN(gdAverage)) {
                final DateValuePair newValue = new DateValuePair( // NOPMD
                        orgValue.getDate(), gdAverage);
                samSeries.add(newValue);
            }
        }
        result[0] = samSeries;
        return result;
    }

    /**
     * Aktualisiert die Summe mit angegeben Wert. Der &uml;lteste Wert aus der
     * Queue wird abgezogen, sodass immer die Summe immer nur &uuml;ber die
     * letzten N-Werte gebildet wird.
     *
     * @param value
     */
    private void add(final double value) {

        this.sum += value;
        this.temp.add(value);
        if (this.temp.size() > this.parameter.getValue()) {
            this.sum -= this.temp.remove();
        }
    }

    /**
     * Berechnet den Durchschnitt der letzten n-Werte.
     *
     * @return den Durchschnitt der letzten n-Werte, falls in Queue schon mehr
     *         als n-1 Werte eingetragen wurden.
     */
    private double getAverage() {

        Double result = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            result = this.sum / this.temp.size();
        }
        return result;
    }
}
