/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.volumen;

import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.time.LocalDate;
import java.util.List;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYBarPainter;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.ItemUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.willuhn.jameica.gui.Action;

/**
 * Volumen-Analysemethode.
 *
 * @author Frank Mnster
 */
public class Volumen extends AbstractKursAnalyse<Integer, XYPlot> {

    /**
     * Der Name der Methode
     */
    private static final String NAME = I18N.tr("Volumen.methodName"); //$NON-NLS-1$

    /**
     * Eine Million
     */
    private static final long MILLION = 1000000;

    /**
     * Ein Tausend
     */
    private static final long TOUSEND = 1000;

    /**
     * Die TimeSerie der
     */
    private TimeSeries serieVol;

    /**
     * der Renderer
     */
    private XYBarRenderer rendererVol;

    /**
     * Liefert den Action-Handler
     *
     * @return den VolumenHandleAction
     */
    public static Action getHandleAction() {

        return new VolumenHandleAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return den Methodennamen
     */
    public static String getMethodName() {

        return Volumen.NAME;
    }

    /**
     * Constructs a new Volume chart
     */
    public Volumen() {
        super();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {

        return I18N.tr("Volumen.methodeTitel"); //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public XYPlot getResult(final DateRange period) {

        final LocalDate fromDate = period.getStartDate();
        final LocalDate toDate = period.getEndDate();

        if (this.anaG == null) {
            final DateAxis dateAxisKurs = new DateAxis(I18N.tr("Volumen.axisDate")); //$NON-NLS-1$
            final NumberAxis rangeAxisKurs = new NumberAxis(I18N.tr("Volumen.numberAxis")); //$NON-NLS-1$
            rangeAxisKurs.setAutoRangeIncludesZero(false);
            rangeAxisKurs.setNumberFormatOverride(new DecimalFormat() {

                /**
                 * Serial UID
                 */
                private static final long serialVersionUID = -7160700156383880184L;

                /**
                 * @see DecimalFormat#format(double arg0, StringBuffer arg1,
                 *      FieldPosition arg2)
                 */
                @Override
                public StringBuffer format(final double number, final StringBuffer toAppendTo,
                        final FieldPosition pos) {

                    if (number >= Volumen.MILLION) {
                        toAppendTo.append((int) (number / Volumen.MILLION));
                        toAppendTo.append('m');
                    } else if (number >= Volumen.TOUSEND) {
                        toAppendTo.append((int) (number / Volumen.TOUSEND));
                        toAppendTo.append('k');
                    } else {
                        toAppendTo.append((int) number);
                    }
                    return toAppendTo;
                }
            });
            this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
            this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
            this.rendererVol = new XYBarRenderer();
            this.rendererVol.setSeriesVisibleInLegend(0, false);
            this.rendererVol.setShadowVisible(false);
            this.rendererVol.setMargin(0.85);
            this.rendererVol.setBarPainter(new StandardXYBarPainter());
            ItemUtils.addLegendTitle(this.anaG);
        }
        if (this.serieVol == null) {
            this.calculate();
        }
        final Day fromDay = new Day(DateUtils.asDate(fromDate));
        final Day toDay = new Day(DateUtils.asDate(toDate));
        try {
            final TimeSeriesCollection tmpMom = new TimeSeriesCollection(this.serieVol.createCopy(fromDay, toDay));
            this.anaG.setDataset(0, tmpMom);
            this.anaG.setRenderer(0, this.rendererVol);
        } catch (final CloneNotSupportedException e) {
            de.willuhn.logging.Logger.error(I18N.tr("Volumen.logCreateTsDataError"), e); //$NON-NLS-1$
        }
        return this.anaG;
    }

    /**
     * Calculate Volumen
     */
    private void calculate() {

        this.serieVol = new TimeSeries(this.getLegendTitel());
        final List<HistoryValueBean> wphv = this.wertPapierHistory.getHistoryValues();
        // Loop thru all Values
        for (final HistoryValueBean historyValue : wphv) {
            final Day day = new Day(DateUtils.asDate(historyValue.getLocalDate())); // NOPMD
            this.serieVol.add(day, historyValue.getVolume());
        }
    }
}
