
package de.frankmuenster.jameica.depot.utils;

import java.io.File;
import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.importer.CsvOrderBean;
import de.frankmuenster.jameica.depot.importer.CsvOrderBeanRegistry;
import de.frankmuenster.jameica.depot.importer.RegisterCsvOrderBean;
import de.willuhn.logging.Logger;

/**
 * Class that loops over all classes to process any project related annotations
 *
 * @author Frank Mnster
 *
 */
public final class AnnotationProcessor {

    /**
     * Scans all classes accessible from the context class loader which belong
     * to the given package and sub packages.
     *
     * @param packageName
     *            The base package
     * @return The classes
     * @throws IOException
     */
    public static Class<?>[] getClasses(final String packageName) throws IOException {

        final ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        assert classLoader != null;

        // New array list to hold found classes
        final ArrayList<Class<?>> classes = new ArrayList<>();

        final String path = packageName.replace('.', '/');
        final Enumeration<URL> resources = classLoader.getResources(path);
        // Loop of found resources
        while (resources.hasMoreElements()) {
            final URL url = resources.nextElement();
            final File file = new File(url.getPath()); // NOPMD
            if (file.isDirectory()) {
                // if directory search files inside
                classes.addAll(AnnotationProcessor.findClasses(file, packageName, classLoader));
            } else {
                // if a jar, lookup inside the jar
                classes.addAll(AnnotationProcessor.findClassesInJar(url, classLoader));
            }
        }
        // return the list a array
        return classes.toArray(new Class[0]);
    }

    /**
     * Loops over all classes in the JaDepot package and process the
     * annotations: <br />
     * <ul>
     * <li>{@link RegisterCsvOrderBean}</li>
     * </ul>
     */
    public static void processAnnotation() {

        try {

            final Class<?>[] classes = AnnotationProcessor.getClasses(JaDepot.class.getPackage().getName());
            for (final Class<?> clazz : classes) {

                final RegisterCsvOrderBean anno = clazz.getAnnotation(RegisterCsvOrderBean.class);
                if (anno != null) {
                    Logger.debug("CsvOrderBean-Name: " + anno.name()); //$NON-NLS-1$
                    AnnotationProcessor.handleRegisterCsvOrderBean(clazz, anno.name());

                }
            }

        } catch (final IOException e) {
            Logger.error(I18N.tr("AnnotationProcessor.errorCreatingClassList"), e); //$NON-NLS-1$
        }
    }

    /**
     * Recursive method used to find all classes in a given directory and sub
     * dirs.
     *
     * @param directory
     *            The base directory
     * @param packageName
     *            The package name for classes found inside the base directory
     * @param classLoader
     * @return The classes
     */
    private static List<Class<?>> findClasses(final File directory, final String packageName,
            final ClassLoader classLoader) {

        final List<Class<?>> classes = new ArrayList<>();
        if (directory.exists()) {
            final File[] files = directory.listFiles();
            for (final File file : files) {
                if (file.isDirectory()) {
                    assert !file.getName().contains("."); //$NON-NLS-1$
                    classes.addAll(
                            AnnotationProcessor.findClasses(file, packageName + "." + file.getName(), classLoader)); //$NON-NLS-1$
                } else if (file.getName().endsWith(".class")) { //$NON-NLS-1$
                    final String myClass = packageName + '.' + file.getName().substring(0, file.getName().length() - 6);
                    try {
                        classes.add(Class.forName(myClass, false, classLoader));
                    } catch (NoClassDefFoundError | ClassNotFoundException | ExceptionInInitializerError e) {
                        // simply skip the class in case it was not found or
                        // could not be initialised
                        Logger.trace(myClass + " not found"); //$NON-NLS-1$
                    }
                }
            }
        }
        return classes;
    }

    /**
     * Searches for classes in the jar file (only based on files)
     *
     * @param url
     *            to the jar file
     * @param classLoader
     * @return all classes found in the jar file
     * @throws IOException
     */
    private static List<? extends Class<?>> findClassesInJar(final URL url, final ClassLoader classLoader)
            throws IOException {

        final List<Class<?>> classes = new ArrayList<>();

        final JarURLConnection jarConnection = (JarURLConnection) url.openConnection();

        final Enumeration<JarEntry> entries = jarConnection.getJarFile().entries();
        JarEntry entry;
        while (entries.hasMoreElements()) {
            entry = entries.nextElement();
            if (entry.getName().endsWith(".class")) { //$NON-NLS-1$
                Logger.info(entry.getName());
                final String className = entry.getName().replaceAll("/", "\\."); //$NON-NLS-1$ //$NON-NLS-2$
                final String myClass = className.substring(0, className.lastIndexOf('.'));
                try {
                    classes.add(Class.forName(myClass, false, classLoader));
                } catch (NoClassDefFoundError | ClassNotFoundException e) {
                    // simply skip the class in case it was not found or could
                    // not be initialised
                    Logger.trace(myClass + " not found"); //$NON-NLS-1$
                }
            }
        }

        return classes;
    }

    /**
     * Registers the class in the CsvOrderBean register
     *
     * @param clazz
     *            the class to register
     * @param anno
     *            the name the name of the bean
     */
    private static void handleRegisterCsvOrderBean(final Class<?> clazz, final String name) {

        CsvOrderBean csvOrderBean;
        try {
            csvOrderBean = (CsvOrderBean) clazz.newInstance();
            CsvOrderBeanRegistry.register(csvOrderBean, name);

        } catch (final InstantiationException | IllegalAccessException e) {
            Logger.error(I18N.tr("AnnotationProcessor.errorRegisterOrderBean"), e); //$NON-NLS-1$
        }

    }

    private AnnotationProcessor() {
    }

}
