/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.utils;

import java.io.IOException;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;

import de.willuhn.logging.Logger;

/**
 * Die &Uuml;bersetzer Klasse. Eigenst&auml;endig u.a. wegen JUNIT Tests
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.ShortClassName")
public final class I18N {

    /**
     * Resource Pfad zu den Sprachdateien
     */
    private static final String BUNDLE_NAME = "lang.jadepot"; //$NON-NLS-1$

    /**
     * Resource Pfad zu den Sprachdateien
     */
    /**
     * Das Resource Bundle mit der Sprachdatei
     */
    private static ResourceBundle bundle;

    /**
     * Das Resource Bundle mit der Sprachdatei
     */
    private static ResourceBundle bundleFallBack;

    /**
     * Speichert alle nicht bersetzten Texte
     */
    private static Properties unresolved = new Properties();

    /**
     * Die Locale
     */
    private static Locale locale = Locale.getDefault();
    static {
        final Locale localeDe = new Locale("de", "DE"); //$NON-NLS-1$ //$NON-NLS-2$
        try {
            I18N.bundle = ResourceBundle.getBundle(I18N.BUNDLE_NAME, I18N.locale);
            I18N.bundleFallBack = ResourceBundle.getBundle(I18N.BUNDLE_NAME, localeDe);
        } catch (final MissingResourceException e) {
            // Wir laden die deutsche Sprachdatei
            I18N.locale = localeDe;
            I18N.bundle = ResourceBundle.getBundle(I18N.BUNDLE_NAME, I18N.locale);
            I18N.bundleFallBack = I18N.bundle;
            // Die Zeile ist nur fr den Eclipse Externalize String wizard
            // Das dort angegebenen 'Resource File' wird benutzt
            // @SuppressWarnings("unused")
            ResourceBundle.getBundle("lang.jadepot_de_DE", I18N.locale); //$NON-NLS-1$
        }
    }

    /**
     * Schreibt alle bis dato nicht uebersetzbaren Strings in den angegebenen
     * OutputStream.
     *
     * @param oStream
     *            Stream, in den geschrieben werden soll.
     * @throws IOException
     */
    public static void storeUntranslated(final OutputStream oStream) throws IOException {

        Logger.info(I18N.tr("I18N.storeMissingTranslations")); //$NON-NLS-1$
        I18N.unresolved.store(oStream, "unresolved strings for locale " + I18N.locale.toString()); //$NON-NLS-1$
    }

    /**
     * Liefert die bersetzung in der aktuellen Local
     *
     * @param key
     *            das Schlsselwort
     * @return die bersetzung in der aktuellen Local
     */
    public static String tr(final String key) { // NOPMD

        String result;
        try {
            result = I18N.bundle.getString(key);
        } catch (final MissingResourceException e) {
            try {
                // Versuche mit deutscher Sprachdatei
                result = I18N.bundleFallBack.getString(key);
            } catch (final MissingResourceException e1) {
                result = key;
            }
            I18N.unresolved.put(key, result);
        }
        return result;
    }

    /**
     * Liefert die bersetzung in der aktuellen Local. Bietet zustzlich noch
     * Variablen im Text an ({0}, etc.), die nach der bersetzung eingefgt
     * werden.
     *
     * @param key
     *            das Schlsselwort
     * @param replacements
     *            die Ersetzungen
     * @return die bersetzung mit den Ersetzungen in der aktuellen Local
     */
    public static String tr(final String key, final String... replacements) { // NOPMD

        return MessageFormat.format(I18N.tr(key), (Object[]) replacements);
    }

    private I18N() {
        // Absichtlich leer
    }
}
